##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################



from ALString                  import ALString
from ALVerbose                 import ALVerbose
from ALSystem                  import ALSystem
from ALFile                    import ALFile
from ALCommandLine             import ALCommandLine
from ALXsd                     import ALXsd
from ALDiskExplorer            import ALDiskExplorer
from ALDictionary              import ALDictionary
from ALDebug                   import ALDebug


try:
    import ALCommandLineInterfaceDefinition  as ALCommandLineInterfaceDefinition
    import ALCommandLineInterfaceElement     as ALCommandLineInterfaceElement
except ImportError:
    try:
        # You must run this module on the m/modules folder to generate the specific code
        ALXsd().generateModule( "ALCommandLineInterfaceDefinition.py", "ALCommandLineInterfaceDefinition.xsd", "xsd:" )
        ALXsd().generateModule( "ALCommandLineInterfaceElement.py", "ALCommandLineInterfaceElement.xsd", "xsd:" )
    except:
        ALVerbose.warning( "Problems to generate class interfaces for: ALCommandLineInterfaceElement and ALCommandLineInterfaceDefinition" )


class ALCommandLineInterface( ALCommandLine ):
    """
    Base class for command line interface objects.
    """
    def __init__( self, _oStrXmlFileName = None, _oBoolFactoryPluginDynamic = True ):
        ALCommandLine.__init__( self )
        self.m_oalDictCmdInterface       = ALDictionary()
        self.m_oStrXmlFileName           = _oStrXmlFileName
        self.m_oBoolFactoryPluginDynamic = _oBoolFactoryPluginDynamic
        self.createCommandLine( self.m_oStrXmlFileName )


    def createCommandLine( self, _oStrXmlFileName ):
        if (_oStrXmlFileName==None or _oStrXmlFileName==ALString.empty()) :
            ALVerbose.DEBUG( "No CommandLineInterface generated :: [ Empty file name ]" )
            return
        if (ALDiskExplorer.existFile( _oStrXmlFileName )==False):
            ALVerbose.DEBUG( "[NO] CommandLineInterface Generated From File :: [" + ALString( _oStrXmlFileName ) + "]" )
            ALVerbose.DEBUG( "                       The file doesn't exist :: [" + ALString( _oStrXmlFileName ) + "]" )
            return

        ALVerbose.DEBUG( "[OK] CommandLineInterface Generated From File :: [" + ALString( _oStrXmlFileName ) + "]" )

        oalCmdLineDefinition = ALCommandLineInterfaceDefinition.unMarshalling( _oStrXmlFileName )
        oalList = oalCmdLineDefinition.getCommandLineElement()

        for iElem in oalList:
            self.m_oalDictCmdInterface.addObject( iElem.getCommand(), iElem )

        self.__createCommandLineCallBack()



    def __createCommandLineCallBack( self ):
        for iElem in self.m_oalDictCmdInterface.ordered():
            oalCmdElement = self.m_oalDictCmdInterface[iElem]
            oalCmdClass = oalCmdElement.getClass()
            if (oalCmdClass!=ALString.empty()):
                self.__importClassObjectCallBack( oalCmdElement )
            else:
                self.__importPluginObjectCallBack( oalCmdElement )



    def __importClassObjectCallBack( self, _oalCmdElement ):
        if (self.m_oBoolFactoryPluginDynamic==False):
            return
        oalCmdName          = _oalCmdElement.getCommand()
        oalCmdClass         = _oalCmdElement.getClass()
        oalCmdClassMethod   = _oalCmdElement.getClassMethod()
        oalCmdClassFileName = ALDiskExplorer.normPath( _oalCmdElement.getClassFileName() )

        self.__importClassObject( oalCmdName, oalCmdClass, oalCmdClassMethod, oalCmdClassFileName )



    def __importClassObject( self, _oalCmdName, _oalCmdClass, _oalCmdClassMethod, _oalCmdClassFileName ):
        if (self.m_oBoolFactoryPluginDynamic==False):
            return
        oalCmdName          = _oalCmdName
        oalCmdClass         = _oalCmdClass
        oalCmdClassMethod   = _oalCmdClassMethod
        oalCmdClassFileName = _oalCmdClassFileName

        if (oalCmdClassFileName==ALString.empty()):
             oalCmdClassFileName = oalCmdClass + ".py"

        if (ALDiskExplorer.existFile( oalCmdClassFileName )==False):
            ALVerbose.DEBUG( "        The Defined Plugin File doesn't exist :: [" + oalCmdClassFileName + "]" )
            return

        if ALFile.isExtension( oalCmdClassFileName.upper(), ".PY" ):
            oalPath =  ALFile.getFolderName( oalCmdClassFileName )
            if (oalPath!=ALString.empty()):
                ALSystem.setPath( oalPath )
            oNameModule = ALFile.getBaseName( oalCmdClassFileName )
            try:
                oModule = __import__( oNameModule )
                oBoolStatus = False
                for iKey in oModule.__dict__:
                    if (iKey==oalCmdClass):
                        oBoolStatus = True
                if (oBoolStatus==True):
                    try:
                        oClass  = oModule.__dict__[ oalCmdClass ]()
                        oMethod = getattr( oClass, oalCmdClassMethod )
                        self.addCommand( oalCmdName, oMethod )
                    except:
                        self.__raiseControl()
                else:
                    ALVerbose.DEBUG( "The defined class was not found :: [" + oNameModule + "]" )
            except:
                ALVerbose.DEBUG( "Problem Compiling and Loading Class :: [" + ifile + "]" )
                ALVerbose.DEBUG( "Please DEBUG the plugin code to fix the problem." )
                self.__raiseControl()

        else:
             ALVerbose.DEBUG( "Not valid Python file: " + oalCmdClassFileName )



    def __importPluginObjectCallBack( self, _oalCmdElement ):
        oalCmdName           = _oalCmdElement.getCommand()
        oalCmdPlugin         = _oalCmdElement.getPlugin()
        oalCmdPluginMethod   = _oalCmdElement.getPluginMethod()
        oalCmdPluginFileName = _oalCmdElement.getPluginFileName()
        if (oalCmdPluginFileName!=ALString.empty()):
            oalCmdPluginFileName = ALDiskExplorer.normPath( _oalCmdElement.getPluginFileName() )

        if (oalCmdPluginMethod==ALString.empty()):
            oalCmdPluginMethod = ALString( "executePlugin" )


        #Warning: this is a workaround to bypass the limitation
        #of the ALApplication design.!!!
        #It will be obsolete in the future with the atomisation of
        # that class and a new architecture
        from ALApplication             import ALApplication
        # to be obsolete in the future.

        oalPlugin = None
        oalPlugin = ALApplication.getPluginObject( oalCmdPlugin )

        if (oalPlugin == None):
            self.__importClassObject( oalCmdName, oalCmdPlugin, oalCmdPluginMethod, oalCmdPluginFileName )
            return


        if (oalPlugin!=None):
            if (oalCmdPluginMethod!=ALString.empty()):
                oMethod = getattr( oalPlugin, oalCmdPluginMethod )
            else:
                oMethod = oalPlugin.executePlugin
            self.addCommand( oalCmdName, oMethod )
        else:
             ALVerbose.DEBUG( "Problem Loading plugin :: [ " + ALString( oalCmdPlugin ) + "]" )


    def getHelp( self, oalStrCommand = None ):
        oalString = ALString( "" )
        if ( oalStrCommand == None):
           for oalStrCommand in self.m_oalDictCmdInterface.getSequenceOrdered():
                oalStringHelp = self.m_oalDictCmdInterface[ oalStrCommand ].getHelp()
                if (oalStringHelp!=ALString.empty()):
                    oalString = oalString + oalStringHelp
        elif ( oalStrCommand == "--all" ):
           for oalStrCommand in self.m_oalDictCmdInterface.getSequenceOrdered():
                oalStrCmdDescription = self.m_oalDictCmdInterface[ oalStrCommand ].getCommandDescription()
                oalStrCmd = self.m_oalDictCmdInterface[ oalStrCommand ].getCommand()
                if (oalStrCmdDescription==ALString.empty()):
                    oalStrCmdDescription = "No Command Description Avaiable"
                if (oalStrCmd!=ALString.empty()):
                    oalString = oalString + ALString( "\n          " + oalStrCmd + " : " +  oalStrCmdDescription )
        else:
           if (oalStrCommand in self.m_oalDictCmdInterface):
                oalStringHelp = self.m_oalDictCmdInterface[ oalStrCommand ].getHelp()
                if (oalStringHelp!=ALString.empty()):
                    oalString = oalStringHelp
        return oalString


    def getManual( self, oalStrCommand = None ):
        oalString = ALString( "" )
        if ( oalStrCommand == None):
            for oalStrCommand in self.m_oalDictCmdInterface.getSequenceOrdered():
                oalStringManual = self.m_oalDictCmdInterface[ oalStrCommand ].getManual()
                if (oalStringManual!=ALString.empty()):
                    oalString = oalString + oalStringManual
        elif ( oalStrCommand == "--all" ):
           for oalStrCommand in self.m_oalDictCmdInterface.getSequenceOrdered():
                oalStrCmdDescription = self.m_oalDictCmdInterface[ oalStrCommand ].getCommandDescription()
                oalStrCmd = self.m_oalDictCmdInterface[ oalStrCommand ].getCommand()
                if (oalStrCmdDescription==ALString.empty()):
                    oalStrCmdDescription = "No Command Description Avaiable"
                if (oalStrCmd!=ALString.empty()):
                    oalString = oalString + ALString( "\n          " + oalStrCmd + " : " +  oalStrCmdDescription )
        else:
            if (oalStrCommand in self.m_oalDictCmdInterface):
                oalStringManual = self.m_oalDictCmdInterface[ oalStrCommand ].getManual()
                if (oalStringManual!=ALString.empty()):
                    oalString = oalStringManual
        return oalString


    def getWebAdress( self, oalStrCommand ):
        oalString = None
        if oalStrCommand in self.m_oalDictCmdInterface:
            oalStringData = self.m_oalDictCmdInterface[ oalStrCommand ].getWebAdress()
            if (oalStringData!=ALString.empty()):
                oalString = oalStringData
        return oalString


    def getWebUpDate( self, oalStrCommand ):
        oalString = None
        if oalStrCommand in self.m_oalDictCmdInterface:
            oalStringData = self.m_oalDictCmdInterface[ oalStrCommand ].getWebUpDate()
            if (oalStringData!=ALString.empty()):
                oalString = oalStringData
        return oalString


    def getCopyright( self, oalStrCommand ):
        oalString = None
        if oalStrCommand in self.m_oalDictCmdInterface:
            oalStringData = self.m_oalDictCmdInterface[ oalStrCommand ].getCopyright()
            if (oalStringData!=ALString.empty()):
                oalString = oalStringData
        return oalString


    def getAuthor( self, oalStrCommand ):
        oalString = None
        if oalStrCommand in self.m_oalDictCmdInterface:
            oalStringData = self.m_oalDictCmdInterface[ oalStrCommand ].getAuthor()
            if (oalStringData!=ALString.empty()):
                oalString = oalStringData
        return oalString


    def getDate( self, oalStrCommand ):
        oalString = None
        if oalStrCommand in self.m_oalDictCmdInterface:
            oalStringData = self.m_oalDictCmdInterface[ oalStrCommand ].getDate()
            if (oalStringData!=ALString.empty()):
                oalString = oalStringData
        return oalString


##############################################################################
