##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDefinition                  import *
from ALString                      import ALString
from ALVerbose                     import ALVerbose
from ALSlot                        import ALSlot
from ALSignal                      import ALSignal
from ALThread                      import ALThread
from ALDebug                       import ALDebug
from ALTrace                       import ALTrace
from ALTime                        import ALTime
from ALExceptionSignal             import ALExceptionSignal
from ALApplicationCommunicator     import ALApplicationCommunicator


class ALThreadControl( ALThread ):
    """
    Interface for AL thread control objects.
    """
    def __init__( self ):
        ALThread.__init__( self )
        self.m_oalSlotInit      = ALSlot()
        self.m_oalSlotRun       = ALSlot()
        self.m_oalSlotEnd       = ALSlot()
        self.m_oalSlotPause     = ALSlot()
        self.m_oalSlotResume    = ALSlot()
        self.m_oalSlotAbort     = ALSlot()
        self.m_oalSlotException = ALSlot()
        self.m_oalSlotError     = ALSlot()


        # Basic Event Callback Connections
        self.m_oalSlotEnd.connect( self.endOfAction )
        self.m_oalSlotAbort.connect( self.verboseAbort, ALSignal.abort() )
        self.m_oalSlotException.connect( self.verboseException, ALSignal.exception() )

        self.m_bIsEnded = False
        self.m_bIsAbort = False
        self.m_bIsPause = False

        # To be controled by the ALApplication framework
        self.m_bRegisterAction = True

        self.m_oalRunTimeOut        = AL_TIME_SECOND * 6000.0  #(100 minutes)
        self.m_oalTimeDuration      = 0.0
        self.m_oalTime              = ALTime()



    def getTimeDuration(self):
        self.m_oalTimeDuration = self.m_oalTime.getDuration()
        return self.m_oalTimeDuration


    def getTimeRun(self):
        return self.m_oalTime


    def getRegisterAction( self ):
        return self.m_bRegisterAction


    def setRegisterAction( self ):
        self.synchronizeOn()
        self.m_bRegisterAction = True
        self.synchronizeOff()

    def setNoRegisterAction( self ):
        self.synchronizeOn()
        self.m_bRegisterAction = False
        self.synchronizeOff()


    def synchronizeOn( self, _oalObject = None ):
        # Option used to debug thread locks
        if (_oalObject!=None):
            ALVerbose.DEBUGID( self.getID(), "ALAction::synchronizeOn - " + ALString( _oalObject.getID() ) )
            ALVerbose.DEBUGID( self.getID(), "ALAction::synchronizeOn - Semaphore is ON" )
        self.m_opySemaphore.acquire()


    def synchronizeOff( self, _oalObject = None ):
        # Option used to debug thread locks
        if (_oalObject!=None):
            ALVerbose.DEBUGID( self.getID(), "ALAction::synchronizeOff - " + ALString( _oalObject.getID() ) )
            ALVerbose.DEBUGID( self.getID(), "ALAction::synchronizeOff - Semaphore is OFF" )
        self.m_opySemaphore.release()



    def connectInit( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotInit.connect( _oMethod )
        self.synchronizeOff()

    def connectPause( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotOause.connect( _oMethod )
        self.synchronizeOff()

    def connectResume( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotResume.connect( _oMethod )
        self.synchronizeOff()

    def connectAbort( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotAbort.connect( _oMethod )
        self.synchronizeOff()

    def connectException( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotException.connect( _oMethod )
        self.synchronizeOff()

    def connectError( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotError.connect( _oMethod )
        self.synchronizeOff()

    def connectEnd( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotEnd.connect( _oMethod )
        self.synchronizeOff()

    def connectExecute( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotRun.connect( _oMethod )
        self.synchronizeOff()

    def connectRun( self, _oMethod ):
        self.connectExecute( _oMethod )



    def threadControl( self ):
        # Patch for Java-Jython
        oTime    = float( self.getTimeDuration() )
        oTimeOut = float( self.getTimeOut() )
        # controls the timeOut method
        if ( oTime>oTimeOut ):
        #if (self.getTimeDuration()>self.getTimeOut()): -> It can not run on Jython
            ALVerbose.DEBUGID( self.getID(), ALString( "Thread::STOPPED under control in threadControl() - TimeOUT: " + self.getName() + " " + self.getID() )  )
            raise ALExceptionSignal( ALSignal.abort())
        if (self.isAbort()==True):
            ALVerbose.DEBUGID( self.getID(), ALString( "Thread::STOPPED under control: " + self.getName() + " " + self.getID() )  )
            raise ALExceptionSignal( ALSignal.abort() )
        if (self.isPause()==True):
            ALVerbose.DEBUGID( self.getID(), ALString( "Thread::PAUSED under control: " + self.getName() + " " + self.getID() )  )
            ALTime.wait( AL_TIME_MINIMUM * 10.0 )
        # Experimental
        oalApplicationSignal = ALApplicationCommunicator.getSignal()
        if (oalApplicationSignal is not None):
            oalSignal = oalApplicationSignal.getSignal()
            if (isinstance( oalSignal, ALSignal)):
                oalStringInformation = oalApplicationSignal.getInformation()
                if ( oalSignal.getSignal()==AL_ACTION_SIGNAL_ABORT or oalSignal.getSignal()==AL_ACTION_SIGNAL_ERROR ) :
                    ALVerbose.DEBUGID( self.getID(), ALString( oalSignal.getSignal() ) + "::" + oalStringInformation )
                    raise ALExceptionSignal( oalSignal )
            else:
                pass # to  verify the ALSignal type ????misterious string comming in a unittest = type check Problem?
                     #The problem to be revised is the type of signal, string or other thing
                #print oalSignal



    def verboseAbort( self, _oalObject = None ):
        self.m_bIsAbort = True
       # ALVerbose.DEBUGID( self.getID(), ALString( "Action::STOPPED under control: " + self.getName() + " " + self.getID() ) )



    def verboseException( self, _oalObject = None ):
        self.m_bIsAbort = True
       # ALApplicationCommunicator.setSignal( ALSignal( AL_ACTION_SIGNAL_ERROR, ALString( "Thread Exception" ) ) )




    def threadControlTimer( self, _oalTimeDelay ):
        """
        Only to be used in internal run kernels
        Otherwise it generates an exception to
        stop the flow
        """
        if (self.isAlive()!=True):
            ALTime.wait( AL_TIME_MINIMUM * 2.0 )
        oalTime = ALTime()
        obStatus = self.isAlive()
        while ( obStatus==True ):
            self.threadControl()
            ALTime.wait( AL_TIME_MINIMUM )
            # Patch for Java-Jython
            oTime    = float( oalTime.getDuration() )
            oTimeOut = float( _oalTimeDelay )
            if ( oTime>oTimeOut ):
                obStatus = False
            if (self.isAlive()!=True):
                obStatus = False
#            if (oalTime.getDuration()>_oalTimeDelay):
#                obStatus = False




    def synchronize( self ):
        # Delay to wait lazy threads
        if (self.isRunning()!=True):
            ALTime.wait( AL_TIME_MINIMUM * 5.0 )

        oalTime = ALTime()
        while( self.isRunning()==True ):
            ALTime.wait( AL_TIME_MINIMUM )
            # Patch for Java-Jython
            oTime    = float( oalTime.getDuration() )
            oTimeOut = float( self.getTimeOut() )
            if ( oTime>oTimeOut ):
                self.close()
#            if (oalTime.getDuration()>self.getTimeOut()):
#                self.close()




    def abort( self, _oalObject = None ):
        self.m_bIsAbort = True


    def stop( self, _oalObject = None ):
        self.abort()


    def close( self, _oalObject = None ):
        self.stop()


    def endOfAction( self, _oalObject = None ):
        self.m_bIsEnded = True


# To be deprecated
    def resume( self, _oalObject = None ):
        self.m_oalSlotResume.call()
        self.m_bIsPause = False


    def pause( self, _oalObject = None ):
        self.m_oalSlotPause.call()
        self.m_bIsPause = True


    def isAbort( self ):
        return self.m_bIsAbort


    def isPause( self ):
        return self.m_bIsPause


    def isRunning( self ):
        return self.isAlive()


    def isEnded( self ):
        return self.m_bIsEnded


    def isExecute( self ):
        return self.isRunning()



    def setTimeOut( self, _ofTime ):
        self.m_oalRunTimeOut = float( _ofTime )


    def getTimeOut( self ):
        return self.m_oalRunTimeOut


##############################################################################

