##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALDefinition    import *
from ALTime          import ALTime
from ALAction        import ALAction
from ALSemaphore     import ALSemaphore


class ALActionController( ALAction ):
     """
     Parent class for all ALActionController objects.

     By default: a cluster of 64 parallel task are executed at the same time
     """
     def __init__( self, _oiClusterSize = 64, _oBoolPriority = False ):
         """
         Constructor for ALActionController.

         """
         ALAction.__init__( self, _oBoolPriority )
         self.m_oSemaphoreController  = ALSemaphore()
         self.m_oiClusterSize         = _oiClusterSize
         self.m_oiActiveActions       = 0
         self.m_oiSuccessActions      = 0
         self.m_oiFailureActions      = 0
         self.m_oiTotalSuccessActions = 0
         self.m_oiTotalFailureActions = 0



     def synchronizeControllerOn( self ):
         self.m_oSemaphoreController.acquire()


     def synchronizeControllerOff( self ):
         self.m_oSemaphoreController.release()



     def getClusterSize( self ):
         self.synchronizeControllerOn()
         oiValue = self.m_oiClusterSize
         self.synchronizeControllerOff()
         return int( oiValue )


     def setClusterSize( self, _oiClusterSize ):
         self.synchronizeControllerOn()
         self.m_oiClusterSize = int(_oiClusterSize )
         self.synchronizeControllerOff()


     def setNumberActiveActions( self, _oiNumberActions ):
         self.synchronizeControllerOn()
         self.m_oiActiveActions = int( _oiNumberActions )
         self.synchronizeControllerOff()


     def getNumberActiveActions( self ):
         self.synchronizeControllerOn()
         oiValue = self.m_oiActiveActions
         self.synchronizeControllerOff()
         return int( oiValue )


     def controlActiveActions( self ):
         self.synchronizeControllerOn()
         self.m_oiActiveActions += 1
         self.synchronizeControllerOff()


     def isClusterFull( self ):
         obStatus = False
         if ( int(self.getNumberActiveActions())>=int(self.getClusterSize()) ):
             obStatus = True
         return obStatus



     def __resetActiveActions(self):
         self.synchronizeControllerOn()
         self.m_oiTotalSuccessActions += self.m_oiSuccessActions
         self.m_oiTotalFailureActions += self.m_oiFailureActions
         self.m_oiActiveActions       = 0
         self.m_oiSuccessActions      = 0
         self.m_oiFailureActions      = 0
         self.synchronizeControllerOff()



     def getTotalSuccessActions(self):
         self.synchronizeControllerOn()
         oiValue = self.m_oiTotalSuccessActions
         self.synchronizeControllerOff()
         return int(oiValue)


     def getTotalFailureActions(self):
         self.synchronizeControllerOn()
         oiValue = self.m_oiTotalFailureActions
         self.synchronizeControllerOff()
         return int(oiValue)


     def getTotalActions(self):
         self.synchronizeControllerOn()
         oiValue = self.m_oiTotalSuccessActions + self.m_oiTotalFailureActions
         self.synchronizeControllerOff()
         return int(oiValue)



     def controlActionSuccessFailure( self ):
         self.synchronizeControllerOn()
         oiValue = self.m_oiSuccessActions + self.m_oiFailureActions
         self.synchronizeControllerOff()
         return int(oiValue)


     def controlActiondFinished(self):
         return self.controlActionSuccessFailure()


     def isActiveActions( self ):
         obStatus = True
         if (int( self.controlActiondFinished())==int(self.getActiveActions()) ):
             obStatus = False
             self.__resetActiveActions()
         return obStatus



     def getActiveActions( self ):
         self.synchronizeControllerOn()
         oiValue = self.m_oiActiveActions
         self.synchronizeControllerOff()
         return oiValue



     def controlActionSuccess( self, _ovdObject = None ):
         self.synchronizeControllerOn()
         self.m_oiSuccessActions += 1
         self.synchronizeControllerOff()


     def controlActionFailure( self, _ovdObject = None ):
         self.synchronizeControllerOn()
         self.m_oiFailureActions += 1
         self.synchronizeControllerOff()



     def synchronizeCluster( self ):
         self.controlActiveActions()
         if( self.isClusterFull()==True ):
             self.__synchronizeActions()


     def synchronizeController( self ):
         self.__synchronizeActions( )


     def __synchronizeActions( self ):
         while( self.isActiveActions()==True ):
             ALTime.wait( AL_TIME_MINIMUM )
         self.__resetActiveActions()




##############################################################################

