##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2006/03/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALActionController   import ALActionController
from ALActionMethod       import ALActionMethod
from ALDefinition         import *
from ALList               import ALList
from ALManagerProcess     import ALManagerProcess
from ALString             import ALString
from ALSubProcess         import ALSubProcess
from ALSystem             import ALSystem
from ALTime               import ALTime
from ALToken              import ALToken
from ALVerbose            import ALVerbose


class ALActionProcess( ALActionController ):
    """
    Interface for ALActionProcess classes.

    The method to be modified by the user (overload):
        def preProcess(  self, _oalObject = None )
        def postProcess( self, _oalObject = None )

    """
    def __init__( self, _oCommand, _oalStringFolderName = "./" ):
        ALActionController.__init__( self )
        self.m_oalSubProcess     = None
        self.m_oalActionMethod     = None
        self.m_oPid              = None
        self.m_oListRead         = None
        self.m_oListWrite        = None
        self.m_oListError        = None
        self.m_oCommand          = ALString( _oCommand )
        self.m_obIsStarted       = False
        self.m_oalManagerProcess = ALManagerProcess()
        self.setWorkingFolder( _oalStringFolderName )
        self.m_oalListDataProcess = None

        self.m_oExecutable = None
        self.__getExecutableName()



    def __getExecutableName(self):
        if(ALSystem.getOSName()=="Windows"):
            oalList = self.m_oCommand.split()
            if (oalList[0]!=""):
                self.m_oExecutable = ALString( oalList[0] ).replace( "/", "\\" )
            else:
                self.m_oExecutable = ALString( oalList[1] ).replace( "/", "\\" )
        else:
            self.m_oExecutable = self.m_oCommand
        self.m_oExecutable = ALString( self.m_oExecutable.capitalize() )


    def getCommandLine( self ):
        return self.m_oCommand


    def getInput( self ):
        self.synchronizeOn()
        oalList = ALList()
        oalList.add( self.m_oCommand )
        self.synchronizeOff()
        return oalList


    def getOutput( self ):
        self.synchronizeOn()
        oalList = ALList()
        if (self.m_oalSubProcess!=None):
            oalList = self.m_oalSubProcess.getOutput()
        self.synchronizeOff()
        return oalList


    def getOutputError( self ):
        self.synchronizeOn()
        oalList = ALList()
        if (self.m_oalSubProcess!=None):
            oalList = self.m_oalSubProcess.getOutputError()
        self.synchronizeOff()
        return oalList



    def executeProcess( self, _oalObject = None ):
        """
        This method starts the thread.
        """
        self.executeAction( _oalObject )




    def process( self, _oalObject = None ):
        """
        This is the main method of the action.
        """
        ALVerbose.DEBUGID( self.getID(), "Execution: ALActionProcess.process()" )

        self.__controlListDataProcess()

        self.synchronizeOn()
        # ALL folder change is deprecated to avoid Python mistakes in massive parallel actions
        self.m_oalSubProcess = ALSubProcess( self.m_oCommand, self.getWorkingFolder() )

        
        # It takes time to start the child process on Lazy systems - Problem of Load Balance on the system
        # Default time is AL_TIME_MINIMUM / 10.0 - 0.08 s
        ALTime.wait( self.getTimeDelaySystemLoadBalance() )
        self.m_oalActionMethod = ALActionMethod( self.m_oalSubProcess.executeProcess )
        self.m_oalActionMethod.setTimeOut( self.getTimeOut() )
        self.m_oalActionMethod.execute( _oalObject )

        self.synchronizeOff()

        self.__controlListDataProcess()
        self.__synchronizeActionMethod()



    def __controlListDataProcess( self ):

        oalUserName = self.m_oalManagerProcess.getUserName()
#        print oalUserName
        self.m_oalManagerProcess.control()
        oalListDataProcess = self.m_oalManagerProcess.getListDataProcess()
        if (self.m_oalListDataProcess==None):
            self.m_oalListDataProcess = ALList( oalListDataProcess )
#            print "New List: " + ALString( self.m_oalListDataProcess.getNumberObjects() )
            for iProcOld in self.m_oalListDataProcess:
                if (oalUserName!=iProcOld.getUserName()):
                    iProcOld.setPid( -1 )
        else:
            for iProc in oalListDataProcess:
                if (oalUserName==iProc.getUserName()):
                    obStatus = True
                    for iProcOld in self.m_oalListDataProcess:
                        if ( int( iProc.getPid()) == int( iProcOld.getPid() )):
                            obStatus = False
                            iProcOld.setPid( -1 )
                    if(obStatus == True and oalUserName==iProc.getUserName()):
#                        print "adding: " + ALString( iProc.getPid() )
                        self.m_oalListDataProcess.append( iProc )


            if (self.m_oPid!=None):
                oiPidOld = self.m_oPid
            else:
                oiPidOld = 0
#            print self.m_oPid
            for iProc in self.m_oalListDataProcess:
                if (int( iProc.getPid() )!=-1):

                    oalString = ALString( iProc.getCommand().capitalize() )
#                    print oalString
#                    if(self.m_oExecutable.contains( oalString )==True):
                    if( oalString.contains( self.m_oExecutable )==True and
                        oalUserName==iProc.getUserName() and
                        oiPidOld < int(iProc.getPid()) ):
#                        print iProc.getImageName()
#                        print iProc.getPid()
#                        print iProc.getSessionName()
#                        print iProc.getSession()
#                        print iProc.getMemoryUsage()
#                        print iProc.getStatus()
#                        print iProc.getUserName()
#                        print iProc.getCpuTime()
#                        print iProc.getWindowTitle()
#                        print iProc.getCommand()
#                        print "\n\n\n"
                        self.m_oPid = iProc.getPid()
                        oiPidOld = iProc.getPid()




    def __synchronizePid( self, _oTimeOut = 2.0 ):
        """
        It controls the lazy startup on charged systems and try to
        catch the pid number
        """
        ALVerbose.DEBUGID( self.getID(), "Execution: ALActionProcess.__synchronizePid()" )

        oalTime = ALTime()
        while (self.m_oPid==None):
            self.actionControl()
            ALTime.wait( AL_TIME_MINIMUM )
            self.__controlListDataProcess()
            if (oalTime.getDuration()>_oTimeOut):
                self.m_oPid = -1

        # To verify lazy startup
        if (self.m_oPid == -1):
            self.__controlListDataProcess()



    def __synchronizeActionMethod( self ):
        ALVerbose.DEBUGID( self.getID(), "Execution: ALActionProcess.synchronize()" )
        self.__synchronizePid()
        oalTime = ALTime()
        while (self.__isRunningActionMethod()==True):
#            print oalTime.getDuration()
            self.actionControl()
            ALTime.wait( AL_TIME_MINIMUM )
            if (oalTime.getDuration()>self.getTimeOut()):
                self.abort()


    def synchronize( self ):
        self.__synchronizeActionMethod()


    def synchronizeAction( self ):
        self.synchronize()


    def __patchLinuxBashProcess( self ):
        #print ">>>>>>>>>>>>>>>>>>>>> teste do if not"
        obIsRunnig = self.m_oalManagerProcess.isRunning( self.m_oCommand )
        if (obIsRunnig==True):
            self.m_oPid = self.m_oalManagerProcess.getPid( self.m_oCommand )
        return obIsRunnig



    def __isRunningActionMethod( self ):
        self.synchronizeOn()
        obIsRunnig = False
        if (ALSystem.getInterpreter()=="Python"):
            #Patch to control if the process is created or Not
            if (self.m_oalActionMethod==None):
                ALVerbose.WarningID( self.getID(), "Problem creating the SubProcess: Please inspect your Python SubProcess Module" ) 
            # The system will explose if the python subprocess crash the child subprocess              
            if (self.m_oalActionMethod.getTerminateSuccess()!=True):
                obIsRunnig = self.m_oalManagerProcess.isRunningPid( self.m_oPid )
                if (obIsRunnig==False):
                    obIsRunnig = self.__patchLinuxBashProcess()
                ALVerbose.DEBUGID( self.getID(), "ALActionProcess.isRunning(): " + ALString( obIsRunnig ) + "   PID: " + ALString( self.m_oPid ) )
            else:
                ALVerbose.DEBUGID( self.getID(),"ALActionMethod class finished the execution of the subprocess" )
        else:
            obIsRunnig = self.m_oalManagerProcess.isRunningPid( self.m_oPid )
            if (obIsRunnig==False):
                obIsRunnig = self.__patchLinuxBashProcess()
            ALVerbose.DEBUGID( self.getID(), "ALActionProcess.isRunning(): " + ALString( obIsRunnig ) + "   PID: " + ALString( self.m_oPid ) )
        self.synchronizeOff()
        return obIsRunnig



    def abort( self, _oalObject = None ):
        self.synchronizeOn()
#        print "abort"
        self.m_oalManagerProcess.killProcess( self.getCommandLine() )
        self.m_bIsAbort = True
        self.synchronizeOff()



##############################################################################
