##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from ALImportPython    import  PyXmlNode
from ALObject          import  ALObject
from ALString          import  ALString
from ALList            import  ALList


class ALDictionary( ALObject, dict ):

    """
    Interface for ALDictionary objects.
    This is a ordered Dictionary.
    This code is based on the roriginal proposed by:
    David Benjamin, in 2002/01/21 - Version no: 1.2
    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/107747

    """

    def __init__(self, _odict = {}):
        ALObject.__init__( self )
        self._keys = []
        dict.__init__( self )
        if ( isinstance( _odict, ALDictionary )):
            for ikey in _odict.ordered():
                self.addObject( ikey, _odict[ikey] )
        elif ( isinstance( _odict, dict )):
            for ikey in _odict:
                self.addObject( ikey, _odict[ikey] )



    def __delitem__(self, key):
        dict.__delitem__(self, key)
        self._keys.remove(key)


    def __setitem__(self, key, item):
        dict.__setitem__(self, key, item)
        # a peculiar sharp edge from copy.deepcopy
        # we'll have our set item called without __init__
        if not hasattr(self, '_keys'):
            self._keys = [key,]
        if key not in self._keys:
            self._keys.append(key)


    def clear(self):
        dict.clear(self)
        self._keys = []


    def copy(self):
        dict = dict.copy(self)
        dict._keys = self._keys[:]
        return dict


    def items(self):
        return zip(self._keys, self.values())


    def keys(self):
        return self._keys


    def popitem(self):
        try:
            key = self._keys[-1]
        except IndexError:
            raise KeyError('dictionary is empty')

        val = self[key]
        del self[key]

        return (key, val)


    def setdefault(self, key, failobj = None):
        dict.setdefault(self, key, failobj)
        if key not in self._keys:
            self._keys.append(key)


    def update(self, dict):
        dict.update(self, dict)
        for key in dict.keys():
            if key not in self._keys:
                self._keys.append(key)


    def values(self):
        return map(self.get, self._keys)




    # AALib evolution
    def getSequence( self ):
        return ALList( self.keys() )

    def getSequenceOrdered( self ):
        self.synchronizeOn()
        odtList = ALList( self.keys() )
        odtList.sort( lambda a, b: cmp( a, b ) )
        self.synchronizeOff()
        return odtList


    def ordered( self ):
        return ALList( self.keys() )


    def getValue( self, _key ):
        return self.get( _key )


    def setValue( self, _key, _value ):
        self[_key] = _value


    def add( self, _key, _value ):
        self[_key] = _value


    # Basic interface to generic objects
    def getObject( self, _key ):
        return self.get( _key )


    def addObject( self, _key, _value ):
        self.synchronizeOn()
        self[_key] = _value
        self.synchronizeOff()


    def removeObject( self, _key ):
        self.synchronizeOn()
        del self[_key]
        self.synchronizeOff()


    def getNumberObjects( self ):
        return len( self )


    def inputXml( self, _oalXmlElement, _oalXml = None  ):
        self.inputXmlObject( _oalXmlElement )

        for inode in _oalXmlElement.childNodes:
            if ( inode.nodeType==PyXmlNode.ELEMENT_NODE ):
                for jnode in inode.childNodes:
                    if ( jnode.nodeType==PyXmlNode.TEXT_NODE ):
                        oalToken  = ALString( inode.nodeName )
                        oalString = ALString( jnode.data )
                        if ( oalToken!="objectID" and oalToken!="objectVersion" and oalToken!="objectName" ):
                            #Workaround to bypass java-1.5 bug reading two time the same node.
                            oalCleanToken  = oalToken.cleanXml()
                            oalCleanString = oalString.cleanXml()
                            if (oalCleanToken!=ALString.empty() and oalCleanString!=ALString.empty()):
                                self.addObject( oalCleanToken, oalCleanString )


    def outputXml( self, _oalXml, _oalXmlElement ):
        oalXmlElementObject = self.outputXmlObject( _oalXml, _oalXmlElement )

        for ikey in self.ordered():
            if ( self[ ikey ] is not None ):
               oalXmlElement = _oalXml.createElement( ALString( ikey ) )
               oalXmlElement.appendChild( _oalXml.createTextNode( ALString( self[ ikey ] ) ) )
               oalXmlElementObject.appendChild( oalXmlElement )



