##############################################################################
#
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/02/02
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005-2007, AALib-PyAALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################



from ALSystem    import ALSystem

"""
Implements the ALSubProcess model for Java-Jython.
"""


if (ALSystem.getInterpreter()=="Jython"):
    """
    Implements the official Jython 2.2 subprocess model.
    """

    from ALImportJython     import *
    from ALObject           import ALObject
    from ALString           import ALString
    from ALVerbose          import ALVerbose
    from ALDiskExplorer     import ALDiskExplorer
    from ALList             import ALList
    from ALSystem           import ALSystem
    from ALTime             import ALTime
    from ALDebug            import ALDebug


    class ALJythonSubProcess( ALObject ):
        """
        Interface for ALSubProcess objects in Jython.
        ref: http://java.sun.com/javase/6/docs/api/java/lang/ProcessBuilder.html#start()
             http://java.sun.com/javase/6/docs/api/java/lang/Process.html
        """
        def __init__( self, _oStrCommandArgs, _oStringFolderName = None ):
            ALObject.__init__( self )
            self.m_ojyProcess = None
            self.m_oalStrCommand = ALString( _oStrCommandArgs )
            # Workaround due the limitation on the java interface to control external processes
            # It will improved with new process interface manager to be developed
            self.m_RETURN_CODE_REFERENCE = -99999.99999
            self.m_oReturnCode  = self.m_RETURN_CODE_REFERENCE
            self.m_obIsFinished = False
            self.m_obIsStarted  = False

            self.m_ojyStdInput    = None
            self.m_ojyStdError    = None
            self.m_ojyStdOutput   = None

            if (_oStringFolderName==None):
                self.m_oStringFolderName = ALSystem.getCurrentFolder()
            else:
                self.m_oStringFolderName = _oStringFolderName
                ALDiskExplorer.createFolder( self.m_oStringFolderName )



        def setWorkingDirectory( self, _oStringFolderName ):
            self.setFolder( _oStringFolderName )


        def setDirectory( self, _oStringFolderName ):
            self.setFolder( _oStringFolderName )


        def setWorkingFolder( self, _oStringFolderName ):
            self.setFolder( _oStringFolderName )


        def setFolder( self, _oStringFolderName ):
            self.synchronizeOn()
            self.m_oStringFolderName = ALDiskExplorer.getAbsolutePath( _oStringFolderName )
            ALDiskExplorer.createFolder( self.m_oStringFolderName )
            self.synchronizeOff()


        def getInput( self ):
            self.synchronizeOn()
            oListStdInput = ALList()
            oListStdInput.add( self.m_oalStrCommand )
            self.synchronizeOff()
            return oListStdInput



        def getOutputError( self ):
            self.synchronizeOn()
            oListStdError = ALList()
            if (self.m_obIsStarted == True and self.m_ojyStdError.ready()):
                ojyStringLine =  self.m_ojyStdError.readLine()
                while ( ojyStringLine != None ):
                    oListStdError.add( ojyStringLine )
                    ojyStringLine =  self.m_ojyStdError.readLine()
            self.synchronizeOff()
            return oListStdError



        def getOutput( self ):
            self.synchronizeOn()
            oListStdOutput = ALList()
            if (self.m_obIsStarted == True and self.m_ojyStdOutput.ready()):
                ojyStringLine =  self.m_ojyStdOutput.readLine()
                while ( ojyStringLine != None ):
                    oListStdOutput.add( ojyStringLine )
                    ojyStringLine =  self.m_ojyStdOutput.readLine()
            self.synchronizeOff()
            return oListStdOutput




        def getReturnCode( self ):
            self.synchronizeOn()
            try:
                # The java can not say if the process is still running
                # The  java creates a stupid exception?????? and after it lost the exitValue()
                if (self.m_ojyProcess!=None):
                    oReturnCode = self.m_ojyProcess.exitValue()
                else:
                    oReturnCode = 0
                self.m_oReturnCode = oReturnCode
            finally:
                ALVerbose.DEBUG( "The Java Process interface crashes when it calls the exitValue() status" )
                ALVerbose.DEBUG( "The Java Process interface will be deprecated in the AALib" )
                ALVerbose.DEBUG( "The AALib will implement a complete...version" )
                self.m_oReturnCode = self.m_RETURN_CODE_REFERENCE
                self.m_oReturnCode = 1
            self.synchronizeOff()
            return self.m_oReturnCode


        def __raiseControl( self ):
            oBoolStatus =  ALDebug.isDebugModeOn() # bug in jython version
            ALDebug.controlDebug( self, oBoolStatus )



        def executeProcess( self, _oalObject = None ):
            ojyFolderName = JyFile( self.m_oStringFolderName )
            self.m_ojyProcess = JyRuntime.getRuntime().exec(self.m_oalStrCommand, None, ojyFolderName  )
            self.m_obIsStarted  = True


            ojyInputReader = JyInputStreamReader( self.m_ojyProcess.getInputStream() )
            self.m_ojyStdOutput = JyBufferedReader( ojyInputReader )

            ojyErrorReader = JyInputStreamReader( self.m_ojyProcess.getErrorStream() )
            self.m_ojyStdError = JyBufferedReader( ojyErrorReader )

            return self.getOutput(), self.getOutputError()


        def getCommunication( self, _oalObject = None ):
            return self.executeProcess( _oalObject )



##############################################################################