#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControlCharacterisationv01.py 475 2008-07-30 16:07:01Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDPluginControl
from EDImport import EDVerbose
from EDImport import EDString
from EDImport import EDList
from EDImport import EDDict
from EDImport import EDMessage
from EDImport import EDUtilsFile
from EDImport import EDApplication

from XSDataCommonv01 import XSDataCrystal
from XSDataCommonv01 import XSDataSpaceGroup

from XSDataCharacterisationv01 import XSDataCollection
from XSDataCharacterisationv01 import XSDataIndexingInput
from XSDataCharacterisationv01 import XSDataCharacterisation
from XSDataCharacterisationv01 import XSDataIntegrationInput
from XSDataCharacterisationv01 import XSDataStrategyInput


class EDPluginControlCharacterisationv01( EDPluginControl ):
    """
    The Plugin that controls the whole characterisation.
    """
    
    def __init__ ( self ):
        """
        """
        EDPluginControl.__init__( self )
               
        self.m_oedStrPluginIndexingName = "EDPluginControlIndexingv01"
        self.m_oedPluginIndexing = None  
        
        self.m_oedStrPluginIntegrationName = "EDPluginControlIntegrationv01"
        self.m_oedPluginIntegration = None  
        
        self.m_oedStrPluginStrategyName = "EDPluginControlStrategyv01"
        self.m_oedPluginStrategy = None  

        self.m_oxsDataCharacterisation = None


    def setDataInput( self, _dataInput ):
        """
        Sets the Plugin Input Data
        _dataInput could be either an String XML or an XSData
        The XML String should be parsed as an Output Object by the final Plugin
        """
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.setDataInput" )
        # Check the type
        if( isinstance( _dataInput, EDString ) ):
            EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.setDataInput: Input Data is of EDString "  )
            self.m_oxsDataInput = XSDataCollection.parseString( _dataInput )
        elif ( isinstance( _dataInput, XSDataCollection ) ):
            self.m_oxsDataInput = _dataInput
        else:
            raise "Wrong input type for plugin EDPluginControlCharacterisationv01.setDataInput"


    def checkParameters( self ):
        """
        Checks the mandatory parameters
        """
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                      "Data Input is None" )
        self.checkMandatoryParameters( self.getDataInput().getDiffractionPlan(), "diffractionPlan" )


    def preProcess( self, _oalObject = None ):
        """
        Gets the Configuration Parameters, if found, overrides default parameters
        """
        EDPluginControl.preProcess( self, _oalObject )
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.preProcess..." )
        
        self.m_oedPluginIndexing = EDApplication.getPluginObject( self.m_oedStrPluginIndexingName )
        
        if ( self.m_oedPluginIndexing is not None ):
            EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.preProcess: " + self.m_oedStrPluginIndexingName + " Found... setting Data Input")
            self.m_oedPluginIndexing.setBaseDirectory( self.getWorkingDirectory() )
            self.m_oedPluginIndexing.setBaseName( "Indexing" )
            # create Data Input for indexing
            xsDataCollection   = self.getDataInput()
            xsDataSample       = xsDataCollection.getSample()
            xsDataSubWedgeList = xsDataCollection.getSubWedge()
            xsDataExperimentalCondition = xsDataSubWedgeList[0].getExperimentalCondition()
            
            xsDataIndexingInput = XSDataIndexingInput()
            xsDataIndexingInput.setDataCollection( xsDataCollection )
            xsDataIndexingInput.setExperimentalCondition( xsDataExperimentalCondition )
            
            xsDataDiffractionPlan = xsDataCollection.getDiffractionPlan()
            xsDataStringForcedSpaceGroup = xsDataDiffractionPlan.getForcedSpaceGroup()
            if ( xsDataStringForcedSpaceGroup is not None ):
                xsDataCrystal = XSDataCrystal()
                xsDataSpaceGroup = XSDataSpaceGroup()
                xsDataSpaceGroup.setName( xsDataStringForcedSpaceGroup )
                xsDataCrystal.setSpaceGroup( xsDataSpaceGroup )
                xsDataIndexingInput.setCrystal( xsDataCrystal )
            
            self.m_oedPluginIndexing.setDataInput( xsDataIndexingInput )
            
            # Populate characterisation object
            self.m_oxsDataCharacterisation = XSDataCharacterisation()
            self.m_oxsDataCharacterisation.setDataCollection( XSDataCollection.parseString( xsDataCollection.marshal() ) )
            
        else:
            errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ('EDPluginControlCharacterisationv01.preProcess', self.m_oedStrPluginIndexingName )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )              
            raise RuntimeError, errorMessage

        self.m_oedPluginIntegration = EDApplication.getPluginObject( self.m_oedStrPluginIntegrationName )
        if ( self.m_oedPluginIntegration is None ):
            errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ('EDPluginControlCharacterisationv01.preProcess', self.m_oedStrPluginIntegrationName )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )              
            raise RuntimeError, errorMessage
        else:
            self.m_oedPluginIntegration.setBaseDirectory( self.getWorkingDirectory() )
            self.m_oedPluginIntegration.setBaseName( "Integration" )
        self.m_oedPluginStrategy = EDApplication.getPluginObject( self.m_oedStrPluginStrategyName )
        if ( self.m_oedPluginStrategy is None ):
            errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ('EDPluginControlStrategyv01.preProcess', self.m_oedStrPluginStrategyName )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )              
            raise RuntimeError, errorMessage
        else:
            self.m_oedPluginStrategy.setBaseDirectory( self.getWorkingDirectory() )
            self.m_oedPluginStrategy.setBaseName( "Strategy" )
          
          
    def process( self, _oalObject = None ):
        """
        """
        EDPluginControl.process( self, _oalObject )
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.process" )
        self.connectProcess( self.m_oedPluginIndexing.executeSynchronous )
        self.m_oedPluginIndexing.connectSUCCESS( self.doIndexingToIntegrationTransition )
        self.m_oedPluginIndexing.connectFAILURE( self.doFailureActionIndexing )
        self.m_oedPluginIntegration.connectSUCCESS( self.doIntegrationToStrategyTransition )
        self.m_oedPluginIntegration.connectFAILURE( self.doFailureActionIntegration )
        self.m_oedPluginStrategy.connectFAILURE( self.doFailureActionStrategy )
        self.m_oedPluginStrategy.connectSUCCESS( self.doSuccessActionStrategy )
        
        
    def postProcess( self, _oalObject = None ):
        """
        """
        EDPluginControl.postProcess( self, _oalObject )
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.postProcess" )
        if ( self.m_oxsDataCharacterisation is not None ):
            self.setDataOutput( self.m_oxsDataCharacterisation )
            
    
    def doIndexingToIntegrationTransition( self,  _oedPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.doIndexingToIntegrationTransition" )
        if ( _oedPlugin.getErrorMessages().getNumberObjects() != 0 ):
            self.doFailureActionIntegration( _oedPlugin )
        else:
            self.retrieveSuccessMessages( _oedPlugin, "EDPluginControlCharacterisationv01.doIntegrationToStrategyTransition" )
            xsDataIndexingResult = self.m_oedPluginIndexing.getDataOutput()
            self.m_oxsDataCharacterisation.setIndexingResult( xsDataIndexingResult )
            oxsDataIntegrationInput = XSDataIntegrationInput()
            oxsDataIntegrationInput.setDataCollection( self.m_oxsDataCharacterisation.getDataCollection() )
            oxsDataIntegrationInput.setExperimentalConditionRefined( xsDataIndexingResult.getSelectedSolution().getExperimentalConditionRefined() )
            oxsDataIntegrationInput.setSelectedIndexingSolution( xsDataIndexingResult.getSelectedSolution() )
            self.m_oedPluginIntegration.setDataInput( oxsDataIntegrationInput )
            self.m_oedPluginIntegration.executeSynchronous()
        
        
    def doIntegrationToStrategyTransition( self,  _oedPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.doIntegrationToStrategyTransition" )
        if ( _oedPlugin.getErrorMessages().getNumberObjects() != 0 ):
            self.doFailureActionIntegration( _oedPlugin )
        else:
            self.retrieveSuccessMessages( _oedPlugin, "EDPluginControlCharacterisationv01.doIntegrationToStrategyTransition" )
        
            xsDataIntegrationOutput = self.m_oedPluginIntegration.getDataOutput()
            self.m_oxsDataCharacterisation.setIntegrationResult( xsDataIntegrationOutput )
            #EDVerbose.DEBUG( self.m_oxsDataExperimentCharacterisation.marshal() )
            xsDataStrategyInput = XSDataStrategyInput()
    
            xsDataSolutionSelected = self.m_oxsDataCharacterisation.getIndexingResult().getSelectedSolution()
            
            xsDataStrategyInput.setCrystalRefined( xsDataSolutionSelected.getCrystal() )
            xsDataStrategyInput.setSample( self.m_oxsDataCharacterisation.getDataCollection().getSample() )
                    
            xsDataIntegrationSubWedgeResultList = xsDataIntegrationOutput.getIntegrationSubWedgeResult()
            
            xsDataStrategyInput.setBestFileContentDat(    xsDataIntegrationSubWedgeResultList[0].getBestfileDat() )
            xsDataStrategyInput.setBestFileContentPar(    xsDataIntegrationSubWedgeResultList[0].getBestfilePar() )
            xsDataStrategyInput.setExperimentalCondition( xsDataIntegrationSubWedgeResultList[0].getExperimentalConditionRefined() )
            
            for xsDataIntegrationSubWedgeResult in xsDataIntegrationSubWedgeResultList:
                xsDataStrategyInput.addBestFileContentHKL( xsDataIntegrationSubWedgeResult.getBestfileHKL() )
                
            xsDataStrategyInput.setDiffractionPlan( self.m_oxsDataCharacterisation.getDataCollection().getDiffractionPlan() )
            
            #print xsDataStrategyInput.marshal()
            self.m_oedPluginStrategy.setDataInput( xsDataStrategyInput )
            self.m_oedPluginStrategy.executeSynchronous()
        


    def doFailureActionIndexing( self,  _oedPlugin = None ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.doFailureActionIndexing" )
        self.retrieveFailureMessages( _oedPlugin, "EDPluginControlCharacterisationv01.doFailureActionIndexing" )        


    def doFailureActionIntegration( self,  _oedPlugin = None ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.doFailureActionIntegration" )
        self.retrieveFailureMessages( _oedPlugin, "EDPluginControlCharacterisationv01.doFailureActionIntegration" )


    def doFailureActionStrategy( self,  _oedPlugin = None ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.doFailureActionStrategy" )
        self.retrieveFailureMessages( _oedPlugin, "EDPluginControlCharacterisationv01.doFailureActionStrategy" )
        
        
    def doSuccessActionStrategy( self, _oedPlugin ):
        """
        retrieve the potential warning messages
        """
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.doSuccessActionStrategy" )
        if ( _oedPlugin.getErrorMessages().getNumberObjects() != 0 ):
            self.doFailureActionStrategy( _oedPlugin )
        else:
            self.retrieveSuccessMessages( _oedPlugin, "EDPluginControlCharacterisationv01.doSuccessActionStrategy" )
            oxsDataStrategyResult = self.m_oedPluginStrategy.getDataOutput()
            self.m_oxsDataCharacterisation.setStrategyResult( oxsDataStrategyResult )
      
      
      
    def generateExecutiveSummary( self, _oedPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv01.generateExecutiveSummary" )
        self.addExecutiveSummaryLine( "Summary of characterisation:" )
        self.addErrorWarningMessagesToExecutiveSummary( "Characterisation failure! Error messages: " )
        self.addExecutiveSummarySeparator()
        xsDataCollection = self.getDataInput()
        xsDataDiffractionPlan = xsDataCollection.getDiffractionPlan()
        self.addExecutiveSummaryLine( "Diffraction plan:" )
        if ( xsDataDiffractionPlan.getAimedCompleteness() is not None ):
            self.addExecutiveSummaryLine( "Aimed completeness                    : %6.1f [%%]" % ( 100.0 * xsDataDiffractionPlan.getAimedCompleteness().getValue() ) )
        if ( xsDataDiffractionPlan.getRequiredCompleteness() is not None ):
            self.addExecutiveSummaryLine( "Required completeness                 : %6.1f [%%]" % ( 100.0 * xsDataDiffractionPlan.getRequiredCompleteness().getValue() ) )                  
        if ( xsDataDiffractionPlan.getAimedIOverSigmaAtHighestResolution() is not None ):
            self.addExecutiveSummaryLine( "Aimed I/sigma at highest resolution   : %6.1f" % xsDataDiffractionPlan.getAimedIOverSigmaAtHighestResolution().getValue() )                  
        if ( xsDataDiffractionPlan.getAimedResolution() is not None ):
            self.addExecutiveSummaryLine( "Aimed resolution                      : %6.1f [A]" % xsDataDiffractionPlan.getAimedResolution().getValue() )                  
        if ( xsDataDiffractionPlan.getRequiredResolution() is not None ):
            self.addExecutiveSummaryLine( "Required resolution                   : %6.1f [A]" % xsDataDiffractionPlan.getRequiredResolution().getValue() )                  
        if ( xsDataDiffractionPlan.getAimedMultiplicity() is not None ):
            self.addExecutiveSummaryLine( "Aimed multiplicity                    : %6.1f" % xsDataDiffractionPlan.getAimedMultiplicity().getValue() )                  
        if ( xsDataDiffractionPlan.getRequiredMultiplicity() is not None ):
            self.addExecutiveSummaryLine( "Required multiplicity                 : %6.1f" % xsDataDiffractionPlan.getRequiredMultiplicity().getValue() )                  
        if ( xsDataDiffractionPlan.getForcedSpaceGroup() is not None ):
            self.addExecutiveSummaryLine( "Forced space group                    : %6s" % xsDataDiffractionPlan.getForcedSpaceGroup().getValue() )                  
        if ( xsDataDiffractionPlan.getMaxExposureTimePerDataCollection() is not None ):
            self.addExecutiveSummaryLine( "Max exposure time per data collection : %6.1f [s]" % xsDataDiffractionPlan.getMaxExposureTimePerDataCollection().getValue() )                  
        self.addExecutiveSummarySeparator()
        if ( self.m_oedPluginIndexing is not None ):
            self.appendExecutiveSummary( self.m_oedPluginIndexing, "Indexing : " )
        if ( self.m_oedPluginIntegration is not None ):
            self.appendExecutiveSummary( self.m_oedPluginIntegration, "Integration : " )
        if ( self.m_oedPluginStrategy is not None ):
            self.appendExecutiveSummary( self.m_oedPluginStrategy, "Strategy : " )
            
      