#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControlGeneratePredictionv01.py 486 2008-08-01 15:03:18Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDPluginControl
from EDImport import EDVerbose
from EDImport import EDString
from EDImport import EDList
from EDImport import EDDict
from EDImport import EDMessage
from EDImport import EDUtilsFile
from EDImport import EDActionCluster
from EDImport import EDApplication
from EDImport import EDMessage
from EDImport import EDConfiguration

from XSDataCommonv01 import XSDataFloat
from XSDataCommonv01 import XSDataImage
from XSDataCommonv01 import XSDataCollection
from XSDataCommonv01 import XSDataSubWedge
from XSDataCommonv01 import XSDataExperimentalCondition

from XSDataCharacterisationv01 import XSDataIndexingSolutionSelected
from XSDataCharacterisationv01 import XSDataExperimentalCondition
from XSDataCharacterisationv01 import XSDataCollection
from XSDataCharacterisationv01 import XSDataIntegrationResult
from XSDataCharacterisationv01 import XSDataGeneratePredictionInput
from XSDataCharacterisationv01 import XSDataGeneratePredictionResult


class EDPluginControlGeneratePredictionv01( EDPluginControl ):
    """
    The Plugin that control the plugin that generates the prediction.
    If several images are available the execution of the plugin which generates
    the prediction is performed in parallel.
    """
    
    def __init__ ( self ):
        """
        """
        EDPluginControl.__init__( self )
               
        self.m_oedStrPluginGeneratePredictionName = "EDPluginMOSFLMv01GeneratePrediction"
        self.m_oedPluginGeneratePredictionList = []
        self.m_dNoSubwedges = None
        self.m_xsDataGeneratePredictionResult = None
        

    def setDataInput( self, _dataInput ):
        """
        Sets the Plugin Input Data
        _dataInput could be either an String XML or an XSDataGeneratePredictionInput
        """
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv01.setDataInput" )
        # Check the type
        if( isinstance( _dataInput, EDString ) ):
            EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv01.setDataInput: Input Data is of EDString "  )
            #print _dataInput
            self.m_oxsDataInput = XSDataGeneratePredictionInput.parseString( _dataInput )
            #print [ self.m_oxsDataInput ]
        elif ( isinstance( _dataInput, XSDataGeneratePredictionInput ) ):
            self.m_oxsDataInput = _dataInput
        else:
            errorMessage = EDMessage.ERROR_WRONG_DATA_TYPE_02 % ('EDPluginControlGeneratePredictionv01.setDataInput', "XMLString or XSDataGeneratePredictionInput expected" )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage


    def checkParameters( self ):
        """
        Checks the mandatory parameters
        """
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv01.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                               "Data Input is None" )
        self.checkMandatoryParameters( self.getDataInput().getDataCollection(),           "dataCollection" )
        self.checkMandatoryParameters( self.getDataInput().getSelectedIndexingSolution(), "selectedIndexingSolution" )


    def preProcess( self, _oalObject = None ):
        """
        Gets the Configuration Parameters, if found, overrides default parameters
        """
        EDPluginControl.preProcess( self, _oalObject )
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv01.preProcess..." )
        
        self.m_oedActionCluster = EDActionCluster()
        # Value to be read from the configuration...
        self.m_oedActionCluster.setClusterSize(4)

        xsDataGeneratePredictionInput = self.getDataInput()       

        xsDataSelectedIndexingSolution     = xsDataGeneratePredictionInput.getSelectedIndexingSolution()
        xsDataExperimentalConditionRefined = xsDataSelectedIndexingSolution.getExperimentalConditionRefined()
        xsDataCollection                   = xsDataGeneratePredictionInput.getDataCollection()
        xsDataSubWedgeList          = xsDataCollection.getSubWedge()
        
        # List containing instances of all the generate prediction plugins
        self.m_oedPluginGeneratePredictionList = EDList()
        
        # Loop through all subwedges
        iIndex = 0
        for xsDataSubWedge in xsDataSubWedgeList:

            xsDataImageList = xsDataSubWedge.getImage()
            # First find the lowest image number
            iLowestImageNumber = None
            for xsDataImage in xsDataImageList:
                iImageNumber = xsDataImage.getNumber().getValue()
                if ( iLowestImageNumber is None ):
                    iLowestImageNumber = iImageNumber
                elif ( iImageNumber < iLowestImageNumber ):
                    iLowestImageNumber = iImageNumber                
            # Then loop through all images in a sub wedge
            for xsDataImage in xsDataImageList:
                oedPluginGeneratePrediction = EDApplication.getPluginObject( self.m_oedStrPluginGeneratePredictionName )
                if ( oedPluginGeneratePrediction is None ):
                    errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ( "EDPluginControlGeneratePredictionv01.preProcess", self.m_oedStrPluginGeneratePredictionName )
                    EDVerbose.error( errorMessage )
                    self.addErrorMessage( errorMessage )              
                    raise RuntimeError, errorMessage
                iIndex += 1
                xsDataGeneratePredictionInput = XSDataGeneratePredictionInput()
                xsDataGeneratePredictionInput.setSelectedIndexingSolution( XSDataIndexingSolutionSelected.parseString( xsDataSelectedIndexingSolution.marshal() )  )
                xsDataCollectionNew = XSDataCollection()
                xsDataSubWedgeNew = XSDataSubWedge()
                xsDataSubWedgeNew.addImage( XSDataImage.parseString( xsDataImage.marshal() ) )
                xsDataSubWedgeNew.setExperimentalCondition( XSDataExperimentalCondition.parseString( xsDataSubWedge.getExperimentalCondition().marshal() ) )
                # We must modify the rotationOscillationStart for the new subwedge
                xsDataGoniostatNew = xsDataSubWedgeNew.getExperimentalCondition().getGoniostat()
                fGoniostatRotationAxisStart = xsDataGoniostatNew.getRotationAxisStart().getValue()
                fGonioStatOscillationRange  = xsDataGoniostatNew.getOscillationWidth().getValue()
                iImageNumber = xsDataImage.getNumber().getValue()
                fGoniostatRotationAxisStartNew = fGoniostatRotationAxisStart + ( iImageNumber - iLowestImageNumber )*fGonioStatOscillationRange
                xsDataGoniostatNew.setRotationAxisStart( XSDataFloat( fGoniostatRotationAxisStartNew ) )
                # 
                xsDataCollectionNew.addSubWedge( xsDataSubWedgeNew )
                xsDataGeneratePredictionInput.setDataCollection( xsDataCollectionNew )
                from EDHandlerXSDataMOSFLMv01 import EDHandlerXSDataMOSFLMv01
                try:
                    xsDataMOSFLMInputGeneratePrediction= EDHandlerXSDataMOSFLMv01.generateXSDataMOSFLMInputGeneratePrediction( xsDataGeneratePredictionInput )
                except Exception, error:
                    errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ( "EDPluginControlGeneratePredictionv01.preProcess: Unexpected error in MOSFLM handler", error )
                    EDVerbose.error( errorMessage )
                    self.addErrorMessage( errorMessage )
                    raise Exception, errorMessage
                oedPluginGeneratePrediction.setDataInput( xsDataMOSFLMInputGeneratePrediction )
                oedPluginGeneratePrediction.setBaseDirectory( self.getWorkingDirectory() )
                oedPluginGeneratePrediction.setBaseName( "%s-%02d" % ( self.m_oedStrPluginGeneratePredictionName, iIndex) )
                self.m_oedPluginGeneratePredictionList.add( oedPluginGeneratePrediction )
                    
          
          
    def process( self, _oalObject = None ):
        """
        """
        EDPluginControl.process( self, _oalObject )
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv01.process" )
        # Prepare the action cluster
        for oedPluginGeneratePrediction in self.m_oedPluginGeneratePredictionList:
            oedPluginGeneratePrediction.connectSUCCESS( self.doSuccessGeneratePrediction )
            oedPluginGeneratePrediction.connectFAILURE( self.doFailureGeneratePrediction )
            self.m_oedActionCluster.addAction( oedPluginGeneratePrediction )
        # Launch the cluster
        self.m_oedActionCluster.executeAction()
        self.m_oedActionCluster.synchronize()
        
        
    def doSuccessGeneratePrediction( self,  _oedPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv01.doSuccessGeneratePrediction" )
        if ( _oedPlugin.getErrorMessages().getNumberObjects() != 0 ):
            EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv01.doSuccessGeneratePrediction: Execution plugin returned error messages, failure!" )
            self.doFailureGeneratePrediction( _oedPlugin )
        else:
            self.retrieveSuccessMessages( _oedPlugin, "EDPluginControlGeneratePredictionv01.doSuccessGeneratePrediction" )
            if ( self.m_xsDataGeneratePredictionResult is None ):
                self.m_xsDataGeneratePredictionResult = XSDataGeneratePredictionResult()
            xsDataMOSFLMOutputGeneratePrediction = _oedPlugin.getDataOutput()
            xsDataImage = xsDataMOSFLMOutputGeneratePrediction.getPredictionImage()
            self.m_xsDataGeneratePredictionResult.addPredictionImage( XSDataImage.parseString( xsDataImage.marshal() ))


    def doFailureGeneratePrediction( self,  _oedPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv01.doFailureGeneratePrediction" )
        self.retrieveFailureMessages( _oedPlugin, "EDPluginControlGeneratePredictionv01.doFailureGeneratePrediction" )


    def postProcess( self, _oalObject = None ):
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv01.postProcess" )
        self.setDataOutput( self.m_xsDataGeneratePredictionResult )


    def generateExecutiveSummary( self, _oedPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv01.generateExecutiveSummary" )
        self.addExecutiveSummaryLine( "Summary of generation of image(s) with prediction:" )
        self.addErrorWarningMessagesToExecutiveSummary( "Generation of prediction failure! Error messages: " )
        xsDataGeneratePredictionResult = self.getDataOutput()
        if ( xsDataGeneratePredictionResult is not None ):
            for predictionImage in xsDataGeneratePredictionResult.getPredictionImage():
                self.addExecutiveSummaryLine( "Path to prediction image number %d : %s" % \
                                             ( predictionImage.getNumber().getValue(), predictionImage.getPath().getValue() ) )
            