#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControlIndexingv01.py 475 2008-07-30 16:07:01Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDPluginControl
from EDImport import EDVerbose
from EDImport import EDString
from EDImport import EDList
from EDImport import EDMessage
from EDImport import EDUtilsFile
from EDImport import EDApplication
from EDImport import EDMessage
from EDImport import EDConfiguration

from XSConfiguration import XSPluginItem

from XSDataCommonv01 import XSDataCollection
from XSDataCommonv01 import XSDataCrystal
from XSDataCommonv01 import XSDataSubWedge
from XSDataCommonv01 import XSDataExperimentalCondition
from XSDataCommonv01 import XSDataImage

from XSDataCharacterisationv01 import XSDataIndexingInput
from XSDataCharacterisationv01 import XSDataGeneratePredictionInput
from XSDataCharacterisationv01 import XSDataIndexingSolutionSelected


class EDPluginControlIndexingv01( EDPluginControl ):
    """
    The plugin which controls the indexing execution plugins.
    This plugin accepts several sub-wedges with one or several images each.
    The indexing is performed on all images together.
    One image with prediction is generated for each input image.
    """
    
    def __init__ ( self ):
        """
        """
        EDPluginControl.__init__( self )
               
        self.m_oedStrPluginIndexingName = "EDPluginMOSFLMv01Indexing"
        self.m_oedPluginIndexing = None
        self.m_oedStrPluginGeneratePredictionName = "EDPluginControlGeneratePredictionv01"
        self.m_oedPluginGeneratePrediction = None
        self.m_oxsDataIndexingOutput = None
        self.m_xsDataExperimentalCondition = None
        self.m_xsDataIndexingResult = None      
        self.m_xsDataCollection = None


    def setDataInput( self, _dataInput ):
        """
        Sets the Plugin Input Data
        _dataInput could be either an String XML or an XSData
        The XML String should be parsed as an Output Object by the final Plugin
        """
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.setDataInput" )
        # Check the type
        if( isinstance( _dataInput, EDString ) ):
            EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.setDataInput: Input Data is of EDString "  )
            self.m_oxsDataInput = XSDataIndexingInput.parseString( _dataInput )
        elif ( isinstance( _dataInput, XSDataIndexingInput ) ):
            self.m_oxsDataInput = _dataInput
        else:
            errorMessage = EDMessage.ERROR_WRONG_DATA_TYPE_02 % ('EDPluginControlIndexingv01.setDataInput', "XMLString or XSDataIndexingInput expected" )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage


    def checkParameters( self ):
        """
        Checks the mandatory parameters
        """
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                     "Data Input is None" )
        self.checkMandatoryParameters( self.getDataInput().getDataCollection(), "dataCollection" )


    def preProcess( self, _oalObject = None ):
        """
        Gets the Configuration Parameters, if found, overrides default parameters
        """
        EDPluginControl.preProcess( self, _oalObject )
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.preProcess..." )
        # Retrieve the input data
        xsDataIndexingInput = self.getDataInput()
        # Save the data collection and the experimental condition for constructing the indexing results
        self.m_xsDataExperimentalCondition = xsDataIndexingInput.getExperimentalCondition()
        self.m_xsDataCollection = xsDataIndexingInput.getDataCollection()
        # Load the MOSFLM index plugin and generate input data
        self.m_oedPluginIndexing = self.loadMOSFLMIndexingPlugin( self.m_oedStrPluginIndexingName, xsDataIndexingInput )        
        # Load the generate prediction plugin
        self.m_oedPluginGeneratePrediction = self.loadGeneratePredictionPlugin( self.m_oedStrPluginGeneratePredictionName )
        
          
    def process( self, _oedObject = None ):
        """
        """
        EDPluginControl.process( self, _oedObject )
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.process" )
        if ( self.m_oedPluginIndexing is not None ):
            self.m_oedPluginIndexing.connectSUCCESS( self.doSuccessActionIndexing )
            self.m_oedPluginIndexing.connectFAILURE( self.doFailureActionIndexing )
            self.m_oedPluginGeneratePrediction.connectSUCCESS( self.doSuccessActionGeneratePrediction )
            self.m_oedPluginGeneratePrediction.connectFAILURE( self.doFailureActionGeneratePrediction )
            self.m_oedPluginIndexing.executeSynchronous()
        
        
    def postProcess( self, _oedObject = None ):
        EDPluginControl.postProcess( self, _oedObject )
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.postProcess" )
        self.setDataOutput( self.m_xsDataIndexingResult )  


    def doSuccessActionIndexing( self, _oedPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.doSuccessActionIndexing" )
        if ( _oedPlugin.getErrorMessages().getNumberObjects() != 0 ):
            self.doFailureActionIndexing( _oedPlugin )
        else:
            # Retrieve the output from the MOSFLM plugin
            self.m_xsDataIndexingResult = self.getDataIndexingResultFromMOSFLM( _oedPlugin )    
            # Add the list of images to the results
            xsDataListImage = self.generateImageList( self.m_xsDataCollection ) 
            self.m_xsDataIndexingResult.setImage( xsDataListImage )
            # Generate prediction images
            xsDataGeneratePredictionInput = XSDataGeneratePredictionInput()
            xsDataGeneratePredictionInput.setDataCollection( XSDataCollection.parseString( self.m_xsDataCollection.marshal() ) )
            xsDataGeneratePredictionInput.setSelectedIndexingSolution( XSDataIndexingSolutionSelected.parseString( self.m_xsDataIndexingResult.getSelectedSolution().marshal() ) )
            self.m_oedPluginGeneratePrediction.setDataInput( xsDataGeneratePredictionInput )
            self.m_oedPluginGeneratePrediction.executeSynchronous()


    def doFailureActionIndexing( self,  _oedPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.doFailureActionIndexing" )
        EDVerbose.screen( "Execution of " + self.m_oedStrPluginIndexingName + "  failed." )
        EDVerbose.screen( "Please inspect the log file for further information." )
        self.retrieveFailureMessages( _oedPlugin, "EDPluginControlIndexingv01.doFailureActionIndexing" )

    
    def loadMOSFLMIndexingPlugin( self, _oedStringBaseName, _xsDataIndexingInput ): 
        """
        Load the MOSFLM indexing plugin and prepare data input. The base name of the plugin
        is given as an argument in the case of creation of several plugins to be launched
        simultaneously.
        """
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.loadMOSFLMIndexingPlugin" )
        oedPluginIndexing = EDApplication.getPluginObject( self.m_oedStrPluginIndexingName )
        if ( oedPluginIndexing is None ):
            errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ( "EDPluginControlIndexingv01.preProcess", self.m_oedStrPluginIndexingName )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )              
            raise RuntimeError, errorMessage
        from EDHandlerXSDataMOSFLMv01 import EDHandlerXSDataMOSFLMv01
        try:
            xsDataMOSFLMIndexingInput = EDHandlerXSDataMOSFLMv01.generateXSDataMOSFLMInputIndexing( _xsDataIndexingInput )
        except Exception, error:
            # This exception handling needs to be rethought, see bug #43.
            errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ( "EDPluginControlIndexingv01.preProcess: Unexpected error in MOSFLM handler: ", error )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage
        oedPluginIndexing.setDataInput( xsDataMOSFLMIndexingInput )
        oedPluginIndexing.setBaseDirectory( self.getWorkingDirectory() )
        oedPluginIndexing.setBaseName( _oedStringBaseName )
        return oedPluginIndexing


    def loadGeneratePredictionPlugin( self, _oedStringBaseName ):
        """
        Load the plugin which will generate the prediction images if indexing is successful.
        The base name of the plugin is given as an argument in the case of creation of 
        several plugins to be launched simultaneously.
        """
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.loadGeneratePredictionPlugin" )
        oedPluginGeneratePrediction = EDApplication.getPluginObject( self.m_oedStrPluginGeneratePredictionName )
        if ( oedPluginGeneratePrediction is None ):
            errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ( "EDPluginControlIndexingv01.preProcess", self.m_oedStrPluginGeneratePredictionName )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )              
            raise RuntimeError, errorMessage
        oedPluginGeneratePrediction.setBaseDirectory( self.getWorkingDirectory() )
        oedPluginGeneratePrediction.setBaseName( _oedStringBaseName )
        return oedPluginGeneratePrediction


    def getDataIndexingResultFromMOSFLM( self, _oedPlugin ):
        """
        This method retrieves the indexing results from a MOSFLM indexing plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.getDataIndexingResultFromMOSFLM" )
        xsDataIndexingResult = None
        oxsDataMOSFLMOutputIndexing = _oedPlugin.getDataOutput()
        from EDHandlerXSDataMOSFLMv01 import EDHandlerXSDataMOSFLMv01
        try:
            xsDataIndexingResult = EDHandlerXSDataMOSFLMv01.generateXSDataIndexingResult( oxsDataMOSFLMOutputIndexing, self.m_xsDataExperimentalCondition )
        except Exception, error:
            # This exception handling needs to be rethought, see bug #43.
            errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ( "EDPluginControlIndexingv01.preProcess: Unexpected error in MOSFLM handler: ", error )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage
        return xsDataIndexingResult
    
    
    def generatePredictionImageList( self, _oedPluginGeneratePrediction, _xsDataCollection, _xsDataIndexingResult ):
        """
        Generate a list containing the prediction images
        """
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.generatePredictionImageList" )
        xsDataGeneratePredictionInput = XSDataGeneratePredictionInput()
        xsDataGeneratePredictionInput.setDataCollection( XSDataCollection.parseString( _xsDataCollection.marshal() ) )
        xsDataGeneratePredictionInput.setSelectedIndexingSolution( XSDataIndexingSolutionSelected.parseString( _xsDataIndexingResult.getSelectedSolution().marshal() ) )
        _oedPluginGeneratePrediction.setDataInput( xsDataGeneratePredictionInput )
        _oedPluginGeneratePrediction.executeSynchronous()
        
        
        
    def doSuccessActionGeneratePrediction( self, _oedPlugin ):
        """
        Executed if the generate prediction plugin terminated correctly.
        """
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.doSuccessActionGeneratePrediction" )
        xsDataImageList = None
        if ( _oedPlugin.getErrorMessages().getNumberObjects() != 0 ):
            # Translate all error messages to warning messages
            for errorMessage in _oedPlugin.getErrorMessages():
                warningMessage = EDMessage.WARNING_CANNOT_USE_PLUGIN_03 % ( "EDPluginControlIndexingv01.generatePredictionImageList", \
                                                                            self.m_oedStrPluginGeneratePredictionName, \
                                                                            errorMessage  )
                EDVerbose.warning( warningMessage )
                self.addWarningMessages( [ warningMessage ] )
        else:
            self.retrieveSuccessMessages(_oedPlugin, "EDPluginControlIndexingv01.doSuccessActionGeneratePrediction" )               
            xsDataGeneratePredictionResult = _oedPlugin.getDataOutput()
            self.m_xsDataIndexingResult.setPredictionResult( xsDataGeneratePredictionResult )
    
    
    def doFailureActionGeneratePrediction( self, _oedPlugin ):
        """
        Executed if the generate prediction plugin did not terminate correctly.
        """
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.doFailureActionGeneratePrediction" )
        warningMessage = EDMessage.WARNING_CANNOT_USE_PLUGIN_03 % ( "EDPluginControlIndexingv01.doFailureActionGeneratePrediction", \
                                                                    self.m_oedStrPluginGeneratePredictionName, \
                                                                    errorMessage  )
        EDVerbose.warning( warningMessage )
        self.addWarningMessages( [ warningMessage ] )

    
    
    
    def generateImageList( self, _xsDataCollection ):
        """
        Make a list of all images in the subwedges
        """
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.generateImageList" )
        oedListImage = None
        if ( _xsDataCollection is not None ):
            oedListImage = EDList()
            xsDataSubWedgeList = _xsDataCollection.getSubWedge()
            for xsDataSubWedge in xsDataSubWedgeList:
                xsDataImageList = xsDataSubWedge.getImage()
                for xsDataImage in xsDataImageList:
                    oedListImage.add( XSDataImage.parseString( xsDataImage.marshal() ) )   
        return oedListImage
    

    def generateExecutiveSummary( self, _oedPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginControlIndexingv01.generateExecutiveSummary" )
        self.addExecutiveSummaryLine( "Summary of indexing:" )
        self.addErrorWarningMessagesToExecutiveSummary( "Indexing failure! Error messages: " )
        if ( self.m_oedPluginIndexing is not None ):
            self.appendExecutiveSummary( self.m_oedPluginIndexing, "MOSFLM : " )
        if ( self.m_oedPluginGeneratePrediction is not None ):
            self.appendExecutiveSummary( self.m_oedPluginGeneratePrediction, "Prediction : " )
        