#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControlIntegrationv01.py 475 2008-07-30 16:07:01Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDPluginControl
from EDImport import EDVerbose
from EDImport import EDString
from EDImport import EDList
from EDImport import EDDict
from EDImport import EDMessage
from EDImport import EDUtilsFile
from EDImport import EDActionCluster
from EDImport import EDApplication
from EDImport import EDMessage
from EDImport import EDConfiguration
from XSConfiguration import XSPluginItem

from XSDataCharacterisationv01 import XSDataIntegrationInput
from XSDataCharacterisationv01 import XSDataIndexingSolutionSelected
from XSDataCharacterisationv01 import XSDataExperimentalCondition
from XSDataCharacterisationv01 import XSDataCollection
from XSDataCharacterisationv01 import XSDataIntegrationResult


class EDPluginControlIntegrationv01( EDPluginControl ):
    """
    The Plugin that controls the integration.
    """
    
    def __init__ ( self ):
        """
        """
        EDPluginControl.__init__( self )
               
        self.m_oedStrPluginIntegrationName = "EDPluginMOSFLMv01Integration"
        self.m_oedPluginIntegrationList = []
        self.m_dNoSubwedges = None
        self.m_oedStrCONF_CONTROL_INTEGRATION_MAX_RMS = "maxRMSSpotDeviation"
        self.m_fMaxRMSSpotDeviation = None
        self.m_oxsDataExperimentalConditionRefined = None
        self.m_oedActionCluster = None
        self.m_xsDataIntegrationResult = None


    def setDataInput( self, _dataInput ):
        """
        Sets the Plugin Input Data
        _dataInput could be either an String XML or an XSData
        The XML String should be parsed as an Output Object by the final Plugin
        """
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv01.setDataInput" )
        # Check the type
        if( isinstance( _dataInput, EDString ) ):
            EDVerbose.DEBUG( "*** EDPluginControlIntegrationv01.setDataInput: Input Data is of EDString "  )
            #print _dataInput
            self.m_oxsDataInput = XSDataIntegrationInput.parseString( _dataInput )
            #print [ self.m_oxsDataInput ]
        elif ( isinstance( _dataInput, XSDataIntegrationInput ) ):
            self.m_oxsDataInput = _dataInput
        else:
            errorMessage = EDMessage.ERROR_WRONG_DATA_TYPE_02 % ('EDPluginControlIntegrationv01.setDataInput', "XMLString or XSDataIntegrationInput expected" )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage


    def checkParameters( self ):
        """
        Checks the mandatory parameters
        """
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv01.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                               "Data Input is None" )
        self.checkMandatoryParameters( self.getDataInput().getDataCollection(),           "dataCollection" )
        self.checkMandatoryParameters( self.getDataInput().getSelectedIndexingSolution(), "selectedIndexingSolution" )


    def configure( self ):
        EDPluginControl.configure( self )
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv01.configure")
        oxsPluginItem = self.getConfiguration()
        if ( oxsPluginItem == None ):
            EDVerbose.warning( "EDPluginControlIntegrationv01.configure: No plugin item defined." )
            oxsPluginItem = XSPluginItem()
        oedStringMaxRMSSpotDeviation = EDConfiguration.getStringParamValue( oxsPluginItem, self.m_oedStrCONF_CONTROL_INTEGRATION_MAX_RMS )
        if( oedStringMaxRMSSpotDeviation == None ):
            EDVerbose.DEBUG( "EDPluginControlIntegrationv01.configure: No configuration parameter found for: " + self.m_oedStrCONF_CONTROL_INTEGRATION_MAX_RMS + ", no default value." )
        else:
            self.setMaxRMSSpotDeviation( EDString( oedStringMaxRMSSpotDeviation ).atof() )


    def setMaxRMSSpotDeviation( self, _fMaxRMSSpotDeviation ):
        self.m_fMaxRMSSpotDeviation = _fMaxRMSSpotDeviation
        
    def getMaxRMSSpotDeviation( self ):
        return self.m_fMaxRMSSpotDeviation


    def preProcess( self, _oalObject = None ):
        """
        Gets the Configuration Parameters, if found, overrides default parameters
        """
        EDPluginControl.preProcess( self, _oalObject )
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv01.preProcess..." )
        
        self.m_oedActionCluster = EDActionCluster()
        # Value to be read from the configuration...
        self.m_oedActionCluster.setClusterSize(4)

        xsDataIntegrationInput = self.getDataInput()       
        xsDataSelectedIndexingSolution     = xsDataIntegrationInput.getSelectedIndexingSolution()
        self.m_oxsDataExperimentalConditionRefined = xsDataIntegrationInput.getExperimentalConditionRefined()
        # To be changed (see bug #40)
        if ( self.m_oxsDataExperimentalConditionRefined is None ):
            self.m_oxsDataExperimentalConditionRefined = xsDataSelectedIndexingSolution.getExperimentalConditionRefined()
        xsDataCrystalRefined               = xsDataIntegrationInput.getCrystalRefined()
        xsDataCollection                   = xsDataIntegrationInput.getDataCollection()
        xsDataSubWedgeList                 = xsDataCollection.getSubWedge()
        
        self.m_oedPluginIntegrationList = EDList()
        
        iIndex = 0
        for xsDataSubWedge in xsDataSubWedgeList:
  
          oedPluginIntegration = EDApplication.getPluginObject( self.m_oedStrPluginIntegrationName )
          
          if ( not oedPluginIntegration is None ):
              iIndex += 1
              xsDataIntegrationInputSubWedge = XSDataIntegrationInput()
              xsDataIntegrationInputSubWedge.setSelectedIndexingSolution( XSDataIndexingSolutionSelected.parseString( xsDataSelectedIndexingSolution.marshal() )  )
              xsDataIntegrationInputSubWedge.setExperimentalConditionRefined( XSDataExperimentalCondition.parseString( self.m_oxsDataExperimentalConditionRefined.marshal() ) )
              xsDataCollection = XSDataCollection()
              xsDataCollection.addSubWedge( xsDataSubWedge )
              xsDataIntegrationInputSubWedge.setDataCollection( xsDataCollection )
              try:
                  from EDHandlerXSDataMOSFLMv01 import EDHandlerXSDataMOSFLMv01
                  xsDataMOSFLMInputIntegration = EDHandlerXSDataMOSFLMv01.generateXSDataMOSFLMInputIntegration( xsDataIntegrationInputSubWedge )
                  oedPluginIntegration.setDataInput( xsDataMOSFLMInputIntegration )
                  oedPluginIntegration.setBaseDirectory( self.getWorkingDirectory() )
                  oedPluginIntegration.setBaseName( "%s-%02d" % ( self.m_oedStrPluginIntegrationName, iIndex) )
                  oedPluginIntegration.connectSUCCESS( self.doSuccessActionIntegration )
                  oedPluginIntegration.connectFAILURE( self.doFailureActionIntegration )                  
                  self.m_oedPluginIntegrationList.add( oedPluginIntegration )
              except Exception, errorMessage:
                  self.addErrorMessage( errorMessage )
                  raise Exception, errorMessage              
          else:
              errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ('EDPluginControlIntegrationv01.preProcess', self.m_oedStrPluginRaddoseName )
              EDVerbose.error( errorMessage )
              self.addErrorMessage( errorMessage )              
              raise RuntimeError, errorMessage
          
          
    def process( self, _oalObject = None ):
        """
        """
        EDPluginControl.process( self, _oalObject )
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv01.process" )
        for pluginIntegration in self.m_oedPluginIntegrationList:
            self.m_oedActionCluster.addAction( pluginIntegration )
        self.m_oedActionCluster.executeAction()
        self.m_oedActionCluster.synchronize()
        
        
    def doSuccessActionIntegration( self,  _oedPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv01.doSuccessActionIntegration" )
        if ( _oedPlugin.getErrorMessages().getNumberObjects() != 0 ):
            self.doFailureActionIntegration( _oedPlugin )
        else:
            #self.synchronizeOn()
            self.retrieveSuccessMessages( _oedPlugin, "EDPluginControlIntegrationv01.doSuccessActionIntegration" )        
            #self.synchronizeOff()
                


    def doFailureActionIntegration( self,  _oedPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv01.doFailureActionIntegration" )
        self.retrieveFailureMessages( _oedPlugin, "EDPluginControlIntegrationv01.doFailureActionIntegration" )


    def postProcess( self, _oalObject = None ):
        EDPluginControl.postProcess( self, _oalObject )
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv01.postProcess" )
        from EDHandlerXSDataMOSFLMv01 import EDHandlerXSDataMOSFLMv01
        for pluginIntegration in self.m_oedPluginIntegrationList:
            xsDataMOSFLMOutputIntegration = pluginIntegration.getDataOutput()
            if ( xsDataMOSFLMOutputIntegration is not None):
                xsDataIntegrationSubWedgeResult = None
                try:
                    xsDataIntegrationSubWedgeResult = EDHandlerXSDataMOSFLMv01.generateXSDataIntegrationSubWedgeResult( xsDataMOSFLMOutputIntegration, self.m_oxsDataExperimentalConditionRefined )
                except Exception, errorMessage:
                    self.addErrorMessage( errorMessage )
                if ( xsDataIntegrationSubWedgeResult is not None ):
                    xsDataStatisticsIntegration = xsDataIntegrationSubWedgeResult.getStatistics()
                    fRMSSpotDeviation = xsDataStatisticsIntegration.getRMSSpotDeviation().getValue()
                    if ( self.m_fMaxRMSSpotDeviation is not None ):
                        if ( self.m_fMaxRMSSpotDeviation < fRMSSpotDeviation ):
                            iImageStart  = pluginIntegration.getDataInput().getImageStart().getValue()
                            iImageEnd    = pluginIntegration.getDataInput().getImageEnd().getValue()
                            errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ( "EDPluginControlIntegrationv01.postProcess", \
                                                                               "MOSFLM Integration : RMS spot deviation (%.3f [mm]) larger than max value from configuration (%.3f [mm]) for images %d to %d"  % \
                                                                               ( fRMSSpotDeviation, self.m_fMaxRMSSpotDeviation, iImageStart, iImageEnd ) )
                            EDVerbose.error( errorMessage )
                            self.addErrorMessage( errorMessage )
                    if ( xsDataIntegrationSubWedgeResult is not None ):
                        if ( self.m_xsDataIntegrationResult is None ):
                            self.m_xsDataIntegrationResult = XSDataIntegrationResult()
                        self.m_xsDataIntegrationResult.addIntegrationSubWedgeResult( xsDataIntegrationSubWedgeResult )
        self.setDataOutput( self.m_xsDataIntegrationResult )


    def generateExecutiveSummary( self, _oedPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv01.generateExecutiveSummary" )
        self.addExecutiveSummaryLine( "Summary of integration:" )
        self.addErrorWarningMessagesToExecutiveSummary()
        for oedPluginIntegration in self.m_oedPluginIntegrationList:
            if ( oedPluginIntegration is not None ):
                if ( oedPluginIntegration.getDataOutput() is not None ):
                    self.appendExecutiveSummary( oedPluginIntegration, "MOSFLM : " )
