#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginMOSFLMv01.py 475 2008-07-30 16:07:01Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDPluginExecProcessScript
from EDImport import EDVerbose
from EDImport import EDString
from EDImport import EDList
from EDImport import EDDiskExplorer
from EDImport import EDMessage

from XSDataMOSFLMv01 import *


class EDPluginMOSFLMv01( EDPluginExecProcessScript ):

    def __init__( self ):
        EDPluginExecProcessScript.__init__( self )
        self.m_oedStringMOSFLMNewmatFileName = None
        self.m_oedStringMOSFLMMatrixFileName = None
        

    def preProcess( self, _oedObject = None ):
        EDPluginExecProcessScript.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01.preProcess")


    def process( self, _oedObject = None ):
        EDPluginExecProcessScript.process( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01.process")
        # It should not be possible to execute this abstract plugin
        if ( self.getPluginName().equals( "EDPluginMOSFLMv01" ) ):
             raise ExectuteAbstractPluginError

    
    def postProcess( self, _oedObject = None ):
        EDPluginExecProcessScript.postProcess( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01.postProcess")


    
    def configure( self ):
        EDPluginExecProcessScript.configure( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01.configure")
        self.setRequireCCP4( True )
        self.setScriptCommandline( " DNA " + self.getScriptBaseName() + "_dnaTables.xml" )
#        if ( self.m_oedStringMOSFLMNewmatFileName == None ):
#            self.m_oedStringMOSFLMNewmatFileName = EDString( self.m_oedStringScriptBaseName + ".mat" ) 


    def checkParameters( self ):
        """
        Checks the mandatory parameters for all MOSLFM plugins
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                 "Data Input is None" )
        self.checkMandatoryParameters( self.getDataInput().getBeam(),       "beamPosition" )
        self.checkMandatoryParameters( self.getDataInput().getDetector(),   "detector" )
        self.checkMandatoryParameters( self.getDataInput().getDirectory(),  "directory" )
        self.checkMandatoryParameters( self.getDataInput().getDistance(),   "distance" )
        self.checkMandatoryParameters( self.getDataInput().getTemplate(),   "template" )
        self.checkMandatoryParameters( self.getDataInput().getWavelength(), "wavelength" )


    def generateMOSFLMCommands( self ):
        """
        This method creates a list of MOSFLM indexing commands given a valid
        XSDataMOSFLMInput as self.getDataInput()
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01.generateMOSFLMCommands")
        oxsDataMOSFLMInput = self.getDataInput()

        if ( oxsDataMOSFLMInput is not None):
                        
            xsDataFloatWavelength = oxsDataMOSFLMInput.getWavelength()
            if ( xsDataFloatWavelength is not None ): 
                self.addListCommandExecution( "WAVELENGTH " + EDString( xsDataFloatWavelength.getValue() ) )

            xsDataLengthDistance = oxsDataMOSFLMInput.getDistance() 
            if ( xsDataLengthDistance is not None ):
                self.addListCommandExecution( "DISTANCE " + EDString( xsDataLengthDistance.getValue() ) )

            oxsDataMOSFLMBeamPosition = oxsDataMOSFLMInput.getBeam()
            if ( oxsDataMOSFLMBeamPosition is not None ):
                oedStringBeamX = EDString( oxsDataMOSFLMBeamPosition.getX().getValue() )
                oedStringBeamY = EDString( oxsDataMOSFLMBeamPosition.getY().getValue() )
                self.addListCommandExecution( "BEAM " + oedStringBeamX + " " + oedStringBeamY )

            oxsDataMOSFLMDetector = oxsDataMOSFLMInput.getDetector() 
            if ( oxsDataMOSFLMDetector is not None ):
                oedStringDetectorType = oxsDataMOSFLMDetector.getType()
                if ( oedStringDetectorType is not None ):
                    self.addListCommandExecution( "DETECTOR " + oedStringDetectorType.getValue() )
                    
            xsDataStringDirectory = oxsDataMOSFLMInput.getDirectory()
            if ( xsDataStringDirectory is not None ):
                self.addListCommandExecution( "DIRECTORY " + xsDataStringDirectory.getValue() )

            xsDataStringTemplate = oxsDataMOSFLMInput.getTemplate()
            if ( xsDataStringTemplate is not None ):
                self.addListCommandExecution( "TEMPLATE " + xsDataStringTemplate.getValue() )
            
            xsDataStringSymmetry = oxsDataMOSFLMInput.getSymmetry()
            if ( xsDataStringSymmetry is not None ):
                self.addListCommandExecution( "SYMMETRY " + xsDataStringSymmetry.getValue() )
            oedStringNewmatFileName = self.getNewmatFileName()

            xsDataNewmatMatrix = oxsDataMOSFLMInput.getMatrix()
            if ( xsDataNewmatMatrix is not None ):
                self.writeDataMOSFLMNewmat( xsDataNewmatMatrix, self.getMatrixFileName() )
                self.addListCommandExecution( "MATRIX " + self.getMatrixFileName() )
            
            xsDataFloatMosaicity = oxsDataMOSFLMInput.getMosaicity()
            if ( xsDataFloatMosaicity is not None ):
                self.addListCommandExecution( "MOSAIC " + EDString( xsDataFloatMosaicity.getValue() ) )


    def getNewmatFileName( self ):
        if ( self.m_oedStringMOSFLMNewmatFileName is None ):
            self.m_oedStringMOSFLMNewmatFileName = EDString( self.getScriptBaseName() + "_newmat.mat" )
        return EDString( self.m_oedStringMOSFLMNewmatFileName )

    
    def setNewmatFileName( self, _ostrMOSFLMNewmatFileName ):
        self.synchronizeOn()
        self.m_oedStringMOSFLMNewmatFileName = EDString( _ostrMOSFLMNewmatFileName )
        self.synchronizeOff()
        

    def getMatrixFileName( self ):
        if ( self.m_oedStringMOSFLMMatrixFileName is None ):
            self.m_oedStringMOSFLMMatrixFileName = EDString( self.getScriptBaseName() + "_matrix.mat" )
        return EDString( self.m_oedStringMOSFLMMatrixFileName )

    
    def setMatrixFileName( self, _ostrMOSFLMMatrixFileName ):
        self.synchronizeOn()
        self.m_oedStringMOSFLMMatrixFileName = EDString( _ostrMOSFLMMatrixFileName )
        self.synchronizeOff()
    
    
    def splitStringIntoListOfFloats( self, _oedStringInput ):
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01.splitStringIntoListOfFloats")
        oedListFloats = EDList()
        oedListString = EDString( _oedStringInput ).split()
        for pyStringElement in oedListString:
            oedStringElement = EDString( pyStringElement )
            if ( oedStringElement.equals( "" ) == False ):
                oedListFloats.add( EDString( oedStringElement ).atof() )
        return oedListFloats
    
    
    def getDataMOSFLMMatrix( self, _oedStringMatrixFileName = None  ):
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01.getDataMOSFLMMatrix")
        oedStringMatrixFileName = None
        if ( _oedStringMatrixFileName is None ):
            oedStringMatrixFileName = self.getMatrixFileName()
        else:
            oedStringMatrixFileName = _oedStringMatrixFileName
        oxsDataMOSFLMNewmatMatrix = self.getDataMOSFLMNewmat( oedStringMatrixFileName )
        return oxsDataMOSFLMNewmatMatrix
        
    
    def getDataMOSFLMNewmat( self, _oedStringNewmatFileName = None ):
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01.getDataMOSFLMNewmat")
        oxsDataMOSFLMNewmat = None
        oedStringNewmatFileName = None
        oedListOfListOfFloat = EDList()
        if ( _oedStringNewmatFileName == None ):
            oedStringNewmatFileName = self.getNewmatFileName()
        else:
            oedStringNewmatFileName = _oedStringNewmatFileName
        oedStringNewmat = None    
        try:
            oedStringNewmat = self.readProcessFile( oedStringNewmatFileName )
        except:
            oedStringError = self.readProcessErrorLogFile()
            if( (oedStringError is not None) and ( oedStringError != EDString.empty() ) ):
                errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginMOSFLMv01.getDataMOSFLMNewmat', self.getClassName(), oedStringError )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )
            else:
                errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ( "EDPluginMOSFLMv01.getDataMOSFLMNewmat", \
                                                                   "Cannot read MOSFLM NEWMAT file : " + oedStringNewmatFileName )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage ) 
        #print oedStringNewmat
        if ( oedStringNewmat is not None ):
            oedListLine = oedStringNewmat.split("\n")
            # Convert into list of lists of float
            for oedStringLine in oedListLine:
                oedListOfListOfFloat.add( self.splitStringIntoListOfFloats( oedStringLine ) )
            # Fill in the data
            oxsDataMOSFLMNewmat = XSDataMOSFLMNewmat()

            oxsDataMatrixA = XSDataMatrix()
            oxsDataMatrixA.setM11( oedListOfListOfFloat[0][0] )            
            oxsDataMatrixA.setM12( oedListOfListOfFloat[0][1] )
            oxsDataMatrixA.setM13( oedListOfListOfFloat[0][2] )
            oxsDataMatrixA.setM21( oedListOfListOfFloat[1][0] )
            oxsDataMatrixA.setM22( oedListOfListOfFloat[1][1] )
            oxsDataMatrixA.setM23( oedListOfListOfFloat[1][2] )
            oxsDataMatrixA.setM31( oedListOfListOfFloat[2][0] )
            oxsDataMatrixA.setM32( oedListOfListOfFloat[2][1] )
            oxsDataMatrixA.setM33( oedListOfListOfFloat[2][2] )
            oxsDataMOSFLMNewmat.setAMatrix( oxsDataMatrixA )
            
            oxsDataMOSFLMMissettingsAngles = XSDataMOSFLMMissettingsAngles()
            oxsDataMOSFLMMissettingsAngles.setPhix( XSDataAngle( oedListOfListOfFloat[3][0] ) )
            oxsDataMOSFLMMissettingsAngles.setPhiy( XSDataAngle( oedListOfListOfFloat[3][1] ) )
            oxsDataMOSFLMMissettingsAngles.setPhiz( XSDataAngle( oedListOfListOfFloat[3][2] ) )
            oxsDataMOSFLMNewmat.setMissettingAngles( oxsDataMOSFLMMissettingsAngles )

            oxsDataMatrixU = XSDataMatrix()
            oxsDataMatrixU.setM11( oedListOfListOfFloat[4][0] )
            oxsDataMatrixU.setM12( oedListOfListOfFloat[4][1] )
            oxsDataMatrixU.setM13( oedListOfListOfFloat[4][2] )
            oxsDataMatrixU.setM21( oedListOfListOfFloat[5][0] )
            oxsDataMatrixU.setM22( oedListOfListOfFloat[5][1] )
            oxsDataMatrixU.setM23( oedListOfListOfFloat[5][2] )
            oxsDataMatrixU.setM31( oedListOfListOfFloat[6][0] )
            oxsDataMatrixU.setM32( oedListOfListOfFloat[6][1] )
            oxsDataMatrixU.setM33( oedListOfListOfFloat[6][2] )
            oxsDataMOSFLMNewmat.setUMatrix( oxsDataMatrixU )
            
            oxsDataCellRefined = XSDataCell()
            oxsDataCellRefined.setLength_a(    XSDataLength( oedListOfListOfFloat[7][0] ) )
            oxsDataCellRefined.setLength_b(    XSDataLength( oedListOfListOfFloat[7][1] ) )
            oxsDataCellRefined.setLength_c(    XSDataLength( oedListOfListOfFloat[7][2] ) )
            oxsDataCellRefined.setAngle_alpha( XSDataAngle(  oedListOfListOfFloat[7][3] ) )
            oxsDataCellRefined.setAngle_beta(  XSDataAngle(  oedListOfListOfFloat[7][4] ) )
            oxsDataCellRefined.setAngle_gamma( XSDataAngle(  oedListOfListOfFloat[7][5] ) )
            oxsDataMOSFLMNewmat.setRefinedCell( oxsDataCellRefined )
            
        return oxsDataMOSFLMNewmat
    
    
    
    def writeDataMOSFLMNewmat( self, _oxsDataMOSFLNNewmat, _oedStringNewmatFileName = None ):
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01.writeDataMOSFLMNewmat")
        oedStringNewmatFileName = None
        if ( _oedStringNewmatFileName is None ):
            oedStringNewmatFileName = self.getNewmatFileName()
        else:
            oedStringNewmatFileName = _oedStringNewmatFileName
        oxsDataMatrixA = _oxsDataMOSFLNNewmat.getAMatrix()
        oedStringNewmat  = " %11.8f %11.8f %11.8f\n" % ( oxsDataMatrixA.getM11(), oxsDataMatrixA.getM12(), oxsDataMatrixA.getM13() )
        oedStringNewmat += " %11.8f %11.8f %11.8f\n" % ( oxsDataMatrixA.getM21(), oxsDataMatrixA.getM22(), oxsDataMatrixA.getM23() )
        oedStringNewmat += " %11.8f %11.8f %11.8f\n" % ( oxsDataMatrixA.getM31(), oxsDataMatrixA.getM32(), oxsDataMatrixA.getM33() )
        oxsDataMOSFLMMissettingAngles = _oxsDataMOSFLNNewmat.getMissettingAngles()
        oedStringNewmat += " %11.3f %11.3f %11.3f\n" % ( oxsDataMOSFLMMissettingAngles.getPhix().getValue(),
                                                         oxsDataMOSFLMMissettingAngles.getPhiy().getValue(),
                                                         oxsDataMOSFLMMissettingAngles.getPhiz().getValue())
        oxsDataMatrixU = _oxsDataMOSFLNNewmat.getUMatrix()
        oedStringNewmat += " %11.7f %11.7f %11.7f\n" % ( oxsDataMatrixU.getM11(), oxsDataMatrixU.getM12(), oxsDataMatrixU.getM13() )
        oedStringNewmat += " %11.7f %11.7f %11.7f\n" % ( oxsDataMatrixU.getM21(), oxsDataMatrixU.getM22(), oxsDataMatrixU.getM23() )
        oedStringNewmat += " %11.7f %11.7f %11.7f\n" % ( oxsDataMatrixU.getM31(), oxsDataMatrixU.getM32(), oxsDataMatrixU.getM33() )

        oxsDataCellRefined = _oxsDataMOSFLNNewmat.getRefinedCell()
        oedStringNewmat += " %11.4f %11.4f %11.4f" %   ( oxsDataCellRefined.getLength_a().getValue(),
                                                         oxsDataCellRefined.getLength_b().getValue(),
                                                         oxsDataCellRefined.getLength_c().getValue())
        oedStringNewmat += " %11.4f %11.4f %11.4f\n" % ( oxsDataCellRefined.getAngle_alpha().getValue(),
                                                         oxsDataCellRefined.getAngle_beta().getValue(),
                                                         oxsDataCellRefined.getAngle_gamma().getValue())
        oedStringNewmat += " %11.3f %11.3f %11.3f\n" % ( oxsDataMOSFLMMissettingAngles.getPhix().getValue(),
                                                         oxsDataMOSFLMMissettingAngles.getPhiy().getValue(),
                                                         oxsDataMOSFLMMissettingAngles.getPhiz().getValue()) 
        
        self.writeProcessFile( oedStringNewmatFileName, oedStringNewmat )
        
        
        
    def generateExecutiveSummary( self, _oedPlugin ):
        """
        Generates a summary of the execution of the plugin.
        This method is common to all MOSFLM plugins.
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01.generateExecutiveSummary" )
        if ( self.getStringVersion() is not None ):
            self.addExecutiveSummaryLine( self.getStringVersion() )
