#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginMOSFLMv01Indexing.py 475 2008-07-30 16:07:01Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDVerbose
from EDImport import EDString
from EDImport import EDUtilsTable
from EDImport import EDList

from EDPluginMOSFLMv01 import EDPluginMOSFLMv01

from XSDataCommonv01 import XSDataAngle
from XSDataCommonv01 import XSDataLength
from XSDataCommonv01 import XSDataInteger
from XSDataCommonv01 import XSDataFloat
from XSDataCommonv01 import XSDataString
from XSDataCommonv01 import XSDataCell

from XSDataMOSFLMv01 import XSDataMOSFLMBeamPosition
from XSDataMOSFLMv01 import XSDataMOSFLMInputIndexing
from XSDataMOSFLMv01 import XSDataMOSFLMIndexingSolution
from XSDataMOSFLMv01 import XSDataMOSFLMOutputIndexing

from XSDataDnaTables import dna_tables


class EDPluginMOSFLMv01Indexing( EDPluginMOSFLMv01 ):


    def __init__( self ):
        EDPluginMOSFLMv01.__init__( self )


    def preProcess( self, _oedObject = None ):
        EDPluginMOSFLMv01.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Indexing.preProcess")
        self.generateMOSFLMCommands()


    def postProcess( self, _oedObject = None ):
        EDPluginMOSFLMv01.postProcess( self )        
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Indexing.postProcess")
        xsDataMOSFLMOutputIndexing = self.createDataMOSFLMOutputIndexing()
        if ( xsDataMOSFLMOutputIndexing is not None ):
            self.setDataOutput( xsDataMOSFLMOutputIndexing )


    def configure( self ):
        EDPluginMOSFLMv01.configure( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Indexing.configure")   
        oxsPluginItem = self.getConfiguration()
        if ( oxsPluginItem == None ):
            EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Indexing.configure: oxsPluginItem is None")                 


    def setDataInput( self, _dataInput ):
        """
        Sets the Plugin Input Data
        _dataInput could be either an String XML or an XSData
        The XML String should be parsed as an Output Object by the final Plugin
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Indexing.setDataInput" )
        # Check the type
        if( isinstance( _dataInput, EDString ) ):
            EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Indexing.setDataInput: Input Data is of EDString " )
            self.m_oxsDataInput = XSDataMOSFLMInputIndexing.parseString( _dataInput )
        elif ( isinstance( _dataInput, XSDataMOSFLMInputIndexing ) ):
            self.m_oxsDataInput = _dataInput
        else:
            raise "Wrong input type for plugin EDPluginMOSFLMv01Indexing.setDataInput"


    def checkParameters( self ):
        """
        Checks the mandatory parameters for MOSLFM indexing
        """
        EDPluginMOSFLMv01.checkParameters( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Indexing.checkParameters")
        self.checkMandatoryParameters( self.getDataInput().getImage(), "image")
        
        
    def generateMOSFLMCommands( self ):
        """
        This method creates a list of MOSFLM indexing commands given a valid
        XSDataMOSFLMIndexingingInput as self.getDataInput()
        """
        EDPluginMOSFLMv01.generateMOSFLMCommands( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Indexing.generateMOSFLMCommands")

        oxsDataMOSFLMIndexingInput = self.getDataInput()

        if ( oxsDataMOSFLMIndexingInput != None):

            self.addListCommandExecution( "NEWMAT " + self.getNewmatFileName() )
            
            oxsDataMOSFLMImageList = oxsDataMOSFLMIndexingInput.getImage()
            for oxsDataMOSFLMImage in oxsDataMOSFLMImageList:
                iImageNumber = oxsDataMOSFLMImage.getNumber().getValue()
                fRotationAxisStart = oxsDataMOSFLMImage.getRotationAxisStart().getValue()
                fRotationAxisEnd = oxsDataMOSFLMImage.getRotationAxisEnd().getValue()
                self.addListCommandExecution( "AUTOINDEX DPS REFINE IMAGE " + EDString( iImageNumber ) + " " \
                                                     "PHI " + EDString( fRotationAxisStart ) + " " + EDString( fRotationAxisEnd ) )
            self.addListCommandExecution( "MOSAIC ESTIMATE" )
            self.addListCommandExecution( "GO" )
            
#            oedStringPrefix = oedStringTemplate.split("#")[0][:-1]
#            oedStringGenFileName = oedStringPrefix + ".gen"
#            oedStringSptFileName = oedStringPrefix + ".spt"
                        
        EDVerbose.DEBUG( "*** Finished EDPluginMOSFLMv01Indexing.generateMOSFLMIndexingCommands")

            
    def createDataMOSFLMOutputIndexing( self ):
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Indexing.createDataMOSFLMOutputIndexing")
        xsDataMOSFLMOutputIndexing = None
        # Read Newmat file
        xsDataMOSFLMNewmat = self.getDataMOSFLMNewmat()
        if ( xsDataMOSFLMNewmat is not None ):
            xsDataMOSFLMOutputIndexing = XSDataMOSFLMOutputIndexing()
            xsDataMOSFLMOutputIndexing.setRefinedNewmat( xsDataMOSFLMNewmat )
            # Then read the XML file
            oedStringDnaTablesXML = self.readProcessFile( self.getScriptBaseName() + "_dnaTables.xml" )
            xsDataDnaTables = dna_tables.parseString( oedStringDnaTablesXML )
            
            xsTableMosaicityEstimation = EDUtilsTable.getTableFromTables( xsDataDnaTables, "mosaicity_estimation" )
            xsListMosaicity            = EDUtilsTable.getListsFromTable( xsTableMosaicityEstimation, "mosaicity" )[0]
            dMosaicityValue            = EDString( EDUtilsTable.getItemFromList( xsListMosaicity, "value" ).getValueOf_() ).atof()
            xsDataFloatMosaicityEstimation = XSDataFloat()
            xsDataFloatMosaicityEstimation.setValue( dMosaicityValue )
            xsDataMOSFLMOutputIndexing.setMosaicityEstimation( xsDataFloatMosaicityEstimation )
            
            xsTableRefinement          = EDUtilsTable.getTableFromTables( xsDataDnaTables, "refinement" )
            xsListDeviations           = EDUtilsTable.getListsFromTable( xsTableRefinement, "deviations" )[0]
            dDeviationAngular          = EDString( EDUtilsTable.getItemFromList( xsListDeviations, "angular" ).getValueOf_() ).atof()
            dDeviationPositional       = EDString( EDUtilsTable.getItemFromList( xsListDeviations, "positional" ).getValueOf_() ).atof()
            xsDataMOSFLMOutputIndexing.setDeviationAngular( XSDataAngle( dDeviationAngular ) )
            xsDataMOSFLMOutputIndexing.setDeviationPositional( XSDataLength( dDeviationPositional ) )
            xsListResults              = EDUtilsTable.getListsFromTable( xsTableRefinement, "results" )[0]
            dDetectorDistance          = EDString( EDUtilsTable.getItemFromList( xsListResults, "detector_distance" ).getValueOf_() ).atof()
            xsDataMOSFLMOutputIndexing.setRefinedDistance( XSDataLength( dDetectorDistance ) )
            xsListParameters           = EDUtilsTable.getListsFromTable( xsTableRefinement, "parameters" )[0]
            iSpotsUsed                 = EDString( EDUtilsTable.getItemFromList( xsListParameters, "used" ).getValueOf_() ).atoi()
            iSpotsTotal                = EDString( EDUtilsTable.getItemFromList( xsListParameters, "out_of" ).getValueOf_() ).atoi()
            xsDataMOSFLMOutputIndexing.setSpotsUsed( XSDataInteger( iSpotsUsed ) )
            xsDataMOSFLMOutputIndexing.setSpotsTotal( XSDataInteger( iSpotsTotal ) )
            
            xsTableSolutionRefinement  = EDUtilsTable.getTableFromTables( xsDataDnaTables, "solution_refinement" )
            xsListParameters           = EDUtilsTable.getListsFromTable( xsTableSolutionRefinement, "selection" )[0]
            iSelectedSolutionNumber    = EDString( EDUtilsTable.getItemFromList( xsListParameters, "number" ).getValueOf_() ).atoi()
            xsDataMOSFLMOutputIndexing.setSelectedSolutionNumber( XSDataInteger( iSelectedSolutionNumber ) )
            oedStringSelectedSolutionSpaceGroup = EDString( EDUtilsTable.getItemFromList( xsListParameters, "spacegroup" ).getValueOf_() )
            xsDataMOSFLMOutputIndexing.setSelectedSolutionSpaceGroup( XSDataString( oedStringSelectedSolutionSpaceGroup ) )
            iSelectedSolutionSpaceGroupNumber = EDString( EDUtilsTable.getItemFromList( xsListParameters, "spacegroup_number" ).getValueOf_() ).atoi()
            xsDataMOSFLMOutputIndexing.setSelectedSolutionSpaceGroupNumber( XSDataInteger( iSelectedSolutionSpaceGroupNumber ) )
            
            xsTableAutoIndexSolutions  = EDUtilsTable.getTableFromTables( xsDataDnaTables, "autoindex_solutions" )
            xsListsSolution = xsTableAutoIndexSolutions.getList()
            for xsListSolution in xsListsSolution:
                xsDataMOSFLMIndexingSolution = XSDataMOSFLMIndexingSolution()
                xsDataCell = XSDataCell()
                iSolutionNumber    = EDString( EDUtilsTable.getItemFromList( xsListSolution, "index" ).getValueOf_() ).atoi()
                xsDataMOSFLMIndexingSolution.setIndex( XSDataInteger( iSolutionNumber ) )
                iPenalty           = EDString( EDUtilsTable.getItemFromList( xsListSolution, "penalty" ).getValueOf_() ).atoi()
                xsDataMOSFLMIndexingSolution.setPenalty( XSDataInteger( iPenalty ) )
                oedStringLattice   = EDString( EDUtilsTable.getItemFromList( xsListSolution, "lattice" ).getValueOf_() )
                xsDataMOSFLMIndexingSolution.setLattice( XSDataString( oedStringLattice ) )
                dA = EDString( EDUtilsTable.getItemFromList( xsListSolution, "a" ).getValueOf_() ).atof()
                xsDataCell.setLength_a( XSDataLength( dA ) )
                dB = EDString( EDUtilsTable.getItemFromList( xsListSolution, "b" ).getValueOf_() ).atof()
                xsDataCell.setLength_b( XSDataLength( dB ) )
                dC = EDString( EDUtilsTable.getItemFromList( xsListSolution, "c" ).getValueOf_() ).atof()
                xsDataCell.setLength_c( XSDataLength( dC ) )
                dAlpha = EDString( EDUtilsTable.getItemFromList( xsListSolution, "alpha" ).getValueOf_() ).atof()
                xsDataCell.setAngle_alpha( XSDataAngle( dAlpha ) )
                dBeta = EDString( EDUtilsTable.getItemFromList( xsListSolution, "beta" ).getValueOf_() ).atof()
                xsDataCell.setAngle_beta( XSDataAngle( dBeta ) )
                dGamma = EDString( EDUtilsTable.getItemFromList( xsListSolution, "gamma" ).getValueOf_() ).atof()
                xsDataCell.setAngle_gamma( XSDataAngle( dGamma ) )
                xsDataMOSFLMIndexingSolution.setCell( xsDataCell )
                xsDataMOSFLMOutputIndexing.addPossibleSolutions(xsDataMOSFLMIndexingSolution )
    
            xsTableBeamRefinement  = EDUtilsTable.getTableFromTables( xsDataDnaTables, "beam_refinement" )
            xsDataMOSFLMBeamPositionRefined = XSDataMOSFLMBeamPosition()
            xsDataMOSFLMBeamPositionShift   = XSDataMOSFLMBeamPosition()
            dInitialBeamX = 0.0
            dInitialBeamY = 0.0
            dRefinedBeamX = 0.0
            dRefinedBeamY = 0.0
            xsListInitialBeam        = EDUtilsTable.getListsFromTable( xsTableBeamRefinement, "initial_beam" )[0]
            fInitialBeamPositionX    = EDString( EDUtilsTable.getItemFromList( xsListInitialBeam, "x" ).getValueOf_() ).atof()
            fInitialBeamPositionY    = EDString( EDUtilsTable.getItemFromList( xsListInitialBeam, "y" ).getValueOf_() ).atof()
            xsListRefinedBeam        = EDUtilsTable.getListsFromTable( xsTableBeamRefinement, "refined_beam" )[0]
            fRefinedBeamPositionX    = EDString( EDUtilsTable.getItemFromList( xsListRefinedBeam, "x" ).getValueOf_() ).atof()
            fRefinedBeamPositionY    = EDString( EDUtilsTable.getItemFromList( xsListRefinedBeam, "y" ).getValueOf_() ).atof()
            xsDataMOSFLMBeamPositionRefined.setX( XSDataLength( fRefinedBeamPositionX ) )
            xsDataMOSFLMBeamPositionRefined.setY( XSDataLength( fRefinedBeamPositionY ) )
            xsDataMOSFLMBeamPositionShift.setX( XSDataLength( fInitialBeamPositionX - fRefinedBeamPositionX ) )
            xsDataMOSFLMBeamPositionShift.setY( XSDataLength( fInitialBeamPositionY - fRefinedBeamPositionY ) )
            xsDataMOSFLMOutputIndexing.setRefinedBeam( xsDataMOSFLMBeamPositionRefined )
            xsDataMOSFLMOutputIndexing.setBeamShift( xsDataMOSFLMBeamPositionShift )

        return xsDataMOSFLMOutputIndexing
    
    
    def generateExecutiveSummary( self, _oedPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDPluginMOSFLMv01.generateExecutiveSummary( self, _oedPlugin )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Indexing.createDataMOSFLMOutputIndexing")
        xsDataMOSFLMInputIndexing  = self.getDataInput()
        xsDataMOSFLMOutputIndexing = self.getDataOutput()
        if ( self.getErrorMessages().getNumberObjects() == 0 ):
            self.addExecutiveSummaryLine( "Execution of MOSFLM indexing successful." )
        self.addExecutiveSummaryLine( "Image directory         : %s" %  xsDataMOSFLMInputIndexing.getDirectory().getValue() )
        self.addExecutiveSummaryLine( "Image template          : %s" %  xsDataMOSFLMInputIndexing.getTemplate().getValue() )
        oedStringImagesUsed         = "Images used in indexing : "
        for xsDataMOSFLMImage in xsDataMOSFLMInputIndexing.getImage():
            oedStringImagesUsed += "%3d" % xsDataMOSFLMImage.getNumber().getValue()
        self.addExecutiveSummaryLine( oedStringImagesUsed )
        if ( xsDataMOSFLMInputIndexing.getSymmetry() is not None ):
            self.addExecutiveSummaryLine( "Target symmetry     : %s" % xsDataMOSFLMInputIndexing.getSymmetry().getValue() )
            self.addExecutiveSummaryLine( "" )
        if ( xsDataMOSFLMOutputIndexing is not None ):
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "List of possible solutions (index, penalty, lattice and cell):" )
            iSelectedSolutionNumber = xsDataMOSFLMOutputIndexing.getSelectedSolutionNumber().getValue()
            # Add all solutions with penalty < 100 + 1 solution with penalty > 100
            bAddSolution = False
            edListMOSFLMOutputIndexing = EDList( xsDataMOSFLMOutputIndexing.getPossibleSolutions() )
            iNumberOfSolutions = edListMOSFLMOutputIndexing.getNumberObjects()
            for i in range( iNumberOfSolutions ):
                    
                iPenalty         = edListMOSFLMOutputIndexing[i].getPenalty().getValue() 
                if ( i < ( iNumberOfSolutions - 1 ) ):
                    iPenaltyNext = edListMOSFLMOutputIndexing[i+1].getPenalty().getValue() 
                    if ( ( iPenalty >= 100 ) and ( iPenaltyNext <= 100 ) ):
                         bAddSolution = True
                if ( bAddSolution ):                
                    iIndex           = edListMOSFLMOutputIndexing[i].getIndex().getValue()
                    xsDataCell       = edListMOSFLMOutputIndexing[i].getCell()
                    oedStringLattice = EDString( edListMOSFLMOutputIndexing[i].getLattice().getValue() )
                    oedStringPossibleSolution = EDString( "%3d %4d %2s %6.2f %6.2f %6.2f %6.2f %6.2f %6.2f" % \
                                                ( iIndex, iPenalty, oedStringLattice, \
                                                  xsDataCell.getLength_a().getValue(), 
                                                  xsDataCell.getLength_b().getValue(), 
                                                  xsDataCell.getLength_c().getValue(), 
                                                  xsDataCell.getAngle_alpha().getValue(), 
                                                  xsDataCell.getAngle_beta().getValue(), 
                                                  xsDataCell.getAngle_gamma().getValue(), 
                                                 ) )
                    self.addExecutiveSummaryLine( oedStringPossibleSolution )
                iPenaltyOld = iPenalty
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "Choosen solution number   : %14d" % iSelectedSolutionNumber  )
            oedStringSelectedSpaceGroup = EDString( xsDataMOSFLMOutputIndexing.getSelectedSolutionSpaceGroup().getValue() )
            self.addExecutiveSummaryLine( "Selected space group      : %14s" % ( oedStringSelectedSpaceGroup ) )
            xsDataCellRefined = xsDataMOSFLMOutputIndexing.getRefinedNewmat().getRefinedCell()
            self.addExecutiveSummaryLine( "Refined cell              : %6.2f %7.2f %7.2f %5.1f %5.1f %5.1f" % ( \
                                          xsDataCellRefined.getLength_a().getValue(), 
                                          xsDataCellRefined.getLength_b().getValue(), 
                                          xsDataCellRefined.getLength_c().getValue(), 
                                          xsDataCellRefined.getAngle_alpha().getValue(), 
                                          xsDataCellRefined.getAngle_beta().getValue(), 
                                          xsDataCellRefined.getAngle_gamma().getValue() 
                                          ) )
            iSpotsTotal = xsDataMOSFLMOutputIndexing.getSpotsTotal().getValue()
            iSpotsUsed  = xsDataMOSFLMOutputIndexing.getSpotsUsed().getValue()
            self.addExecutiveSummaryLine( "Number of spots used      : %14d " % ( iSpotsUsed ) )
            self.addExecutiveSummaryLine( "Number of spots total     : %14d " % ( iSpotsTotal ) )
            fDeviationPositional = xsDataMOSFLMOutputIndexing.getDeviationPositional().getValue()
            fDeviationAngular    = xsDataMOSFLMOutputIndexing.getDeviationAngular().getValue()
            self.addExecutiveSummaryLine( "Spot deviation positional : %14.2f [mm]" % ( fDeviationPositional ) ) 
            self.addExecutiveSummaryLine( "Spot deviation angular    : %14.2f [degrees]" % ( fDeviationAngular ) ) 
            fBeamshiftX = xsDataMOSFLMOutputIndexing.getBeamShift().getX().getValue()
            fBeamshiftY = xsDataMOSFLMOutputIndexing.getBeamShift().getY().getValue()
            self.addExecutiveSummaryLine( "Beam shift (X, Y)         : %6.3f, %6.3f [mm]" % \
                                          ( fBeamshiftX, fBeamshiftY ) )
            fMosaicityEstimated = xsDataMOSFLMOutputIndexing.getMosaicityEstimation().getValue()
            self.addExecutiveSummaryLine( "Estimated mosaicity       : %14.2f [degrees]" % fMosaicityEstimated )
            