#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginMOSFLMv01Integration.py 475 2008-07-30 16:07:01Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDVerbose
from EDImport import EDString
from EDImport import EDUtilsTable
from EDImport import EDMessage
from EDImport import EDDiskExplorer
 
from EDPluginMOSFLMv01 import EDPluginMOSFLMv01

from XSDataMOSFLMv01 import XSDataMOSFLMInputIntegration
from XSDataMOSFLMv01 import XSDataMOSFLMOutputIntegration
from XSDataMOSFLMv01 import XSDataMOSFLMBeamPosition
from XSDataMOSFLMv01 import XSDataMOSFLMIntegrationStatistics
from XSDataMOSFLMv01 import XSDataMOSFLMIntegrationStatisticsPerReflectionType
from XSDataMOSFLMv01 import XSDataMOSFLMIntegrationStatisticsPerResolutionBin

from XSDataDnaTables import dna_tables

from XSDataCommonv01 import XSDataAngle
from XSDataCommonv01 import XSDataLength
from XSDataCommonv01 import XSDataInteger
from XSDataCommonv01 import XSDataFloat
from XSDataCommonv01 import XSDataString
from XSDataCommonv01 import XSDataCell
from XSDataCommonv01 import XSDataFile


class EDPluginMOSFLMv01Integration( EDPluginMOSFLMv01 ):


    def __init__( self ):
        EDPluginMOSFLMv01.__init__( self )


    def process( self, _oedObject = None ):
        EDPluginMOSFLMv01.process( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Integration.process")

    
    def preProcess( self, _oedObject = None ):
        EDPluginMOSFLMv01.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Integration.preProcess")
        self.generateMOSFLMCommands()


    def postProcess( self, _oedObject = None ):
        EDPluginMOSFLMv01.postProcess( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Integration.postProcess")
        xsDataMOSFLMOutputIntegration = self.createDataMOSFLMOutputIntegration()
        if ( xsDataMOSFLMOutputIntegration is not None ):
            self.setDataOutput( xsDataMOSFLMOutputIntegration )


    def configure( self ):
        EDPluginMOSFLMv01.configure( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Integration.configure")   
        oxsPluginItem = self.getConfiguration()
        if ( oxsPluginItem == None ):
            EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Integration.configure: oxsPluginItem is None")                 


    def setDataInput( self, _dataInput ):
        """
        Sets the Plugin Input Data
        _dataInput could be either an String XML or an XSData
        The XML String should be parsed as an Output Object by the final Plugin
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Integration.setDataInput" )
        # Check the type
        if( isinstance( _dataInput, EDString ) ):
            EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Integration.setDataInput: Input Data is of EDString " )
            self.m_oxsDataInput = XSDataMOSFLMInputIntegration.parseString( _dataInput )
        elif ( isinstance( _dataInput, XSDataMOSFLMInputIntegration ) ):
            self.m_oxsDataInput = _dataInput
        else:
            raise "Wrong input type for plugin EDPluginMOSFLMv01Integration.setDataInput"


    def checkParameters( self ):
        """
        Checks the mandatory parameters for MOSLFM indexing
        """
        EDPluginMOSFLMv01.checkParameters( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Integration.checkParameters")
        self.checkMandatoryParameters( self.getDataInput().getImageEnd(),          "imageEnd" )
        self.checkMandatoryParameters( self.getDataInput().getImageStart(),        "imageStart" )
        self.checkMandatoryParameters( self.getDataInput().getOscillationWidth(),  "oscillationWidth" )
        self.checkMandatoryParameters( self.getDataInput().getRotationAxisStart(), "rotationAxisStart" ) 
        
        
    def generateMOSFLMCommands( self ):
        """
        This method creates a list of MOSFLM integration commands given a valid
        XSDataMOSFLMIntegrationingInput as self.getDataInput()
        """
        EDPluginMOSFLMv01.generateMOSFLMCommands( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Integration.generateMOSFLMCommands")

        oxsDataMOSFLMInputIntegration = self.getDataInput()

        if ( oxsDataMOSFLMInputIntegration is not None):
            
                        
            iImageStart = oxsDataMOSFLMInputIntegration.getImageStart().getValue()
            iImageEnd   = oxsDataMOSFLMInputIntegration.getImageEnd().getValue()
            fRotationAxisStart = oxsDataMOSFLMInputIntegration.getRotationAxisStart().getValue()
            fOscillationWidth = oxsDataMOSFLMInputIntegration.getOscillationWidth().getValue()

            self.addListCommandExecution( "HKLOUT %s_process_%d_%d.mtz" % 
                                                       ( self.getScriptBaseName(), iImageStart, iImageEnd ) )
            self.addListCommandExecution( "PROCESS %d TO %d START %f ANGLE %f" %
                                                       ( iImageStart, iImageEnd, fRotationAxisStart, fOscillationWidth ) )
                                                

            self.addListCommandExecution( "BEST ON" )
            self.addListCommandExecution( "GO" )
            self.addListCommandExecution( "BEST OFF" )
            
        EDVerbose.DEBUG( "*** Finished EDPluginMOSFLMv01Integration.generateMOSFLMCommands")

            
    def createDataMOSFLMOutputIntegration( self ):
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Integration.createDataMOSFLMOutputIntegration")
        oxsDataMOSFLMOutputIntegration = XSDataMOSFLMOutputIntegration()
        # Read bestfile.par, bestfile.hkl and bestfile.dat
        oedStringBestfilePar = self.readBestFile( "bestfile.par" )
        if ( oedStringBestfilePar is not None ):
            oxsDataMOSFLMOutputIntegration.setBestfilePar( XSDataString( oedStringBestfilePar ) )
        else:
            oxsDataMOSFLMOutputIntegration = None              
          
        if ( oxsDataMOSFLMOutputIntegration is not None ):
            oedStringBestfileHKL = self.readBestFile( "bestfile.hkl" )
            if ( oedStringBestfileHKL is not None ):
                oxsDataMOSFLMOutputIntegration.setBestfileHKL( XSDataString( oedStringBestfileHKL ) )
            else:
                oxsDataMOSFLMOutputIntegration = None              

        if ( oxsDataMOSFLMOutputIntegration is not None ):
            oedStringBestfileDat = self.readBestFile( "bestfile.dat" )
            if ( oedStringBestfileDat is not None ):
                oxsDataMOSFLMOutputIntegration.setBestfileDat( XSDataString( oedStringBestfileDat ) )
            else:
                oxsDataMOSFLMOutputIntegration = None              
          
        if ( oxsDataMOSFLMOutputIntegration is not None ):        
            oedStringDnaTablesXML = self.readProcessFile( self.getBaseName() + "_dnaTables.xml" )
            xsDataDnaTables = dna_tables.parseString( oedStringDnaTablesXML )
            
            xsTableIntegrationResults = EDUtilsTable.getTableFromTables( xsDataDnaTables, "integration_results" )
            xsListFinalResiduals      = EDUtilsTable.getListsFromTable( xsTableIntegrationResults, "final_residuals" )[0]
            oedStringRMSSpotDeviation = EDString( EDUtilsTable.getItemFromList( xsListFinalResiduals, "rms" ).getValueOf_() )
            oxsDataMOSFLMOutputIntegration.setRMSSpotDeviation( XSDataFloat( oedStringRMSSpotDeviation.atof() ) )
            oedStringBeamPositionX = EDString( EDUtilsTable.getItemFromList( xsListFinalResiduals, "xcen" ).getValueOf_() )
            oedStringBeamPositionY = EDString( EDUtilsTable.getItemFromList( xsListFinalResiduals, "ycen" ).getValueOf_() )
            xsDataMOSFLMBeamPosition = XSDataMOSFLMBeamPosition()
            xsDataMOSFLMBeamPosition.setX( XSDataLength( oedStringBeamPositionX.atof() ) )
            xsDataMOSFLMBeamPosition.setY( XSDataLength( oedStringBeamPositionY.atof() ) )
            oxsDataMOSFLMOutputIntegration.setRefinedBeam( xsDataMOSFLMBeamPosition )
            
            xsDataMOSFLMNewmatMatrix = self.getDataMOSFLMMatrix()
            oxsDataMOSFLMOutputIntegration.setRefinedNewmat( xsDataMOSFLMNewmatMatrix )
            
            # New results (described in bug #63)
            
            xsTableIntegrationOutput = EDUtilsTable.getTableFromTables( xsDataDnaTables, "integration_output" )
            xsListOutputFiles = EDUtilsTable.getListsFromTable( xsTableIntegrationOutput, "output_files" )[0]
            
            oedStringMTZFilename = EDString( EDUtilsTable.getItemFromList( xsListOutputFiles, "hklout" ).getValueOf_() )
            oedStringMTZPath = EDDiskExplorer.mergePath( self.getWorkingDirectory(), oedStringMTZFilename )
            xsDataFile = XSDataFile()
            xsDataFile.setPath( XSDataString( oedStringMTZPath ))
            oxsDataMOSFLMOutputIntegration.setGeneratedMTZFile( xsDataFile )
            
            xsTableSummaryInformation = EDUtilsTable.getTableFromTables( xsDataDnaTables, "summary_information" )
            xsListSummary = EDUtilsTable.getListsFromTable( xsTableSummaryInformation, "summary" )[0]
            xsListSpots = EDUtilsTable.getListsFromTable( xsTableSummaryInformation, "spots" )[0]
            
            oedStringDistance = EDString( EDUtilsTable.getItemFromList( xsListSummary, "distance" ).getValueOf_() )
            oxsDataMOSFLMOutputIntegration.setRefinedDistance( XSDataLength( oedStringDistance.atof() ) )
            
            oedStringYScale = EDString( EDUtilsTable.getItemFromList( xsListSummary, "yscale" ).getValueOf_() )
            oxsDataMOSFLMOutputIntegration.setRefinedYScale( XSDataFloat( oedStringYScale.atof() ) )
            
            oedStringOverallIOverSigma = EDString( EDUtilsTable.getItemFromList( xsListSummary, "isigall" ).getValueOf_() )
            oxsDataMOSFLMOutputIntegration.setOverallIOverSigma( XSDataFloat( oedStringOverallIOverSigma.atof() ) )
            
            oedStringHighestResolutionIOverSigma = EDString( EDUtilsTable.getItemFromList( xsListSummary, "isigout" ).getValueOf_() )
            oxsDataMOSFLMOutputIntegration.setHighestResolutionIOverSigma( XSDataFloat( oedStringHighestResolutionIOverSigma.atof() ) )
            
            oedStringNumberOfBadReflections = EDString( EDUtilsTable.getItemFromList( xsListSpots, "bad_spots" ).getValueOf_() )
            oxsDataMOSFLMOutputIntegration.setNumberOfBadReflections( XSDataInteger( oedStringNumberOfBadReflections.atoi() ) )

            oedStringNumberOfFullyRecordedReflections = EDString( EDUtilsTable.getItemFromList( xsListSpots, "full" ).getValueOf_() )
            oxsDataMOSFLMOutputIntegration.setNumberOfFullyRecordedReflections( XSDataInteger( oedStringNumberOfFullyRecordedReflections.atoi() ) )

            oedStringNumberOfNegativeReflections = EDString( EDUtilsTable.getItemFromList( xsListSpots, "negative" ).getValueOf_() )
            oxsDataMOSFLMOutputIntegration.setNumberOfNegativeReflections( XSDataInteger( oedStringNumberOfNegativeReflections.atoi() ) )
            
            oedStringNumberOfOverlappedReflections = EDString( EDUtilsTable.getItemFromList( xsListSpots, "overlap" ).getValueOf_() )
            oxsDataMOSFLMOutputIntegration.setNumberOfOverlappedReflections( XSDataInteger( oedStringNumberOfOverlappedReflections.atoi() ) )

            oedStringNumberOfPartialReflections = EDString( EDUtilsTable.getItemFromList( xsListSpots, "partial" ).getValueOf_() )
            oxsDataMOSFLMOutputIntegration.setNumberOfPartialReflections( XSDataInteger( oedStringNumberOfPartialReflections.atoi() ) )

            # Fill in the statistics
            
            oxsDataMOSFLMOutputIntegration.setOverallStatistics( self.getMOSFLMIntegrationStatisticsPerResolutionBin( xsDataDnaTables, "bin_0" ) )
            
            iIndex = 1            
            bContinue = True
            fMinResolution = None
            while ( bContinue == True ):
                oedStringBin = "bin_%d" % iIndex
                # I don't know how many intensity bins there are. Since the EDUtilsTable.getListsFromTable
                # method crashes if the list is not present, I had to wrap the calls to this method in
                # try - except:
                try:
                    xsDataMOSFLMIntegrationStatisticsPerResolutionBin = self.getMOSFLMIntegrationStatisticsPerResolutionBin( xsDataDnaTables, oedStringBin, fMinResolution )
                    fMinResolution =  xsDataMOSFLMIntegrationStatisticsPerResolutionBin.getMaxResolution().getValue()
                    oxsDataMOSFLMOutputIntegration.addStatisticsPerResolutionBin( xsDataMOSFLMIntegrationStatisticsPerResolutionBin )
                    iIndex += 1
                except:
                    bContinue = False

        return oxsDataMOSFLMOutputIntegration
    
    
    def readBestFile( self, _oedStringBestfileName ):
        oedStringBestfile = None
        try:
            oedStringBestfile = self.readProcessFile( _oedStringBestfileName )
        except:    
            oedStringError = self.readProcessErrorLogFile()
            if( (oedStringError is not None) and ( oedStringError != EDString.empty() ) ):
                errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginMOSFLMv01Integration.readBestFile', 'EDPluginMOSFLMv01Integration', oedStringError )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )
            else:
                errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ( "EDPluginMOSFLMv01Integration.readBestFile", \
                                                                   "Cannot read MOSFLM " + _oedStringBestfileName + " file" )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )
        return oedStringBestfile
    
    
    def getMOSFLMIntegrationStatisticsPerResolutionBin( self, _xsDataDnaTables, _oedStringListName, _fMinResolution = None ):
        """
        This method creates an XSDataMOSFLMIntegrationStatisticsPerResolutionBin object given an XSDataDnaTables object
        and a list name, e.g. "bin_1". If _fMinResolution is provided it is used for setting the minimum resolution,
        otherwise the min and max resolution are set to the same value provided by the xsDataList object.
        """
        xsDataTableProfileFittedFull    = EDUtilsTable.getTableFromTables( _xsDataDnaTables, "profile_fitted_full" )
        xsDataTableProfileFittedPartial = EDUtilsTable.getTableFromTables( _xsDataDnaTables, "profile_fitted_partial" )
        xsDataTableSummationFull        = EDUtilsTable.getTableFromTables( _xsDataDnaTables, "summation_full" )
        xsDataTableSummationPartial     = EDUtilsTable.getTableFromTables( _xsDataDnaTables, "summation_partial" )

        xsListProfileFittedFull    = EDUtilsTable.getListsFromTable( xsDataTableProfileFittedFull, _oedStringListName )[0]
        xsListProfileFittedPartial = EDUtilsTable.getListsFromTable( xsDataTableProfileFittedPartial, _oedStringListName )[0]
        xsListSummationFull        = EDUtilsTable.getListsFromTable( xsDataTableSummationFull, _oedStringListName )[0]
        xsListSummationPartial     = EDUtilsTable.getListsFromTable( xsDataTableSummationPartial, _oedStringListName )[0]
        
        xsDataMOSFLMIntegrationDataProfileFittedFull     = self.getMOSFLMIntegrationStatistics( xsListProfileFittedFull )
        xsDataMOSFLMIntegrationDataProfileFittedPartials = self.getMOSFLMIntegrationStatistics( xsListProfileFittedPartial )
        xsDataMOSFLMIntegrationSummationFull             = self.getMOSFLMIntegrationStatistics( xsListSummationFull )
        xsDataMOSFLMIntegrationSummationPartial          = self.getMOSFLMIntegrationStatistics( xsListSummationPartial )

        xsDataMOSFLMIntegrationStatisticsPerReflectionTypeProfileFitted = XSDataMOSFLMIntegrationStatisticsPerReflectionType()
        xsDataMOSFLMIntegrationStatisticsPerReflectionTypeProfileFitted.setFullyRecorded( xsDataMOSFLMIntegrationDataProfileFittedFull )
        xsDataMOSFLMIntegrationStatisticsPerReflectionTypeProfileFitted.setPartials( xsDataMOSFLMIntegrationDataProfileFittedPartials )
        
        xsDataMOSFLMIntegrationStatisticsPerReflectionTypeSummation = XSDataMOSFLMIntegrationStatisticsPerReflectionType()
        xsDataMOSFLMIntegrationStatisticsPerReflectionTypeSummation.setFullyRecorded( xsDataMOSFLMIntegrationSummationFull )
        xsDataMOSFLMIntegrationStatisticsPerReflectionTypeSummation.setPartials( xsDataMOSFLMIntegrationSummationPartial )
        
        xsDataMOSFLMIntegrationStatisticsPerResolutionBin = XSDataMOSFLMIntegrationStatisticsPerResolutionBin()
        xsDataMOSFLMIntegrationStatisticsPerResolutionBin.setProfileFitted( xsDataMOSFLMIntegrationStatisticsPerReflectionTypeProfileFitted )
        xsDataMOSFLMIntegrationStatisticsPerResolutionBin.setSummation( xsDataMOSFLMIntegrationStatisticsPerReflectionTypeSummation )

        oedStringMaxResolution = EDString( EDUtilsTable.getItemFromList( xsListProfileFittedFull, "resolution" ).getValueOf_() )
        if ( not oedStringMaxResolution.equals( "overall" ) ):
            xsDataMOSFLMIntegrationStatisticsPerResolutionBin.setMaxResolution( XSDataFloat( oedStringMaxResolution.atof() ) )
            if ( _fMinResolution is not None ):
                xsDataMOSFLMIntegrationStatisticsPerResolutionBin.setMinResolution( XSDataFloat( _fMinResolution ) )
            else:
                xsDataMOSFLMIntegrationStatisticsPerResolutionBin.setMinResolution( XSDataFloat( oedStringMaxResolution.atof() ) )

        return xsDataMOSFLMIntegrationStatisticsPerResolutionBin
    
    

    def getMOSFLMIntegrationStatistics( self, _xsDataList ):
        """
        This method creates a new XSDataMOSFLMIntegrationStatistics object and populates it with
        values (averaged intensity, sigma, average I over sigma and number of reflections) 
        from the _xsDataList object.
        """
        xsDataMOSFLMIntegrationStatistics = XSDataMOSFLMIntegrationStatistics()

        oedStringAverageIntensity = EDString( EDUtilsTable.getItemFromList( _xsDataList, "i" ).getValueOf_() )
        xsDataMOSFLMIntegrationStatistics.setAverageIntensity( XSDataFloat( oedStringAverageIntensity.atof() ) )

        oedStringAverageSigma = EDString( EDUtilsTable.getItemFromList( _xsDataList, "sig" ).getValueOf_() )
        xsDataMOSFLMIntegrationStatistics.setAverageSigma( XSDataFloat( oedStringAverageSigma.atof() ) )

        oedStringAverageIOverSigma = EDString( EDUtilsTable.getItemFromList( _xsDataList, "i_sig" ).getValueOf_() )
        xsDataMOSFLMIntegrationStatistics.setAverageIOverSigma( XSDataFloat( oedStringAverageIOverSigma.atof() ) )

        oedStringNumberOfReflections = EDString( EDUtilsTable.getItemFromList( _xsDataList, "count" ).getValueOf_() )
        xsDataMOSFLMIntegrationStatistics.setNumberOfReflections( XSDataInteger( oedStringNumberOfReflections.atoi() ) )

        return xsDataMOSFLMIntegrationStatistics
            
    
    def generateExecutiveSummary( self, _oedPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDPluginMOSFLMv01.generateExecutiveSummary( self, _oedPlugin )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv01Integration.createDataMOSFLMOutputIndexing")
        xsDataMOSFLMInputIntegration  = self.getDataInput()
        xsDataMOSFLMOutputIntegration = self.getDataOutput()
        if ( xsDataMOSFLMOutputIntegration is not None):
            self.addExecutiveSummaryLine( "Execution of MOSFLM integration successful." )
            self.addExecutiveSummaryLine( "Image directory     : %s" %  xsDataMOSFLMInputIntegration.getDirectory().getValue() )
            self.addExecutiveSummaryLine( "Image template      : %s" %  xsDataMOSFLMInputIntegration.getTemplate().getValue() )
            self.addExecutiveSummaryLine( "Image start         : %4d" %  xsDataMOSFLMInputIntegration.getImageStart().getValue() )
            self.addExecutiveSummaryLine( "Image end           : %4d" %  xsDataMOSFLMInputIntegration.getImageEnd().getValue() )
            fRotationAxisStart = xsDataMOSFLMInputIntegration.getRotationAxisStart().getValue()
            fOscillationWidth  = xsDataMOSFLMInputIntegration.getOscillationWidth().getValue()
            self.addExecutiveSummaryLine( "Rotation axis start : %4.1f [degrees]" % fRotationAxisStart )
            self.addExecutiveSummaryLine( "Rotation axis end   : %4.1f [degrees]" % ( fRotationAxisStart + fOscillationWidth ) )
            self.addExecutiveSummaryLine( "" )
            iNumberOfFullyRecordedReflections =  xsDataMOSFLMOutputIntegration.getNumberOfFullyRecordedReflections().getValue()
            self.addExecutiveSummaryLine( "Number of fully recorded reflections          : %5d" % iNumberOfFullyRecordedReflections )
            self.addExecutiveSummaryLine( "Number of partials                            : %5d" % xsDataMOSFLMOutputIntegration.getNumberOfPartialReflections().getValue() )
            self.addExecutiveSummaryLine( "Number of overlapped reflections              : %5d" % xsDataMOSFLMOutputIntegration.getNumberOfOverlappedReflections().getValue() )
            self.addExecutiveSummaryLine( "Number of reflections with negative intensity : %5d" % xsDataMOSFLMOutputIntegration.getNumberOfNegativeReflections().getValue() )
            self.addExecutiveSummaryLine( "Number of bad reflections                     : %5d" % xsDataMOSFLMOutputIntegration.getNumberOfBadReflections().getValue() )
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "RMS spot deviation                            : %5.3f [mm]" % xsDataMOSFLMOutputIntegration.getRMSSpotDeviation().getValue() )
            self.addExecutiveSummaryLine( "Average I/sigma overall                       : %5.1f" % xsDataMOSFLMOutputIntegration.getOverallIOverSigma().getValue() )
            self.addExecutiveSummaryLine( "Average I/sigma at highest resolution         : %5.1f" % xsDataMOSFLMOutputIntegration.getHighestResolutionIOverSigma().getValue() )
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "Analysis as a function of resolution:" )
            oedStringResolution                                  = "Res (Ang) "
    
            oedStringNumberProfileFittedFullyRecorded            = "Number    "
            oedStringAverageIntensityProfileFittedFullyRecorded  = " <I>      "
            oedStringAverageIOverSigmaProfileFittedFullyRecorded = " <I/sigma>"
    
            oedStringNumberProfileFittedPartials            = "Number    "
            oedStringAverageIntensityProfileFittedPartials  = " <I>      "
            oedStringAverageIOverSigmaProfileFittedPartials = " <I/sigma>"
              
            for xsDataMOSFLMIntegrationStatisticsPerResolutionBin in xsDataMOSFLMOutputIntegration.getStatisticsPerResolutionBin():
                fMaxResolution = xsDataMOSFLMIntegrationStatisticsPerResolutionBin.getMaxResolution().getValue()
                oedStringResolution += "%7.2f" % fMaxResolution
                xsDataMOSFLMIntegrationStatisticsProfileFitted             = xsDataMOSFLMIntegrationStatisticsPerResolutionBin.getProfileFitted()
                
                xsDataMOSFLMIntegrationStatisticsProfileFittedFullyRecorded = xsDataMOSFLMIntegrationStatisticsProfileFitted.getFullyRecorded()
                oedStringNumberProfileFittedFullyRecorded            += "%7d" % xsDataMOSFLMIntegrationStatisticsProfileFittedFullyRecorded.getNumberOfReflections().getValue()
                oedStringAverageIntensityProfileFittedFullyRecorded  += "%7.0f" % xsDataMOSFLMIntegrationStatisticsProfileFittedFullyRecorded.getAverageIntensity().getValue()
                oedStringAverageIOverSigmaProfileFittedFullyRecorded += "%7.1f" % xsDataMOSFLMIntegrationStatisticsProfileFittedFullyRecorded.getAverageIOverSigma().getValue()
                
                xsDataMOSFLMIntegrationStatisticsProfileFittedPartials      = xsDataMOSFLMIntegrationStatisticsProfileFitted.getPartials()
                oedStringNumberProfileFittedPartials            += "%7d" % xsDataMOSFLMIntegrationStatisticsProfileFittedPartials.getNumberOfReflections().getValue()
                oedStringAverageIntensityProfileFittedPartials  += "%7.0f" % xsDataMOSFLMIntegrationStatisticsProfileFittedPartials.getAverageIntensity().getValue()
                oedStringAverageIOverSigmaProfileFittedPartials += "%7.1f" % xsDataMOSFLMIntegrationStatisticsProfileFittedPartials.getAverageIOverSigma().getValue()
                        
            oedStringResolution += " Overall"
            self.addExecutiveSummaryLine( oedStringResolution )
            
            oedStringNumberProfileFittedFullyRecorded            += "%8d"   % xsDataMOSFLMOutputIntegration.getOverallStatistics().getProfileFitted().getFullyRecorded().getNumberOfReflections().getValue()
            oedStringAverageIntensityProfileFittedFullyRecorded  += "%8.0f" % xsDataMOSFLMOutputIntegration.getOverallStatistics().getProfileFitted().getFullyRecorded().getAverageIntensity().getValue()
            oedStringAverageIOverSigmaProfileFittedFullyRecorded += "%8.1f" % xsDataMOSFLMOutputIntegration.getOverallStatistics().getProfileFitted().getFullyRecorded().getAverageIOverSigma().getValue()
    
            oedStringNumberProfileFittedPartials            += "%8d"   % xsDataMOSFLMOutputIntegration.getOverallStatistics().getProfileFitted().getPartials().getNumberOfReflections().getValue()
            oedStringAverageIntensityProfileFittedPartials  += "%8.0f" % xsDataMOSFLMOutputIntegration.getOverallStatistics().getProfileFitted().getPartials().getAverageIntensity().getValue()
            oedStringAverageIOverSigmaProfileFittedPartials += "%8.1f" % xsDataMOSFLMOutputIntegration.getOverallStatistics().getProfileFitted().getPartials().getAverageIOverSigma().getValue()
            
            if ( iNumberOfFullyRecordedReflections > 0 ):
                self.addExecutiveSummaryLine( "Profile fitted fully recorded:" )
                self.addExecutiveSummaryLine( oedStringNumberProfileFittedFullyRecorded )
                self.addExecutiveSummaryLine( oedStringAverageIntensityProfileFittedFullyRecorded )
                self.addExecutiveSummaryLine( oedStringAverageIOverSigmaProfileFittedFullyRecorded )
            
            self.addExecutiveSummaryLine( "Profile fitted partials:" )
            self.addExecutiveSummaryLine( oedStringNumberProfileFittedPartials )
            self.addExecutiveSummaryLine( oedStringAverageIntensityProfileFittedPartials )
            #self.addExecutiveSummaryLine( oedStringAverageSigmaProfileFittedPartials )
            self.addExecutiveSummaryLine( oedStringAverageIOverSigmaProfileFittedPartials )
                    
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "Path to MTZ file : %s" % xsDataMOSFLMOutputIntegration.getGeneratedMTZFile().getPath().getValue() )