#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDApplicationPrototype.py 477 2008-07-31 08:44:38Z incardon $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDApplication
from EDImport import EDVerbose
from EDImport import EDMessage
from EDImport import EDConfiguration
from EDImport import EDCompiler
from EDImport import EDString
from EDImport import EDList
from EDImport import EDDiskExplorer
from EDImport import EDTime
from EDImport import PyOs

from XSDataCommonv01 import XSDataFile
from XSDataCommonv01 import XSDataString
from XSDataCommonv01 import XSDataTime
from XSDataCommonv01 import XSDataFloat
from XSDataCommonv01 import XSDataDiffractionPlan
from XSDataCommonv01 import XSDataSize
from XSDataCommonv01 import XSDataLength

from XSDataCharacterisationv01 import XSDataCollection


class EDApplicationPrototype( EDApplication ):

    m_oedStrAPPLICATION_NAME              = "EDApplicationPrototype"
    m_oedStrPROTOTYPE_VERSION             = "1.0.0"
    m_oedStrIMAGE_PARAM_LABEL             = "--image"
    m_oedStrDATASET_PARAM_LABEL           = "--data"
    m_oedStrFLUX_PARAM_LABEL              = "--flux"
    m_oedStrBEAM_SIZE_PARAM_LABEL         = "--beamSize"
    m_oedStrMAX_EXPOSURE_TIME_PARAM_LABEL = "--maxExposureTimePerDataCollection"
    m_oedStrMIN_EXPOSURE_TIME_PARAM_LABEL = "--minExposureTimePerImage"
    m_oedStrDEBUG_PARAM_LABEL             = "--DEBUG"
    m_oedStrVERSION_PARAM_LABEL           = "--version or -v"
    m_oedStrPLUGIN_PARAM_LABEL            = "--PLUGIN"

    """
    Derives from EDApplication in order to add a Configuration Feature
    """
    def __init__( self, _strConfigurationFileName = None ):
        """
        """
        EDApplication.__init__( self, EDApplicationPrototype.m_oedStrAPPLICATION_NAME, \
                                EDApplicationPrototype.m_oedStrPROTOTYPE_VERSION, \
                                _strConfigurationFileName )
        self.m_oedStringPluginReadImageHeaderName         = EDString( "EDPluginReadImageHeaderv01" )
        self.m_oedPluginReadImageHeader                   = None
        self.m_oedStringPluginControlCharacterisationName = EDString( "EDPluginControlCharacterisationv01" )
        self.m_oedPluginControlCharacterisation           = None
        self.m_oedListImagePaths                          = None
        self.m_oedStrDatasetFileName                      = None
        self.m_oedStringApplicationWorkingDirectory       = None
        self.m_fFlux                                      = None
        self.m_fMaxExposureTimePerDataCollection          = 10000 # s, default prototype value
        self.m_fMinExposureTimePerImage                   = None
        self.m_fBeamSize                                  = None
    
    
    def preProcess( self ):
        """
        Processes the command line, creates the plugins
        """
        EDApplication.preProcess( self )
        EDVerbose.DEBUG( "*** EDApplicationPrototype.preProcess" )
        
        self.readAndProcessCommandLine()

        self.createApplicationWorkingDirectory()

        self.m_oedPluginReadImageHeader = EDApplicationPrototype.getPluginObject( self.m_oedStringPluginReadImageHeaderName )
        if( self.m_oedPluginReadImageHeader == None ):            
            errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ( "EDApplicationPrototype.preProcess", self.m_oedStringPluginReadImageHeaderName )
            EDVerbose.error( errorMessage )          
        
        self.m_oedPluginControlCharacterisation = EDApplicationPrototype.getPluginObject( self.m_oedStringPluginControlCharacterisationName )
        if( self.m_oedPluginControlCharacterisation == None ):            
            errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ( "EDApplicationPrototype.preProcess", self.m_oedStringPluginControlCharacterisationName )
            EDVerbose.error( errorMessage )       
            
            
    def process( self ):
        """
        If necessary, first read the image headers. 
        Then the characterisation plugin is executed.
        """
        EDApplication.process( self )
        EDVerbose.DEBUG( "*** EDApplicationPrototype.process" )
        # Stop the application if one of the plugins didn't load
        if ( self.m_oedPluginReadImageHeader != None and self.m_oedPluginControlCharacterisation != None ):       
        # Create data input for characterisation plugin
            xsDataCollection = None
            # Check if XML data is give ans input :
            if ( self.m_oedStrDatasetFileName is not None ):
                try:
                    xsDataCollection = XSDataCollection.parseFile( self.m_oedStrDatasetFileName )
                except:
                    errorMessage = EDMessage.ERROR_CANNOT_READ_FILE_02 % ( "EDApplicationPrototype.process", self.m_oedStrDatasetFileName )
                    EDVerbose.error( errorMessage )
            elif ( self.m_oedListImagePaths is not None ): 
                xsDataCollection = self.createDataCollectionFromImageHeaders( self.m_oedListImagePaths )

            if ( xsDataCollection is not None ):
                # Run the characterisation plugin
                self.m_oedPluginControlCharacterisation.setDataInput( xsDataCollection )
                self.m_oedPluginControlCharacterisation.setBaseDirectory( self.m_oedStringApplicationWorkingDirectory )
                self.m_oedPluginControlCharacterisation.setBaseName( "Characterisation" )
                self.m_oedPluginControlCharacterisation.connectSUCCESS( self.doCharacterisationSUCCESS )
                self.m_oedPluginControlCharacterisation.connectFAILURE( self.doCharacterisationFAILURE )
                self.m_oedPluginControlCharacterisation.executeSynchronous()
                
                
    def readAndProcessCommandLine( self ):
        """
        Reads and processes the command line
        """
        EDVerbose.DEBUG( "*** EDApplicationPrototype.readAndProcessCommandLine" )
        # Check if m_oedStrDATASET_PARAM_LABEL is given:
        self.m_oedStrDatasetFileName = self.getCommandLineArgument( EDApplicationPrototype.m_oedStrDATASET_PARAM_LABEL )
        if ( self.m_oedStrDatasetFileName is not None ):
            EDVerbose.screen( "Reading input data from : %s" % ( self.m_oedStrDatasetFileName ) )
            EDVerbose.screen( "(Any other command line arguments are neglected.)" )
        else:
            # Special processing necessary for image list:
            EDVerbose.DEBUG( "*** EDApplicationPrototype.readAndProcessCommandLine: Searching for " + EDApplicationPrototype.m_oedStrIMAGE_PARAM_LABEL + " tag." )
            oedStringCommandLine = EDApplicationPrototype.getCommandLine()
            EDVerbose.DEBUG( "*** EDApplicationPrototype.readAndProcessCommandLine: Command line: " + oedStringCommandLine )
            oedListCommandLine = oedStringCommandLine.split()
            # First go through in order to find if there are any images
            bFoundImageToken = False
            for pyStringCommandLineItem in oedListCommandLine:
                oedStringCommandLineItem = EDString( pyStringCommandLineItem )
                if ( bFoundImageToken == False):
                    if ( oedStringCommandLineItem.equals( EDApplicationPrototype.m_oedStrIMAGE_PARAM_LABEL ) ):
                        bFoundImageToken = True
                else:
                    if ( EDString( oedStringCommandLineItem[0:2] ).equals( "--" ) ):
                        bFoundImageToken = False
                    else:
                        EDVerbose.DEBUG( "*** EDApplicationPrototype.readAndProcessCommandLine: Found image path: " + oedStringCommandLineItem )
                        if ( self.m_oedListImagePaths is None ):
                            self.m_oedListImagePaths = EDList()
                        self.m_oedListImagePaths.add( oedStringCommandLineItem )
            # Check that we have at least one image...
            if ( self.m_oedListImagePaths is not None ):
                # Then find any other optional parameters
                oedStrFlux = self.getCommandLineArgument( EDApplicationPrototype.m_oedStrFLUX_PARAM_LABEL )
                if ( oedStrFlux is not None ):
                    self.m_fFlux = EDString( oedStrFlux ).atof()
                    EDVerbose.screen( "Beamline flux set to                         : %7.1f [photons/s]" % ( self.m_fFlux ) )
                oedStrMaxExposureTimePerDataCollection = self.getCommandLineArgument( EDApplicationPrototype.m_oedStrMAX_EXPOSURE_TIME_PARAM_LABEL )
                if ( oedStrMaxExposureTimePerDataCollection is not None ):
                    self.m_fMaxExposureTimePerDataCollection = EDString( oedStrMaxExposureTimePerDataCollection ).atof()
                    EDVerbose.screen( "Max exposure time per data collection set to : %7.1f [s]" % ( self.m_fMaxExposureTimePerDataCollection ) )
                else:
                    EDVerbose.screen( "Max exposure time per data collection set to : %7.1f [s] (default value for the prototype)" % ( self.m_fMaxExposureTimePerDataCollection ) )                    
                oedStrMinExposureTimePerImage = self.getCommandLineArgument( EDApplicationPrototype.m_oedStrMIN_EXPOSURE_TIME_PARAM_LABEL )
                if ( oedStrMinExposureTimePerImage is not None ):
                    self.m_fMinExposureTimePerImage = EDString( oedStrMinExposureTimePerImage ).atof()
                    EDVerbose.screen( "Minimum exposure time per image set to       : %7.3f [s]" % ( self.m_fMinExposureTimePerImage ) )
                oedStrBeamSize = self.getCommandLineArgument( EDApplicationPrototype.m_oedStrBEAM_SIZE_PARAM_LABEL )
                if ( oedStrBeamSize is not None ):
                    self.m_fBeamSize = EDString( oedStrBeamSize ).atof()
                    EDVerbose.screen( "Beam size (beam assumed to be square)        : %7.3f [mm]" % ( self.m_fBeamSize ) )
            else:
                self.usage()
            
            
    def usage( self ):
        """
        Print usage...
        """
        EDVerbose.screen( "" )
        EDVerbose.screen( "Usage: " )
        EDVerbose.screen( "----- " )
        EDVerbose.screen( "" )
        EDVerbose.screen( " Either use:" )
        EDVerbose.screen( "" )
        EDVerbose.screen( " - the full/advanced XML input:" )
        EDVerbose.screen( "" )
        EDVerbose.screen( "   edna-prototype --data path_to_xsDataCollection_xml_file" )
        EDVerbose.screen( "" )
        EDVerbose.screen( " - or use the --image keyword (note that all these keywords are ignored if the --data option is used):" )
        EDVerbose.screen( "" )
        EDVerbose.screen( "   edna-prototype --image image1 [image2 image3 ...]" )
        EDVerbose.screen( "" )
        EDVerbose.screen( " In the case of using the --image command line keyword the following arguments are optional:" )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Beam flux [photon/s]" % ( self.m_oedStrFLUX_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Beam size [mm] (beam assumed to be square)" % ( self.m_oedStrBEAM_SIZE_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Max exposure time per data collection [s]" % ( self.m_oedStrMAX_EXPOSURE_TIME_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Min exposure time per image [s]" % ( self.m_oedStrMIN_EXPOSURE_TIME_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "-----------------------------------------------------------------------------------------------------------" )
        EDVerbose.screen( "" )
        EDVerbose.screen( " Additional options available:" )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : DEBUG log traces" % ( self.m_oedStrDEBUG_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Executable version info" % ( self.m_oedStrVERSION_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Plugins connected to the application" % ( self.m_oedStrPLUGIN_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        

    def createApplicationWorkingDirectory( self ):
        EDVerbose.DEBUG( "*** EDApplicationPrototype.createApplicationWorkingDirectory" )
        oedStringBaseDirectory = PyOs.environ.get("EDNA_BASE_DIRECTORY")
        if ( oedStringBaseDirectory is None):
            EDVerbose.DEBUG( "*** EDApplicationPrototype.createApplicationWorkingDirectory: Using current base directory as working directory." )
            oedStringBaseDirectory = EDDiskExplorer.getCwd()
        oedStringDateTime = EDTime.getTimeName()
        self.m_oedStringApplicationWorkingDirectory = EDDiskExplorer.mergePath( oedStringBaseDirectory, 
                                                                         "%s_%s-v%s" % \
                                                                         ( oedStringDateTime, \
                                                                           EDApplicationPrototype.m_oedStrAPPLICATION_NAME, \
                                                                           EDApplicationPrototype.m_oedStrPROTOTYPE_VERSION ) )
        # Check that a folder / file with the same name already exists
        if( EDDiskExplorer.existFolder( self.m_oedStringApplicationWorkingDirectory ) or \
            EDDiskExplorer.existFile( self.m_oedStringApplicationWorkingDirectory ) ):
            # It does exist so we have to modify the name of the working directory
            iIndex = 1
            bContinueFlag = True
            while ( bContinueFlag ):
                self.m_oedStringApplicationWorkingDirectory = EDDiskExplorer.mergePath( oedStringBaseDirectory, 
                                                                                        "%s_%s-v%s_%d" % \
                                                                                        ( oedStringDateTime, \
                                                                                          EDApplicationPrototype.m_oedStrAPPLICATION_NAME, \
                                                                                          EDApplicationPrototype.m_oedStrPROTOTYPE_VERSION, \
                                                                                          iIndex ) )
                if( EDDiskExplorer.existFolder( self.m_oedStringApplicationWorkingDirectory ) or \
                    EDDiskExplorer.existFile( self.m_oedStringApplicationWorkingDirectory ) ):
                    iIndex += 1
                else:
                    bContinueFlag = False
        # Make the directory
        EDDiskExplorer.createFolder( self.m_oedStringApplicationWorkingDirectory )
        

    def doCharacterisationSUCCESS( self, _oedPlugin ):
        EDVerbose.DEBUG( "*** EDApplicationPrototype.doCharacterisationSUCCESS" )
        EDVerbose.DEBUG( "*** Checking error messages...")
        if ( _oedPlugin.getErrorMessages().getNumberObjects() != 0 ):
            self.doCharacterisationFAILURE( _oedPlugin )
        elif ( _oedPlugin.getWarningMessages().getNumberObjects() ):
            EDVerbose.screen( "Characterisation successful with warning messages, please check the log file." )
        else:
            EDVerbose.screen( "Characterisation successful!" )
        _oedPlugin.verboseScreenExecutiveSummary()
              
                    
    def doCharacterisationFAILURE( self, _oedObject ):
        EDVerbose.screen( "Characterisation failure!"  )
                    
                    
    def createDataCollectionFromImageHeaders( self, _oedListImagePaths ):    
        """
        """          
        EDVerbose.DEBUG( "*** EDApplicationPrototype.createDataCollectionFromImageHeaders" )
        xsDataCollection = None
        if ( len( _oedListImagePaths ) > 0 ):
            xsDataCollection = XSDataCollection()
            iIndex = 1
            for pyStrImagePath in self.m_oedListImagePaths:
                self.m_oedPluginReadImageHeader = EDApplicationPrototype.getPluginObject( self.m_oedStringPluginReadImageHeaderName )
                if( self.m_oedPluginReadImageHeader == None ):            
                    errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ( "EDApplicationPrototype.createDataCollectionFromImageHeaders", self.m_oedStringPluginReadImageHeaderName )
                    EDVerbose.error( errorMessage )      
                xsDataFileImagePath = XSDataFile()
                xsDataFileImagePath.setPath( XSDataString( pyStrImagePath ) ) 
                self.m_oedPluginReadImageHeader.setDataInput( xsDataFileImagePath )
                self.m_oedPluginReadImageHeader.setBaseDirectory( self.m_oedStringApplicationWorkingDirectory )
                self.m_oedPluginReadImageHeader.setBaseName( "ReadImageHeader_%d" % ( iIndex ) )
                self.m_oedPluginReadImageHeader.executeSynchronous()
                xsDataSubWedge = self.m_oedPluginReadImageHeader.getDataOutput()
                if ( xsDataSubWedge is None ):
                    EDVerbose.warning( "Could not read image header from file " + pyStrImagePath )
                else:
                    # Default exposure time (for the moment, this value should be
                    # possible to read from the command line)
                    xsDataDiffractionPlan = XSDataDiffractionPlan()
                    xsDataDiffractionPlan.setMaxExposureTimePerDataCollection( XSDataTime( self.m_fMaxExposureTimePerDataCollection ) )
                    if ( self.m_fFlux is not None ):
                        xsDataSubWedge.getExperimentalCondition().getBeam().setFlux( XSDataFloat( self.m_fFlux ) )
                    if ( self.m_fBeamSize is not None ):
                        xsDataSize = XSDataSize()
                        xsDataSize.setX( XSDataLength( self.m_fBeamSize ) )
                        xsDataSize.setY( XSDataLength( self.m_fBeamSize ) )
                        xsDataSubWedge.getExperimentalCondition().getBeam().setSize( xsDataSize )
                    if ( self.m_fMinExposureTimePerImage is not None ):
                        xsDataSubWedge.getExperimentalCondition().getBeam().setMinExposureTimePerImage( XSDataFloat( self.m_fMinExposureTimePerImage ) )
                    xsDataCollection.addSubWedge( xsDataSubWedge )
                    xsDataCollection.setDiffractionPlan( xsDataDiffractionPlan )
                iIndex += 1
            # Check that at least one image could be read :
            xsDataSubWedgeList = xsDataCollection.getSubWedge()
            if ( len( xsDataSubWedgeList ) == 0 ):
                EDVerbose.error( "No image headers could be read." )
                xsDataCollection = None
        return xsDataCollection



if __name__ == '__main__':
     
    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edApplicationPrototype = EDApplicationPrototype()
    edApplicationPrototype.execute()
    
    