#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDConfiguration.py 474 2008-07-30 15:51:54Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport          import EDObject
from EDImport          import EDString
from EDImport          import EDVerbose
from EDImport          import EDDiskExplorer
from EDImport          import EDXsdDataBinding


class EDConfiguration( EDObject ):
    """
    This class should be seen as a utilitary class to retrieve param value in a 
    straightforward way. Ideally, subclassing the generated classes from the xml/xsd 
    should be the right way to go (EDConfiguration form XSConfiguration, EDPluginItem
    from XSPluginItem...) but the generated classes a not pure, they are modules that
    contain utilitary methods also (i.e XsdDataBinding uses a method module.unmarshalling
    that is a module method and not a class method)... so subclassing is not possible.
    """
    
    def __init__( self, _xmlFileName, _xsdFileName = None, _generate = False ):
        """
        The Constructor initializes the files (xsd and xml) objects
        The Configuration will be fully constructed after the load method
        """
        # Configuration File
        self.m_oedStrXmlFileName = EDString( EDDiskExplorer.absPath( _xmlFileName ) )
        # XSD file
        self.m_oedStrXsdFileName = EDString( EDDiskExplorer.absPath( _xsdFileName ) )
        self.m_bGenerate = _generate
        # Object generated automatically with xsd/xml mechanism
        self.m_oxsConfiguration = None
        self.m_bLoaded = False
        
        
    def load( self ):
        """
        Loads a configuration File either:
        - in a dynamic way (generation of the python module)
        - or a static way (importing the python Module)
        """
        
        if( self.m_bGenerate == True ):
            self.m_oedXsdDataBindingConf = EDXsdDataBinding( self.m_oedStrXsdFileName )
            self.m_oxsConfiguration = self.m_oedXsdDataBindingConf.getObject( self.m_oedStrXmlFileName )
        else:
            EDVerbose.DEBUG("*** EDConfiguration.load: importing XSConfiguration module...")
            import XSConfiguration
            self.m_oxsConfiguration = XSConfiguration.unMarshalling( self.m_oedStrXmlFileName )
            EDVerbose.DEBUG("*** EDConfiguration.load: XSConfiguration module imported")
            
        self.m_bLoaded = True
        
        
    def isLoaded( self ):
        """
        Returns true if the configuration file has been loaded
        """
        return self.m_bLoaded
        
        
    def getXsdFileName( self ):
        """
        Returns the absolute path of the XSD file
        """
        return self.m_oedStrXsdFileName
    
    
    def getXmlFileName( self ):
        """
        Returns the absolute path of the XML file
        """
        return self.m_oedStrXmlFileName
        
        
    def getXSConfiguration( self ):
        """
        Gets the auto-generated XSConfiguration module
        """
        return self.m_oxsConfiguration
    
    
    def getPluginList( self ):
        """
        Returns an XSPluginList object that encapsulates a list of XSPluginItem
        """
        pluginList = self.m_oxsConfiguration.getXSPluginList()
        
        return pluginList
    

    def getPluginItem( self, _oedStrPluginName):
        """
        Returns a particular XSPluginItem given its name
        """
        oxsPluginList = self.m_oxsConfiguration.getXSPluginList()
        oxsPluginItems = oxsPluginList.getXSPluginItem()
        pluginItem = None
        
        for oxsPluginItem in oxsPluginItems:
            if ( oxsPluginItem.getName() == _oedStrPluginName ):
               pluginItem = oxsPluginItem
               
        return pluginItem
    
    
    def getOptionItem( _oxsPluginItem, _strOptionName ):
        """
        Returns a XSOptionItem given an option name
        """
        oxsOptionList = _oxsPluginItem.getXSOptionList()
        oxsOptionItems = oxsOptionList.getXSOptionItem()
        optionItem = None
        
        for oxsOptionItem in oxsOptionItems:
            if ( oxsOptionItem.getName() == _strOptionName ):
                optionItem = oxsOptionItem
                
        return optionItem
    getOptionItem = staticmethod( getOptionItem )
    
    
    def getParamItem( _oxsPluginItem, _strParamName ):
        """
        Returns a XSParamItem given a param name
        """
        oxsParamList = _oxsPluginItem.getXSParamList()
        oxsParamItems = oxsParamList.getXSParamItem()
        paramItem = None
        
        for oxsParamItem in oxsParamItems:
            if ( oxsParamItem.getName() == _strParamName ):
                paramItem = oxsParamItem
                
        return paramItem
    getParamItem = staticmethod( getParamItem )
    
    
    def getStringParamValue( _oxsPluginItem, _strParamName ):
        """
        Returns the parameter value in a string format
        """
        oxsParamList = _oxsPluginItem.getXSParamList()
        paramValue = None
        
        if ( oxsParamList != None ):
            oxsParamItems = oxsParamList.getXSParamItem()
        
            for oxsParamItem in oxsParamItems:
                if ( oxsParamItem.getName() == _strParamName ):
                    paramValue = oxsParamItem.getValue()
                
        return paramValue
    getStringParamValue = staticmethod( getStringParamValue )

    
    def getIntegerParamValue( _oxsPluginItem, _strParamName ):
        """
        Returns the parameter value in a integer format
        """
        strParamValue = EDConfiguration.getStringParamValue( _oxsPluginItem, _strParamName )
        intParamValue = int( strParamValue )
                
        return intParamValue
    getIntegerParamValue = staticmethod( getIntegerParamValue )
            
    
    def getPluginListSize( self ):
        """
        Returns the size of the XSPluginItem list
        """
        size = len( self.getPluginList().getXSPluginItem() )
        return size
            
        
        