#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPlugin.py 474 2008-07-30 15:51:54Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from ALPlugin import ALPlugin

from EDImport import EDInformation
from EDImport import EDVerbose
from EDImport import EDString
from EDImport import EDApplication
from EDImport import EDSlot
from EDImport import EDList
from EDImport import EDTime
from EDImport import EDRandom
from EDImport import EDConfiguration
from EDImport import EDDiskExplorer
from EDImport import EDMessage

from EDUtilsFile import EDUtilsFile

import os as PyOS

from XSConfiguration import XSPluginItem


class EDPlugin( ALPlugin ):
    """
    A Configurable Plugin Class
    """
    
    def __init__ ( self ):
        """
        an XSPluginItem is a the piece of configuration of a particular plugin
        """
        ALPlugin.__init__( self )
        oedInformation = EDInformation( "EDNA Developers Team", 
                                        "0.1",
                                        "Developed with the AALib Framework", 
                                        "http://www.edna-site.org",
                                        "Under LGPL License" )
        oedInformation.setGroupName( self.getGroup() )
        oedInformation.setFileName( self.getSource() )
        self.setInformation( oedInformation )
        self.m_oxsPluginItem = None
        self.m_oxsDataInput = None
        self.m_oxsDataStrXmlInput = None
        self.m_oxsDataOutput = None
        self.m_oedSlotExportDataOutput = EDSlot()
        self.m_oedStrExecutionContext = "EDApplication"
        
        self.m_oedListErrorMessages = EDList()
        self.m_oedListWarningMessages = EDList()
        
        self.m_oedStrCONF_BASE_DIR_LABEL       = "baseDirectory"
        self.m_oedStringBaseDirectory          = None

        self.m_oedStrCONF_WORKING_DIR_LABEL    = "workingDirectory"
        self.m_oedStringWorkingDirectory       = None

        self.m_oedStringBaseName = None
        
        self.m_oedListExecutiveSummaryLines = EDList()
        self.m_oedStringExecutiveSummarySeparator = "-----------------------------------------------------------------------------------------------------------------"
        
        
    def preProcess( self, _oalObject = None ):
        """
        Derived Plugin should override this method to instanciate the m_oxsDataInput
        """
        EDVerbose.DEBUG( "*** EDPlugin.preProcess")
        self.connectPostProcess( self.exportDataOutput )
        self.connectPreProcess( self.writeDataInput )
        self.connectPostProcess( self.writeDataOutput )
        self.connectPostProcess( self.generateExecutiveSummary )
        
        if( self.m_oedStrExecutionContext == "EDApplication"):
            pluginConfiguration = EDApplication.getPluginConfiguration( self.getClassName() )
            if ( pluginConfiguration == None ):
                EDVerbose.warning( "*** EDPlugin.preProcess: pluginConfiguration is None for "+ self.getClassName())            
            self.setConfiguration( pluginConfiguration )
            
        if ( self.m_oedStringBaseName == None ):
            self.setBaseName( self.createBaseName() )

        self.configure()         
        self.checkParameters()
    
        
        
    def process( self, _oalObject = None ):
        """
        """
        EDVerbose.DEBUG( "*** EDPlugin.process")
        
    
    def postProcess( self, _oalObject = None ):
        """
        """
        EDVerbose.DEBUG( "*** EDPlugin.postProcess")
        # Output Individual Log action File - not fully tested yet
        #EDVerbose.outputActionLog( self.getID(), EDDiskExplorer.mergePath( self.getWorkingDirectory(), self.getBaseName()+"_plugin.log" ) )
       
    
    def setConfiguration( self, _xsPluginItem ):
        """
        Receives an XSPluginItem (Plugin Configuration) from the application
        """
        EDVerbose.DEBUG( "*** EDPlugin.setConfiguration")
        self.m_oxsPluginItem = _xsPluginItem
        
        
    def getConfiguration( self ):
        """
        Gets the Plugin Configuration as an XSPluginItem
        """
        EDVerbose.DEBUG( "*** EDPlugin.getConfiguration")
        return self.m_oxsPluginItem
    
    
    def configure( self ):
        """
        Should be overrided by the Final Plugin If needed
        This method should set its proper members attributes from a Plugin configuration Object
        """
        EDVerbose.DEBUG( "*** EDPlugin.configure")
        oxsPluginItem = self.getConfiguration()
        if ( oxsPluginItem is None ):
            EDVerbose.DEBUG( "*** EDPlugin.configure: No plugin item defined." )
            oxsPluginItem = XSPluginItem()
        # Base directory
        oedStringBaseDirectory = self.getBaseDirectory()
        if ( oedStringBaseDirectory is None):
            # Try to get base directory from plugin configuration
            oedStringBaseDirectory = EDConfiguration.getStringParamValue( oxsPluginItem, self.m_oedStrCONF_BASE_DIR_LABEL )
            if( oedStringBaseDirectory is None ):
                # Try to get working directory from environment variable
                oedStringBaseDirectory = PyOS.environ.get("EDNA_BASE_DIRECTORY")
                if ( oedStringBaseDirectory is None):
                    EDVerbose.DEBUG( "*** EDPlugin.configure: Using current base directory as working directory." )
                    oedStringBaseDirectory = EDDiskExplorer.getCwd()
                else:
                    EDVerbose.DEBUG( "*** EDPlugin.configure: Setting base directory from $EDNA_WORKING_DIRECTORY." )
            else:
                if ( EDString( oedStringBaseDirectory ).equals( "." ) ):
                    EDVerbose.DEBUG( "*** EDPlugin.configure: Using current base directory as working directory." )
                    oedStringBaseDirectory = EDDiskExplorer.getCwd()
                else:
                    oedStringBaseDirectory = EDDiskExplorer.getAbsolutePath( oedStringBaseDirectory )
                    EDVerbose.DEBUG( "*** EDPlugin.configure: Setting base directory from plugin configuration." )
            self.setBaseDirectory( oedStringBaseDirectory )
        else:
            EDVerbose.DEBUG( "*** EDPlugin.configure: Base directory already set before plugin is configured." )          
        # Working directory
        oedStringWorkingDirectory = self.getWorkingDirectory()
        if ( oedStringWorkingDirectory is None):
            # Try to get working directory from plugin configuration
            oedStringWorkingDirectory = EDConfiguration.getStringParamValue( oxsPluginItem, self.m_oedStrCONF_WORKING_DIR_LABEL )
            if( oedStringWorkingDirectory is not None ):
                EDVerbose.DEBUG( "*** EDPlugin.configure: Setting working directory from plugin configuration." )
            else:    
                EDVerbose.DEBUG( "*** EDPlugin.configure: Setting working directory as base directory + base name." )
                oedStringWorkingDirectory = EDDiskExplorer.mergePath( self.getBaseDirectory(), self.getBaseName() )
            self.setWorkingDirectory( oedStringWorkingDirectory )
        else:
            EDVerbose.DEBUG( "*** EDPlugin.configure: Working directory already set before plugin is configured." )          
        
        
    def checkParameters( self ):
        """
        Should be overrided by the Final Plugin If needed
        This method should check that the data input are consistent
        """
        EDVerbose.DEBUG( "*** EDPlugin.checkParameters")
    

    def setDataInput( self, _dataInput ):
        """
        Sets the Plugin Input Data
        _dataInput could be either an String XML or an XSData
        The XML String should be parsed as an Output Object by the final Plugin
        """
        EDVerbose.DEBUG( "*** EDPlugin.setDataInput" )
        raise "Method Should be Implemented by the Final Plugin"
    
        
    def getDataInput( self ):
        """
        Gets the Plugin Input Data
        """
        EDVerbose.DEBUG( "*** EDPlugin.getDataInput")
        return self.m_oxsDataInput
    
    
    def setDataOutput( self, _oxsDataOutput ):
        """
        Sets the Plugin Output Data
        """
        EDVerbose.DEBUG( "*** EDPlugin.setDataOutput: " + EDString ( _oxsDataOutput ))
        self.m_oxsDataOutput = _oxsDataOutput
        
        
    def getDataOutput( self ):
        """
        Gets the Plugin Output Data
        """
        EDVerbose.DEBUG( "*** EDPlugin.getDataOutput")
        return self.m_oxsDataOutput

    
    def exportDataOutput( self, _oObject = None ):
        """
        Exports the Plugin Output Data to slot
        """
        EDVerbose.DEBUG( "*** EDPlugin.exportDataOutput")
        self.m_oedSlotExportDataOutput.call( self.m_oxsDataOutput )


    def connectExportDataOutput( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oedSlotExportDataOutput.connect( _oMethod )
        self.synchronizeOff()
        
        
    def generateExecutiveSummary( self, _oedPlugin ):
        """
        This method should be overridden by plugins who create an executive summary.
        """
        EDVerbose.DEBUG( "*** EDPlugin.generateExecutiveSummary" )

    
    def setExecutionContext(self, _oedStrExecutionContext):
        """
        """
        self.m_oedStrExecutionContext = _oedStrExecutionContext


    def addErrorMessage( self,  _oedStrErrorMessage ):
        """
        """
        self.m_oedListErrorMessages.add( EDString( _oedStrErrorMessage ) )
        
        
    def getErrorMessages( self ):
        """
        """
        return self.m_oedListErrorMessages
    
    
    def addWarningMessage( self, _oedStrWarningMessage ):
        """
        """
        EDVerbose.DEBUG( "*** EDPlugin.addWarningMessage : " + _oedStrWarningMessage )
        self.m_oedListWarningMessages.add( EDString( _oedStrWarningMessage ) )
        
        
    def getWarningMessages( self ):
        """
        """
        return self.m_oedListWarningMessages
    
    
    def writeDataInput( self, _oedObject ):
        EDVerbose.DEBUG( "*** EDPlugin.writeDataInput")
        oxsDataInput = self.getDataInput()
        if ( oxsDataInput is not None ):
            oedStringPathDataInput = EDDiskExplorer.mergePath( self.getWorkingDirectory(), self.getPluginName() + "_dataInput.xml" )
            EDUtilsFile.writeFile( oedStringPathDataInput,  oxsDataInput.marshal() )
        
        
    def writeDataOutput( self, _oedObject ):
        EDVerbose.DEBUG( "*** EDPlugin.writeDataOutput")
        oxsDataOutput = self.getDataOutput()
        if ( oxsDataOutput is not None ):
            oedStringPathDataOutput = EDDiskExplorer.mergePath( self.getWorkingDirectory(), self.getPluginName() + "_dataOutput.xml" )
            EDUtilsFile.writeFile( oedStringPathDataOutput,  oxsDataOutput.marshal() )
            
            
    def getBaseName( self ):
        """
        """
        if ( self.m_oedStringBaseName is None ):
            self.m_oedStringBaseName = self.createBaseName()
        return EDString( self.m_oedStringBaseName )
    
    
    def setBaseName( self, _oedStringBaseName ):
        """
        """
        self.synchronizeOn()
        self.m_oedStringBaseName  = EDString( _oedStringBaseName )
        self.setName( self.m_oedStringBaseName )
        self.synchronizeOff()
            
            
    def createBaseName( self ):
        # Create the base name from the class name + date
#        oedStringBaseName  = self.getClassName()
#        oedStringBaseName += "-" + EDTime.getTimeName()
#        oedStringBaseName += "-%04i" % EDRandom.randInt( 0,9999 )
        oedStringBaseName  = EDTime.getTimeName()
        oedStringBaseName += "-%04i" % EDRandom.randInt( 0,9999 )
        oedStringBaseName += "-%s" % self.getClassName()
        return oedStringBaseName
        

    def setBaseDirectory( self, _oedStringBaseDirectory ):
        """
        """
        EDVerbose.DEBUG( "*** EDPlugin.setBaseDirectory : " + _oedStringBaseDirectory )
        #self.synchronizeOn()
        self.m_oedStringBaseDirectory = EDString( _oedStringBaseDirectory )
        #self.synchronizeOff()
        

    def getBaseDirectory( self ):
        """
        """
        EDVerbose.DEBUG( "*** EDPlugin.getBaseDirectory : %s" % self.m_oedStringBaseDirectory)
        returnValue = None
        if ( self.m_oedStringBaseDirectory is not None ):
            returnValue = EDString( self.m_oedStringBaseDirectory )
        return returnValue


    def setWorkingDirectory( self, _oedStringWorkingDirectory ):
        """
        """
        EDVerbose.DEBUG( "*** EDPlugin.setWorkingDirectory : " + _oedStringWorkingDirectory )
        #self.synchronizeOn()
        self.m_oedStringWorkingDirectory = EDString( _oedStringWorkingDirectory )
        EDDiskExplorer.createFolder( self.m_oedStringWorkingDirectory )
        #self.synchronizeOff()
        

    def getWorkingDirectory( self ):
        """
        """
        EDVerbose.DEBUG( "*** EDPlugin.getWorkingDirectory : %s" % self.m_oedStringWorkingDirectory )
        returnValue = None
        if ( self.m_oedStringWorkingDirectory is not None ):
            returnValue = EDString( self.m_oedStringWorkingDirectory )
        return returnValue
    
    
    def checkMandatoryParameters( self, _xsData, _strParamName ):
        """
        """
        if( _xsData == None ):
            errorMessage = EDMessage.ERROR_MANDATORY_PARAM_MISSING_02 % ( self.getPluginName() + '.checkMandatoryParameters', _strParamName )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )    
            raise RuntimeError, errorMessage 
        
        
    def checkImportantParameters( self, _xsData, _strParamName ):
        """
        """
        if( _xsData == None ):
            warningMessage = EDMessage.WARNING_IMPORTANT_PARAM_MISSING_02 % ( self.getPluginName() + '.checkImportantParameters', _strParamName )
            EDVerbose.warning( warningMessage )
            self.addWarningMessage( warningMessage )  
    
    def addExecutiveSummaryLine( self, _oedStringExecutiveSummaryLine ):
        """
        Add a line to the executive summary string.
        """
        EDVerbose.DEBUG( "*** EDPlugin.addExecutiveSummaryLine : %r" % _oedStringExecutiveSummaryLine )
        oedStringExecutiveSummaryLine = _oedStringExecutiveSummaryLine
        if ( not EDString( oedStringExecutiveSummaryLine ).equals( EDString.empty() ) ):
             if ( EDString( oedStringExecutiveSummaryLine[-1] ).equals( "\n" ) ):
                 oedStringExecutiveSummaryLine = oedStringExecutiveSummaryLine[:-1]
        self.m_oedListExecutiveSummaryLines.add( EDString( oedStringExecutiveSummaryLine ) )


    def addExecutiveSummarySeparator( self, _oedStringSeparator=None ):
        oedStringSeparator = _oedStringSeparator
        if ( oedStringSeparator is None ):
            oedStringSeparator = self.m_oedStringExecutiveSummarySeparator
        # Check that the last line doesn't already contain a separator
        if ( not EDString( self.m_oedListExecutiveSummaryLines[-1] ).equals( oedStringSeparator ) ):
            self.addExecutiveSummaryLine( oedStringSeparator )
            
        
    def getListExecutiveSummaryLines( self ):
        return EDList( self.m_oedListExecutiveSummaryLines ) 


    def verboseScreenExecutiveSummary( self ):
        for line in self.getListExecutiveSummaryLines():
            EDVerbose.screen( line )
