#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginExecProcessScript.py 488 2008-08-01 15:22:51Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImport import EDPluginExecProcess
from EDImport import EDVerbose
from EDImport import EDString
from EDImport import EDList
from EDImport import EDStreamFile
from EDImport import EDDiskExplorer
from EDImport import EDConfiguration 
from EDImport import EDTime
from EDImport import EDRandom
from EDImport import EDTime
from EDImport import EDActionExecuteSystemCommand
from EDImport import EDMessage
from EDUtilsFile import EDUtilsFile
from XSConfiguration import XSPluginItem

import os as PyOs
import stat as PyStat


class EDPluginExecProcessScript( EDPluginExecProcess ):
    """
    The super class for all EDNA pluigns who executes a task using a script.
    """
    
    def __init__ ( self ):
        """
        Constructor of EDPluginExecProcessScript
        """
        EDPluginExecProcess.__init__( self )
        self.m_oedStrCONF_EXEC_PROCESS_SCRIPT_SHELL           = "execProcessScriptShell"
        self.m_oedStringConfigShell                           = EDString( "/bin/bash" )
        self.m_oedStrCONF_EXEC_PROCESS_SCRIPT_EXECUTABLE      = "execProcessScriptExecutable"
        self.m_oedStringScriptExecutable                      = EDString( "cat" )
        self.m_oedStrCONF_EXEC_PROCESS_SCRIPT_SETUP_CCP4      = "execProcessScriptSetupCCP4"
        self.m_oedStringConfigSetupCCP4                       = None
        self.m_oedStrCONF_EXEC_PROCESS_SCRIPT_VERSION_STRING  = "execProcessScriptVersionString"
        self.m_oedStringVersion                               = None
        self.m_oedStringScriptCommandline          = EDString()
        self.m_oedStringScriptBaseName             = None
        self.m_oedStringScriptLogFileName          = None
        self.m_oedStringScriptErrorLogFileName     = None
        self.m_oedStringScriptFileName             = None
        self.m_oxsDataInput                        = None
        self.m_oxsDataOuput                        = None
        self.m_bRequireCCP4                        = False
        self.m_oedListCommandPreExecution          = EDList()
        self.m_oedListCommandExecution             = EDList()
        self.m_oedListCommandPostExecution         = EDList()
        self.m_iPollScriptProcessTime              = 1 # [s]
        
    def preProcess( self, _oalObject = None ):
        """
        """
        EDPluginExecProcess.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.preProcess")
        # The generateScript method will be called at the end of the preProcess method
        self.connectPreProcess( self.generateScript )            
        
        
    def generateScript( self, _oedObject = None ):
        """
        This method prepares the script and writes it to disk.
        """
        oedStringScript = self.prepareScript()
        self.writeExecutableScript( oedStringScript )
            
        
    def process( self, _oalObject = None ):
        """
        This method starts the execution of the EDNA script using EDActionExecuteSystemCommand.
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.process starting" )
        self.synchronizeOn()
        oedStringCommand = EDString( self.m_oedStringConfigExecutable + " " + self.m_oedStringConfigCommandline ) 
        EDVerbose.screen( self.getBaseName() + ": Processing" )
        edActionExecuteSystemCommand = EDActionExecuteSystemCommand( oedStringCommand )
        edActionExecuteSystemCommand.execute()
        edTime = EDTime()
        edTime.initTime()
        bContinue = True
        bTimeOut = False
        while ( bContinue ):
            EDVerbose.DEBUG( "*** EDPluginExecProcess.process executing: " + oedStringCommand )
            EDTime.sleep( self.m_iPollScriptProcessTime )
            if ( edActionExecuteSystemCommand.isEnded() ):
                EDVerbose.DEBUG( "*** EDPluginExecProcess.process - execution ended" )
                bContinue = False
            elif ( edTime.getDuration() > self.m_oedStringConfigTimeOut.atoi() ):
                EDVerbose.DEBUG( "*** EDPluginExecProcess.process - execution time out" )
                bContinue = False
                bTimeOut = True
                edActionExecuteSystemCommand.abort()
        if ( bTimeOut == True ):
            EDVerbose.DEBUG( "*** EDPluginExecProcess.process ========================================= ERROR! ================" )
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginExecProcess.process', self.getClassName(), "execution timed out!" ) 
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage 
        else:
            iErrorCode = edActionExecuteSystemCommand.getErrorCode()
            if ( iErrorCode != 0):
                EDVerbose.DEBUG( "*** EDPluginExecProcess.process ========================================= ERROR! ================" )
                oedStringErrorLog = self.readProcessErrorLogFile()
                if ( oedStringErrorLog is None ):
                    errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginExecProcess.process', self.getClassName(), "Error code : " + EDString( iErrorCode ) ) 
                    EDVerbose.error( errorMessage )
                    self.addErrorMessage( errorMessage )
                else:
                    errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginExecProcess.process', self.getClassName(), EDString( oedStringErrorLog ) ) 
                    EDVerbose.error( errorMessage )
                    self.addErrorMessage( errorMessage )
        EDVerbose.DEBUG( "*** EDPluginExecProcess.process finished " )
        self.synchronizeOff()


    def configure( self ):
        """
        """
        EDPluginExecProcess.configure( self )
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.configure")
        oxsPluginItem = self.getConfiguration()
        if ( oxsPluginItem == None ):
            EDVerbose.warning( "EDPluginExecProcessScript.configure: No plugin item defined." )
            oxsPluginItem = XSPluginItem()
        oedStringShell = EDConfiguration.getStringParamValue( oxsPluginItem, self.m_oedStrCONF_EXEC_PROCESS_SCRIPT_SHELL )
        if( oedStringShell == None ):
            EDVerbose.DEBUG( "EDPluginExecProcessScript.configure: No configuration parameter found for: " + self.m_oedStrCONF_EXEC_PROCESS_SCRIPT_SHELL + ", using default value: " + self.getShell() )
        else:
            self.setShell( oedStringShell )
        oedStringScriptExecutable = EDConfiguration.getStringParamValue( oxsPluginItem, self.m_oedStrCONF_EXEC_PROCESS_SCRIPT_EXECUTABLE )
        if( oedStringScriptExecutable == None ):
            EDVerbose.DEBUG( "EDPluginExecProcessScript.configure: No configuration parameter found for: " + self.m_oedStrCONF_EXEC_PROCESS_SCRIPT_EXECUTABLE + ", using default value: " + self.getScriptExecutable() )
        else:
            self.setScriptExecutable( oedStringScriptExecutable )        
        oedStringConfigSetupCCP4 = EDConfiguration.getStringParamValue( oxsPluginItem, self.m_oedStrCONF_EXEC_PROCESS_SCRIPT_SETUP_CCP4 )
        if( oedStringConfigSetupCCP4 == None ):
            EDVerbose.DEBUG( "EDPluginExecProcessScript.configure: No configuration parameter found for: " + self.m_oedStrCONF_EXEC_PROCESS_SCRIPT_SETUP_CCP4 + ", NO default value!" )
        else:
            self.setSetupCCP4( oedStringConfigSetupCCP4 )        
        oedStringVersion = EDConfiguration.getStringParamValue( oxsPluginItem, self.m_oedStrCONF_EXEC_PROCESS_SCRIPT_VERSION_STRING )
        if( oedStringVersion == None ):
            EDVerbose.DEBUG( "EDPluginExecProcessScript.configure: No configuration parameter found for: " + self.m_oedStrCONF_EXEC_PROCESS_SCRIPT_VERSION_STRING + ", NO default value!" )
        else:
            self.setStringVersion( oedStringVersion )        
        if ( self.m_oedStringScriptBaseName == None ):
            self.setScriptBaseName( self.getBaseName() )
        if ( self.m_oedStringScriptFileName == None ):
            self.setScriptFileName( self.m_oedStringScriptBaseName + ".sh" )
        if ( self.m_oedStringScriptLogFileName == None ):
            self.setScriptLogFileName( self.m_oedStringScriptBaseName + ".log" )
        if ( self.m_oedStringScriptErrorLogFileName == None ):
            self.setScriptErrorLogFileName( self.m_oedStringScriptBaseName + ".err" )

 
    def setRequireCCP4( self, _bReqireCCP4 ):
        """
        """
        self.synchronizeOn()
        self.m_bRequireCCP4 = _bReqireCCP4
        self.synchronizeOff()


    def getRequireCCP4( self ):
        """
        """
        return self.m_bRequireCCP4

        
    def setListCommandExecution( self, _oedListCommandExecution ):
        """
        """
        self.synchronizeOn()
        if ( _oedListCommandExecution is not None ):
            self.m_oedListCommandExecution = EDList( _oedListCommandExecution )
        self.synchronizeOff()
        

    def addListCommandExecution( self, _oedStringCommandExecution ):
        """
        """
        self.synchronizeOn()
        if ( _oedStringCommandExecution is not None ):
            self.m_oedListCommandExecution.add( EDString( _oedStringCommandExecution ) )
        self.synchronizeOff()

    
    def getListCommandExecution( self ):
        """
        """
        self.synchronizeOn()
        oedObject = None
        if ( self.m_oedListCommandExecution is not None ):
            oedObject = EDList( self.m_oedListCommandExecution )
        self.synchronizeOff()
        return oedObject

    
    def setListCommandPreExecution( self, _oedListCommandPreExecution ):
        """
        """
        self.synchronizeOn()
        if ( _oedListCommandPreExecution is not None ):
            self.m_oedListCommandPreExecution = EDList( _oedListCommandPreExecution )
        self.synchronizeOff()
        

    def addListCommandPreExecution( self, _oedStringCommandPreExecution ):
        """
        """
        self.synchronizeOn()
        if ( _oedStringCommandPreExecution is not None ):
            self.m_oedListCommandPreExecution.add( EDString( _oedStringCommandPreExecution ) )
        self.synchronizeOff()

    
    def getListCommandPreExecution( self ):
        """
        """
        self.synchronizeOn()
        oedObject = None
        if ( self.m_oedListCommandPreExecution != None ):
            oedObject = EDList( self.m_oedListCommandPreExecution )
        self.synchronizeOff()
        return oedObject

    
    def setListCommandPostExecution( self, _oedListCommandPostExecution ):
        """
        """
        self.synchronizeOn()
        if ( _oedListCommandPostExecution is not None ):
            self.m_oedListCommandPostExecution = EDList( _oedListCommandPostExecution )
        self.synchronizeOff()
        

    def addListCommandPostExecution( self, _oedStringCommandPostExecution ):
        """
        """
        self.synchronizeOn()
        if ( _oedStringCommandPostExecution is not None ):
            self.m_oedListCommandPostExecution.add( EDString( _oedStringCommandPostExecution ) )
        self.synchronizeOff()

    
    def getListCommandPostExecution( self ):
        """
        """
        self.synchronizeOn()
        oedObject = None
        if ( self.m_oedListCommandPostExecution != None ):
            oedObject = EDList( self.m_oedListCommandPostExecution )
        self.synchronizeOff()
        return oedObject

    
    def setShell( self, _oedStringshell ):
        """
        """
        self.synchronizeOn()
        self.m_oedStringConfigShell = EDString( _oedStringshell )
        self.synchronizeOff()
        

    def getShell( self ):
        """
        """
        return EDString( self.m_oedStringConfigShell )

    
    def setSetupCCP4( self, _oedStringSetupCCP4 ):
        """
        """
        self.synchronizeOn()
        self.m_oedStringConfigSetupCCP4  = EDString( _oedStringSetupCCP4 )
        self.synchronizeOff()
        

    def getSetupCCP4( self ):
        """
        """
        return EDString( self.m_oedStringConfigSetupCCP4 )

    
    def setScriptBaseName( self, _oedStringScriptBaseName ):
        """
        """
        self.synchronizeOn()
        self.m_oedStringScriptBaseName  = EDString( _oedStringScriptBaseName )
        self.synchronizeOff()
        

    def getScriptBaseName( self ):
        """
        """
        if ( self.m_oedStringScriptBaseName is None ):
            self.m_oedStringScriptBaseName = self.createBaseName()
        return EDString( self.m_oedStringScriptBaseName )

    
    def setScriptLogFileName( self, _oedStringScriptLogFileName ):
        """
        """
        self.synchronizeOn()
        self.m_oedStringScriptLogFileName  = EDString( _oedStringScriptLogFileName )
        self.synchronizeOff()
        

    def setScriptErrorLogFileName( self, _oedStringScriptErrorLogFileName ):
        """
        """
        self.synchronizeOn()
        self.m_oedStringScriptErrorLogFileName  = EDString( _oedStringScriptErrorLogFileName )
        self.synchronizeOff()

    
    def getScriptLogFileName( self ):
        """
        """
        return EDString( self.m_oedStringScriptLogFileName )

    
    def getScriptErrorLogFileName( self ):
        """
        """
        return EDString( self.m_oedStringScriptErrorLogFileName )

    
    def setScriptFileName( self, _oedStringScriptFileName ):
        """
        """
        self.synchronizeOn()
        self.m_oedStringScriptFileName  = EDString( _oedStringScriptFileName )
        self.synchronizeOff()
        

    def getScriptFileName( self ):
        """
        """
        return EDString( self.m_oedStringScriptFileName )

    
    def getScriptFilePath( self ):
        """
        """
        return EDString( EDDiskExplorer.mergePath( self.getWorkingDirectory(), self.m_oedStringScriptFileName ) )

    
    def setScriptExecutable( self, _oedStringScriptExecutable ):
        """
        """
        self.synchronizeOn()
        self.m_oedStringScriptExecutable  = EDString( _oedStringScriptExecutable )
        self.synchronizeOff()
        

    def getScriptExecutable( self ):
        """
        """
        return EDString( self.m_oedStringScriptExecutable )

    
    def setScriptCommandline( self, _oedStringScriptCommandline ):
        """
        """
        self.synchronizeOn()
        self.m_oedStringScriptCommandline  = EDString( _oedStringScriptCommandline )
        self.synchronizeOff()
        

    def getScriptCommandline( self ):
        """
        """
        return EDString( self.m_oedStringScriptCommandline )

    
    def setStringVersion( self, _oedStringVersion ):
        """
        """
        self.synchronizeOn()
        self.m_oedStringVersion = EDString( _oedStringVersion )
        self.synchronizeOff()
        

    def getStringVersion( self ):
        """
        """
        return EDString( self.m_oedStringVersion )
    
        
    def prepareScript( self ):
        """
        Generates a string containing the script.
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.generateScript")
        self.synchronizeOn()
        oedStringScript = EDString()
        oedStringScript += EDString( "#!" + self.m_oedStringConfigShell + "\n" )
        oedStringScript += EDString( "cd " + self.getWorkingDirectory() + "\n" )
        if ( self.m_bRequireCCP4 ):
            if ( self.m_oedStringConfigSetupCCP4 == None or self.m_oedStringConfigSetupCCP4 == EDString.empty() ):
                EDVerbose.DEBUG( "*** EDPluginExecProcessScript.generateScript : CCP4 setup script not defined." )
            else:
                oedStringScript += EDString( ". " + self.m_oedStringConfigSetupCCP4 + "\n" )
        # Add pre-execution commands - if any
        for oedStringCommandPreExecution in self.m_oedListCommandPreExecution:
            oedStringScript += EDString( oedStringCommandPreExecution ) + "\n"
        # Execution
        oedStringScript += EDString( self.m_oedStringScriptExecutable + " " + self.m_oedStringScriptCommandline + " > " + self.m_oedStringScriptLogFileName + " 2> " + self.m_oedStringScriptErrorLogFileName )
        # Add execution commands - if any
        if ( len( self.m_oedListCommandExecution ) == 0 ):
            oedStringScript += EDString( "\n" )
        else:
            oedStringScript += EDString( " << EOF-EDPluginExecProcessScript\n" )
            for oedStringCommandExecution in self.m_oedListCommandExecution:
                oedStringScript += EDString( oedStringCommandExecution ) + "\n"
            oedStringScript += EDString( "EOF-EDPluginExecProcessScript\n" )
        # Add post-execution commands - if any
        if ( len( self.m_oedListCommandPostExecution ) > 0 ):
            for oedStringCommandPostExecution in self.m_oedListCommandPostExecution:
                oedStringScript += EDString( oedStringCommandPostExecution ) + "\n"            
        self.synchronizeOff()
        return oedStringScript
    

    def writeExecutableScript( self, _oedStringScript ):
        """
        Writes the script to the script file.
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.writeScriptToFile")
        self.synchronizeOn()
        
        oedStreamFileScript = EDStreamFile( self.getScriptFilePath() )
        oedStreamFileScript.write( _oedStringScript )
        oedStreamFileScript.close()
        #PyOs.chmod( self.m_oedStringScriptFileName, PyStat.S_IRWXU + PyStat.S_IRGRP + PyStat.S_IROTH) # + PyStat.S_IXUSR )
        self.synchronizeOff()
        self.setExecutable( self.getShell() )
        self.setCommandline( self.getScriptFilePath() )
        
        
    def findStringInLog( self, _oedStringInput ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.findStringInLog")
        oedStringOutput = None
        bTestSuccess = False
        if ( EDDiskExplorer.existFile( self.getScriptLogFileName() ) == True ):
            oedStreamFileLog = EDStreamFile( self.getScriptLogFileName(), "read" )
            oedListLine = oedStreamFileLog.getLines()
            oedStreamFileLog.close()    
            for oedStringLine in oedListLine:
                if ( EDString( oedStringLine ).existSubString( _oedStringInput ) == True ):
                    oedStringOutput = oedStringLine
                    break
        return oedStringOutput
 
            
    def testVersion( self ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.testVersion")
        bCorrectVersion = False
        if ( self.findStringInLog( self.m_oedStringVersion ) != None ):
            bCorrectVersion = True
        return bCorrectVersion
    

    def writeProcessFile( self, _oedStringFileName, _oedStringContent ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.writeProcessFile")
        oedStringFilePath = EDDiskExplorer.mergePath( self.getWorkingDirectory(), _oedStringFileName )
        oedStringFileContent = EDUtilsFile.writeFile( oedStringFilePath, _oedStringContent )

    
    def readProcessFile( self, _oedStringFileName ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.readProcessFile")
        oedStringFilePath = EDDiskExplorer.mergePath( self.getWorkingDirectory(), _oedStringFileName )
        oedStringFileContent = None
        if EDDiskExplorer.isFile( oedStringFilePath ):
            oedStringFileContent = EDUtilsFile.readFile( oedStringFilePath )
        return oedStringFileContent

    
    def readProcessLogFile( self ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.readProcessLogFile")
        oedStringLogFileContent = self.readProcessFile( self.getScriptLogFileName() )
        return oedStringLogFileContent
        

    def readProcessErrorLogFile( self ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.readProcessErrorLogFile")
        oedStringErrorLogFileContent = self.readProcessFile( self.getScriptErrorLogFileName() )
        return oedStringErrorLogFileContent
        
